!
! $Id: modbc.F,v 1.2 2003/12/17 13:56:09 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Boundary 
C 
      Module Agrif_Boundary  
C
CCC   Description:
CCC   Module to calculate the boundary conditions on the child grids from their
CCC   parent grids.
C
C     Modules used:  
C   
      Use Agrif_Interpolation        
C
C        
      CONTAINS   
C     Define procedures contained in this module
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_bc_1d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_bc_1d(TypeInterp,parent,child,tab,deb,fin,
     &                              weight,pweight)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions on a fine grid for a 1D 
CCC   grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                 ! Type of interpolation (linear,...)
      Type(AGRIF_PVariable) :: parent       ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child        ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp    ! Temporary variable on the child 
                                            ! grid
      Integer :: deb,fin                    ! Positions of the  interpolations 
      Real, Dimension(lbound(child%var%array1,1):
     &                ubound(child%var%array1,1)), Target :: tab ! Values of the grid variable
      Logical :: pweight                    ! Indicates if weight is used for  
                                            ! the temporal interpolation 
      Real :: weight                        ! Coefficient for the time 
                                            ! interpolation
C
C
C     Definition of a temporary AGRIF_PVariable data type representing the grid
C     variable.  
C
      allocate(childtemp % var)  
C
      childtemp % var % root_var => child % var % root_var
C      
C     Number of dimensions of the grid variable
c      childtemp % var % nbdim = 1
C      
C     Values of the grid variable
      childtemp % var % array1 => tab  
C
C     Temporary results for the time interpolation before and after the space 
C     interpolation 
      childtemp % var % oldvalues2D => child % var % oldvalues2D        
C 
C     Index indicating if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex       
C
C     Call to the procedure for the calculations of the boundary conditions      
      Call Agrif_CorrectVariable
     &     (TypeInterp,parent,childtemp,deb,fin,pweight,weight)              
C
      child % var % oldvalues2D => childtemp % var % oldvalues2D
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_bc_1D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_bc_2d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_bc_2d(TypeInterp,parent,child,tab,deb,fin,
     &                              weight,pweight)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions on a fine grid for a 2D 
CCC   grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                 ! Type of interpolation (linear, 
                                            ! lagrange, spline, ... )
      Type(AGRIF_PVariable) :: parent       ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child        ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp    ! Temporary variable on the child 
                                            ! grid
      Integer :: deb,fin                    ! Positions where interpolations are 
                                            ! done on the fine grid 
      Real, Dimension(lbound(child%var%array2,1):
     &         ubound(child%var%array2,1),lbound(child%var%array2,2):
     &         ubound(child%var%array2,2)), Target :: tab ! Values of the grid variable
      Logical :: pweight                    ! Indicates if weight is used for  
                                            ! the temporal interpolation 
      Real :: weight                        ! Coefficient for the time 
                                            ! interpolation
C
C
C     Definition of a temporary AGRIF_PVariable data type representing the grid
C     variable.   
C
      allocate(childtemp % var)  
C
      childtemp % var % root_var => child % var % root_var
C      
C     Number of dimensions of the grid variable
c      childtemp % var % nbdim = 2
C      
C     Values of the grid variable
      childtemp % var % array2 => tab  
C
C     Temporary results for the time interpolation before and after the space 
C     interpolation 
      childtemp % var % oldvalues2D => child % var % oldvalues2D         
C 
C     Index indicating if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex       
C
C     Call to the procedure for the calculations of the boundary conditions
      Call Agrif_CorrectVariable
     &     (TypeInterp,parent,childtemp,deb,fin,pweight,weight)
C   
      child % var % oldvalues2D => childtemp % var % oldvalues2D
C         
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_bc_2D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_bc_3d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_bc_3d(TypeInterp,parent,child,tab,deb,fin,
     &                              weight,pweight)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions on a fine grid for a 3D 
CCC   variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                 ! Type of interpolation (linear, 
                                            ! lagrange, spline, ... )
      Type(AGRIF_PVariable) :: parent       ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child        ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp    ! Temporary variable on the child
                                            ! grid                                  
      Integer :: deb,fin                    ! Positions where interpolations 
                                            ! are done on the fine grid 
      Real, Dimension(lbound(child%var%array3,1):
     &         ubound(child%var%array3,1),lbound(child%var%array3,2):
     &         ubound(child%var%array3,2),lbound(child%var%array3,3):
     &         ubound(child%var%array3,3)), Target :: tab ! Values of the grid variable
      Logical :: pweight                    ! Indicates if weight is used for 
                                            ! the temporal interpolation 
      Real :: weight                        ! Coefficient for the time 
                                            ! interpolation
C
C
C     Definition of a temporary AGRIF_PVariable data type representing the grid 
C     variable.  
C
      allocate(childtemp % var)  
C
      childtemp % var % root_var => child % var % root_var       
C      
C     Number of dimensions of the grid variable
c      childtemp % var % nbdim = 3
C      
C     Values of the grid variable
      childtemp % var % array3 => tab
C
C     Temporary results for the time interpolation before and after the space 
C     interpolation  
      childtemp % var % oldvalues2D => child % var % oldvalues2D        
C 
C     Index indicating if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex
C
C     Call to the procedure for the calculations of the boundary conditions     
      Call Agrif_CorrectVariable
     &     (TypeInterp,parent,childtemp,deb,fin,pweight,weight)
C
      child % var % oldvalues2D => childtemp % var % oldvalues2D
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_bc_3D               
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_bc_4d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_bc_4d(TypeInterp,parent,child,tab,deb,fin,
     &                             weight,pweight)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions on a fine grid for a 4D 
CCC   grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                   ! Type of interpolation (linear, 
                                              ! lagrange, spline, ... )
      Type(AGRIF_PVariable) :: parent         ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child          ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp      ! Temporary varaiable on the child
                                              ! grid                                    
      Integer :: deb,fin                      ! Positions where interpolations 
                                              ! are done on the fine grid 
      Real, Dimension(lbound(child%var%array4,1):
     &        ubound(child%var%array4,1),lbound(child%var%array4,2):
     &        ubound(child%var%array4,2),lbound(child%var%array4,3):
     &        ubound(child%var%array4,3),lbound(child%var%array4,4):
     &        ubound(child%var%array4,4)), Target :: tab ! Values of the grid variable
      Logical :: pweight                      ! Indicates if weight is used for 
                                              ! the temporal interpolation 
      Real :: weight                          ! Coefficient for the time 
                                              ! interpolation
C
C
C     Definition of a temporary AGRIF_PVariable data type representing the grid 
C     variable.  
C
      allocate(childtemp % var)  
C
      childtemp % var % root_var => child % var % root_var 
C      
C     Number of dimensions of the grid variable
c      childtemp % var % nbdim = 4
C      
C     Values of the grid variable
      childtemp % var % array4 => tab  
C
C     Temporary results for the time interpolation before and after the space 
C     interpolation       
      childtemp % var % oldvalues2D => child % var % oldvalues2D         
C 
C     Index indicating if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex       
C
C     Call to the procedure for the calculations of the boundary conditions      
      Call Agrif_CorrectVariable
     &     (TypeInterp,parent,childtemp,deb,fin,pweight,weight)
C
      child % var % oldvalues2D => childtemp % var % oldvalues2D
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_bc_4D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_bc_5d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_bc_5d(TypeInterp,parent,child,tab,deb,fin,
     &                             weight,pweight)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions on a fine grid for a 5D 
CCC   grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                   ! Type of interpolation (linear, 
                                              ! lagrange, spline, ... )
      Type(AGRIF_PVariable) :: parent         ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child          ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp      ! Temporary varaiable on the child
                                              ! grid                                    
      Integer :: deb,fin                      ! Positions where interpolations 
                                              ! are done on the fine grid 
      Real, Dimension(lbound(child%var%array5,1):
     &         ubound(child%var%array5,1),lbound(child%var%array5,2):
     &         ubound(child%var%array5,2),lbound(child%var%array5,3):
     &         ubound(child%var%array5,3),lbound(child%var%array5,4):
     &         ubound(child%var%array5,4),lbound(child%var%array5,5):
     &         ubound(child%var%array5,5)), Target :: tab ! Values of the grid variable
      Logical :: pweight                      ! Indicates if weight is used for 
                                              ! the temporal interpolation 
      Real :: weight                          ! Coefficient for the time 
                                              ! interpolation
C
C
C     Definition of a temporary AGRIF_PVariable data type representing the grid 
C     variable.  
C
      allocate(childtemp % var)  
C
      childtemp % var % root_var => child % var % root_var 
C      
C     Number of dimensions of the grid variable
c      childtemp % var % nbdim = 5
C      
C     Values of the grid variable
      childtemp % var % array5 => tab  
C
C     Temporary results for the time interpolation before and after the space 
C     interpolation       
      childtemp % var % oldvalues2D => child % var % oldvalues2D         
C 
C     Index indicating if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex       
C
C     Call to the procedure for the calculations of the boundary conditions      
      Call Agrif_CorrectVariable
     &     (TypeInterp,parent,childtemp,deb,fin,pweight,weight)
C
      child % var % oldvalues2D => childtemp % var % oldvalues2D
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_bc_5D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_CorrectVariable
C     **************************************************************************               
C
      Subroutine AGRIF_CorrectVariable(TypeInterp,parent,child,deb,fin,
     &                                 pweight,weight)
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions on a fine grid.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments
      Integer :: TypeInterp                   ! Type of interpolation (linear,lagrange,...)
      Type(AGRIF_PVariable) :: parent         ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child          ! Variable on the child grid
      Integer :: deb,fin                      ! Positions where boundary conditions are calculated
      Logical :: pweight                      ! Indicates if weight is used for the time interpolation 
      Real :: weight                          ! Coefficient for the time interpolation
C
C     Local data type
      Type(AGRIF_Variable), Pointer :: root   ! Variable on the root grid
C
C     Local scalars
      Integer :: nbdim                        ! Number of dimensions of the grid variable 
      Integer :: n
C
C     Local arrays       
      Integer,Dimension(5) :: pttab_child     ! Index of the first point inside the domain for the child grid variable  
      Integer,Dimension(5) :: petab_child     ! Index of the last point inside the domain for the child grid variable   
      Integer,Dimension(5) :: pttab_parent    ! Index of the first point inside the domain for the parent grid variable
      Integer,Dimension(5) :: petab_parent    ! Index of the last point inside the domain for the parent grid variable 
      Integer,Dimension(5) :: nbtab_Child     ! Number of the cells
      Integer,Dimension(5) :: posvartab_Child ! Position of the variable on the cell 
      Integer,Dimension(5) :: loctab_Child    ! Indicates if the child grid has a common border with the root grid      
      Type(Agrif_Grid), Pointer :: Agrif_Child_Gr,Agrif_Parent_Gr
      Real, Dimension(5)   :: s_child,s_parent   ! Positions of the parent and child grids
      Real, Dimension(5)   :: ds_child,ds_parent ! Space steps of the parent and child grids 
C
C
      loctab_child(:) = 0
C
      Agrif_Child_Gr => Agrif_Curgrid
C      
      Agrif_Parent_Gr => Agrif_Curgrid % parent
C
      root => child % var % root_var 
C
      nbdim = root % nbdim
C
      Do n = 1,nbdim
C      
        posvartab_child(n) = root % posvar(n)      
C
      End Do
C
C
      Do n = 1,nbdim
C
        Select case(root % interptab(n))
C
          case('x') ! x dimension
C
            nbtab_Child(n) = Agrif_Child_Gr % nbx
C
            pttab_Child(n) = root % pointX
C            
            pttab_Parent(n) = root % pointx
C        
            s_Child(n) = Agrif_Child_Gr % Agrif_x
C
            s_Parent(n) = Agrif_Parent_Gr % Agrif_x 
C        
            ds_Child(n) = Agrif_Child_Gr % Agrif_dx
C
            ds_Parent(n) = Agrif_Parent_Gr % Agrif_dx
C                      
            If (root % posvar(n) == 2) Then
C                  
                s_Child(n) = s_Child(n) + ds_Child(n)/2.
C
                s_Parent(n) = s_Parent(n) + ds_Parent(n)/2.                                
C        
            End If
C
            If (Agrif_CURGRID % NearRootBorderX) then
C
                loctab_child(n) = -1
C
            End if
C
            If (Agrif_CURGRID % DistantRootBorderX) then                                              
C
                loctab_child(n) = -2
C
            End if           
C
            If ((Agrif_CURGRID % NearRootBorderX) .AND. 
     &          (Agrif_CURGRID % DistantRootBorderX)) then 
C
                loctab_child(n) = -3
C
            End if             
C
          case('y') ! y dimension      
C
            nbtab_Child(n) = Agrif_Child_Gr % nby
C
            pttab_Child(n) = root % pointY
C            
            pttab_Parent(n) = root % pointy
C        
            s_Child(n) = Agrif_Child_Gr % Agrif_y
C
            s_Parent(n) = Agrif_Parent_Gr % Agrif_y 
C        
            ds_Child(n) = Agrif_Child_Gr % Agrif_dy
C
            ds_Parent(n) = Agrif_Parent_Gr % Agrif_dy
C                      
            If (root % posvar(n) == 2) Then        
C        
                s_Child(n) = s_Child(n) + ds_Child(n)/2.
C
                s_Parent(n) = s_Parent(n) + ds_Parent(n)/2.                                
C        
            End If        
C
            If (Agrif_CURGRID % NearRootBorderY) then
C
                loctab_child(n) = -1
C
            End if
C
            If (Agrif_CURGRID % DistantRootBorderY) then                                              
C
                loctab_child(n) = -2
C
            End if
C
            If ((Agrif_CURGRID % NearRootBorderY) .AND. 
     &          (Agrif_CURGRID % DistantRootBorderY)) then 
C
                loctab_child(n) = -3
C
            End if
C
          case('z') ! z dimension
C
            nbtab_Child(n) = Agrif_Child_Gr % nbz
C
            pttab_Child(n) = root % pointZ
C            
            pttab_Parent(n) = root % pointz
C        
            s_Child(n) = Agrif_Child_Gr % Agrif_z
C
            s_Parent(n) = Agrif_Parent_Gr % Agrif_z 
C        
            ds_Child(n) = Agrif_Child_Gr % Agrif_dz
C
            ds_Parent(n) = Agrif_Parent_Gr % Agrif_dz
C                      
            If (root % posvar(n) == 2) Then        
C        
                s_Child(n) = s_Child(n) + ds_Child(n)/2.
C
                s_Parent(n) = s_Parent(n) + ds_Parent(n)/2.                                
C        
            End If        
C
            If (Agrif_CURGRID % NearRootBorderZ) then
C
                loctab_child(n) = -1
C
            End if
C
            If (Agrif_CURGRID % DistantRootBorderZ) then                                              
C
                loctab_child(n) = -2
C
            End if
C
            If ((Agrif_CURGRID % NearRootBorderZ) .AND. 
     &          (Agrif_CURGRID % DistantRootBorderZ)) then 
C
                loctab_child(n) = -3
C
            End if        
C
          case('noint') ! No space dimension      
C
            select case (nbdim) 
C      
              case(1)
C      
                nbtab_Child(n) = SIZE(child % var % array1,n) - 1
                pttab_Child(n) = lbound(child % var % array1,n)
C        
              case(2)
C      
                nbtab_Child(n) = SIZE(child % var % array2,n) - 1
                pttab_Child(n) = lbound(child % var % array2,n)
C      
              case(3)
C      
                nbtab_Child(n) = SIZE(child % var % array3,n) - 1
                pttab_Child(n) = lbound(child % var % array3,n)  
C      
              case(4)
C      
                nbtab_Child(n) = SIZE(child % var % array4,n) - 1
                pttab_Child(n) = lbound(child % var % array4,n)
C
              case(5)
C      
                nbtab_Child(n) = SIZE(child % var % array5,n) - 1
                pttab_Child(n) = lbound(child % var % array5,n)      
C
            end select
C
C           No interpolation but only a copy of the values of the grid variable      
C      
            posvartab_child(n) = 1
C            
            pttab_Parent(n)= pttab_Child(n)
C              
            s_Child(n)=0.
C      
            s_Parent(n)=0. 
C      
            ds_Child(n)=1.
C      
            ds_Parent(n)=1.
C            
            loctab_child(n) = -3
C
        End select
C
      End do
C
C
      If (nbdim == 1) then
C
          Call AGRIF_Correct1D
     &         (TypeInterp,parent,child,deb,fin,pweight,weight,
     &          pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &          nbtab_Child(1:nbdim),posvartab_Child(1:nbdim),
     &          loctab_Child(1:nbdim),
     &          s_Child(1:nbdim),s_Parent(1:nbdim),
     &          ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)
C
        Else if (nbdim == 2) then
C    
          Call AGRIF_Correct2D
     &         (TypeInterp,parent,child,deb,fin,pweight,weight,
     &          pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &          nbtab_Child(1:nbdim),posvartab_Child(1:nbdim),
     &          loctab_Child(1:nbdim),
     &          s_Child(1:nbdim),s_Parent(1:nbdim),
     &          ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)
C
        Else if (nbdim == 3) then
C
          Call AGRIF_Correct3D
     &         (TypeInterp,parent,child,deb,fin,pweight,weight,
     &          pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &          nbtab_Child(1:nbdim),posvartab_Child(1:nbdim),
     &          loctab_Child(1:nbdim),
     &          s_Child(1:nbdim),s_Parent(1:nbdim),
     &          ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)              
C
        Else if (nbdim == 4) then
C
          Call AGRIF_Correct4D
     &         (TypeInterp,parent,child,deb,fin,pweight,weight,
     &          pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &          nbtab_Child(1:nbdim),posvartab_Child(1:nbdim),
     &          loctab_Child(1:nbdim),
     &          s_Child(1:nbdim),s_Parent(1:nbdim),
     &          ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)
C
        Else if (nbdim == 5) then
C
          Call AGRIF_Correct5D
     &         (TypeInterp,parent,child,deb,fin,pweight,weight,
     &          pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &          nbtab_Child(1:nbdim),posvartab_Child(1:nbdim),
     &          loctab_Child(1:nbdim),
     &          s_Child(1:nbdim),s_Parent(1:nbdim),
     &          ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)      
C
      End if
C
C
      End subroutine AGRIF_CorrectVariable
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Correct1d
C     **************************************************************************               
C
      Subroutine AGRIF_Correct1d(TypeInterp,parent,child,deb,fin,
     &                           pweight,weight,
     &                           pttab_child,pttab_Parent,
     &                           nbtab_Child,posvartab_Child,
     &                           loctab_Child,
     &                           s_Child,s_Parent,
     &                           ds_Child,ds_Parent,nbdim)
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions for a 1D grid variable on 
CCC   a fine grid by using a space and time interpolations; it is called by the 
CCC   Agrif_CorrectVariable procedure.
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: TypeInterp                          ! Type of interpolation (linear,spline,...)  
      Type(AGRIF_PVariable) :: parent                ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child                 ! Variable on the child grid 
      Integer :: deb,fin                             ! Positions where interpolations are done
      Logical :: pweight                             ! Indicates if weight is used for the temporal interpolation
      Real :: weight                                 ! Coefficient for the temporal interpolation
      Integer                  :: nbdim              ! Number of dimensions of the grid variable
      Integer,Dimension(nbdim) :: pttab_child        ! Index of the first point inside the domain for the parent grid variable              
      Integer,Dimension(nbdim) :: pttab_Parent       ! Index of the first point inside the domain for the child grid variable
      Integer,Dimension(nbdim) :: nbtab_Child        ! Number of cells of the child grid
      Integer,Dimension(nbdim) :: posvartab_Child    ! Position of the grid variable (1 or 2)
      Integer,Dimension(nbdim) :: loctab_Child       ! Indicates if the child grid has a common border with the root grid
      Real,Dimension(nbdim)    :: s_Child,s_Parent   ! Positions of the parent and child grids 
      Real,Dimension(nbdim)    :: ds_Child,ds_Parent ! Space steps of the parent and child grids
C
C     Local variables
      Integer                  :: lbglob,ubglob
      Integer,Dimension(:),Pointer     :: restore  
      Real    :: c1t,c2t                             ! Coefficients for the time interpolation (c2t=1-c1t) 
      Integer :: i                 
      Integer :: kindex                              ! Index used for safeguard and time interpolation
      Integer,Dimension(nbdim,2,2) :: indtab         ! Arrays indicating the limits of the child     
      Integer,Dimension(nbdim,2,2) :: indtruetab     ! grid variable where boundary conditions are 
      Integer,Dimension(nbdim,2) :: ptres            ! calculated
      Integer :: nb,ndir,n,sizetab
      Real, Dimension(:), Allocatable :: tab         ! Array used for the time interpolation
      Integer nind,ir
C
#ifdef AGRIF_MPI
C
      Integer,Dimension(nbdim) :: ltab,utab
      Integer,Dimension(nbdim) :: ltabtrue,utabtrue
      Integer                  :: lb,ub,iming,imaxg,indloc,indglob
      Integer                  :: code      
C
#endif            
C      
C
      Do i = 1,nbdim
C
        indtab(i,2,1) = pttab_child(i) + nbtab_child(i) + deb
C        
        indtab(i,2,2) = pttab_child(i) + nbtab_child(i) + fin
C 
        If (posvartab_child(i) .EQ. 1) then
C
            indtab(i,1,1) = pttab_child(i) - fin
C        
            indtab(i,1,2) = pttab_child(i) - deb
C                    
          Else              
C
            indtab(i,1,1) = pttab_child(i) - fin - 1
C
            indtab(i,1,2) = pttab_child(i) - deb - 1  
C
        Endif               
C
        lbglob = lbound(child % var % array1,i)
        ubglob = ubound(child % var % array1,i)
C
#ifdef AGRIF_MPI
C
        lb = lbound(child % var % array1,i)
        ub = ubound(child % var % array1,i)
C
        iming = huge(1)
        imaxg = -huge(1)
C
        Do indloc = lb,ub,ub-lb
C
          Call AGRIF_InvLoc(indloc,AGRIF_ProcRank,i,indglob)
C
          iming = min(iming,indglob)
C
          imaxg = max(imaxg,indglob)
C
        End Do 
C
        Call MPI_ALLREDUCE(iming,lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     MPI_COMM_WORLD,code)  
C
        Call MPI_ALLREDUCE(imaxg,ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     MPI_COMM_WORLD,code)   
C
#endif
C
        indtruetab(i,1,1) = max(indtab(i,1,1),lbglob)
C     
        indtruetab(i,1,2) = max(indtab(i,1,2),lbglob)       
C
        indtruetab(i,2,1) = min(indtab(i,2,1),ubglob)
C
        indtruetab(i,2,2) = min(indtab(i,2,2),ubglob)         
C
      End Do
C 
C
      If (child % var % interpIndex 
     &        /= Agrif_Curgrid % parent % ngridstep) then
C
C     Space interpolation 
C      
      kindex = 1    
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2                
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then    
C        
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                                                     
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i)   
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                                                                             
C
                    End If                        
C      
                End if
C      
              End do
C
              Call Agrif_Interp1D              
     &             (TYPEInterp,parent,child,
     &              ptres(1:nbdim,1),ptres(1:nbdim,2),
     &              pttab_child(1:nbdim),pttab_Parent(1:nbdim),
     &              s_Child(1:nbdim),s_Parent(1:nbdim),
     &              ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &              restore,.FALSE.,nbdim)
C     
C             Safeguard of the values of the grid variable (at times n and n+1 on the parent grid)
C      
              sizetab = 1
C
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array1,i),
     &                                  ubound(child%var%array1,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array1,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array1,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array1,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array1,i)) 
C
                End If                  
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array1(ir)
       End Do
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array1(ir)
       End Do
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#endif
C
              deallocate(tab)     
C
          End if
C
        End Do       
C
      End Do     
C
C
      child % var % interpIndex = Agrif_Curgrid % parent % ngridstep
C
C
      End if              
C
C
C     Calculation of the coefficients c1t and c2t for the temporary interpolation
      If (pweight) then
C
          c1t = weight
C
        Else
C
          c1t = (Real(AGRIF_Nbstepint()) + 1.) / Agrif_Rhot()
C
      End if
C                                  
      c2t = 1. - c1t
C           
C     Time interpolation
C
      kindex = 1
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2      
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then
C           
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                              
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i) 
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                              
C
                    End If                        
C      
                End if
C      
              End do
C      
              sizetab=1
C     
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array1,i),
     &                                  ubound(child%var%array1,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array1,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array1,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array1,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array1,i)) 
C
                End If               
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array1(ir)
       End Do
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array1(ir)
       End Do
C
#endif      
C                  
              Call timeInterpolation
     &             (child,tab,kindex,c1t,c2t)           
C
#ifdef AGRIF_MPI              
C
	nind=0
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
           child%var%array1(ir)=tab(nind)
       End Do
       
C
#else
C
	nind=0
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
           child%var%array1(ir)=tab(nind)
       End Do
C
#endif  
C         
              deallocate(tab)
C 
          End if
C
        End Do
C      
      End Do
C      
C  
      End Subroutine Agrif_Correct1d
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Correct2d
C     **************************************************************************               
C
      Subroutine AGRIF_Correct2d(TypeInterp,parent,child,deb,fin,
     &                           pweight,weight,
     &                           pttab_child,pttab_Parent,
     &                           nbtab_Child,posvartab_Child,
     &                           loctab_Child,
     &                           s_Child,s_Parent,
     &                           ds_Child,ds_Parent,nbdim)
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions for a 2D grid variable on 
CCC   a fine grid by using a space and time interpolations; it is called by the 
CCC   Agrif_CorrectVariable procedure.
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: TypeInterp                          ! Type of interpolation (linear,spline,...)  
      Type(AGRIF_PVariable) :: parent                ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child                 ! Variable on the child grid 
      Integer :: deb,fin                             ! Positions where interpolations are done
      Logical :: pweight                             ! Indicates if weight is used for the temporal interpolation
      Real :: weight                                 ! Coefficient for the temporal interpolation
      Integer                  :: nbdim              ! Number of dimensions of the grid variable
      Integer,Dimension(nbdim) :: pttab_child        ! Index of the first point inside the domain for the parent grid variable              
      Integer,Dimension(nbdim) :: pttab_Parent       ! Index of the first point inside the domain for the child grid variable
      Integer,Dimension(nbdim) :: nbtab_Child        ! Number of cells of the child grid
      Integer,Dimension(nbdim) :: posvartab_Child    ! Position of the grid variable (1 or 2)
      Integer,Dimension(nbdim) :: loctab_Child       ! Indicates if the child grid has a common border with the root grid
      Real,Dimension(nbdim)    :: s_Child,s_Parent   ! Positions of the parent and child grids 
      Real,Dimension(nbdim)    :: ds_Child,ds_Parent ! Space steps of the parent and child grids
C
C     Local variables
      Integer                  :: lbglob,ubglob
      Integer,Dimension(:,:),Pointer     :: restore  
      Real    :: c1t,c2t                             ! Coefficients for the time interpolation (c2t=1-c1t) 
      Integer :: i                 
      Integer :: kindex                              ! Index used for safeguard and time interpolation
      Integer,Dimension(nbdim,2,2) :: indtab         ! Arrays indicating the limits of the child     
      Integer,Dimension(nbdim,2,2) :: indtruetab     ! grid variable where boundary conditions are 
      Integer,Dimension(nbdim,2) :: ptres            ! calculated
      Integer :: nb,ndir,n,sizetab
      Real, Dimension(:), Allocatable :: tab       ! Array used for the time interpolation
      Integer nind,ir,jr
C
#ifdef AGRIF_MPI
C
      Integer,Dimension(nbdim) :: ltab,utab
      Integer,Dimension(nbdim) :: ltabtrue,utabtrue
      Integer                  :: lb,ub,iming,imaxg,indloc,indglob
      Integer                  :: code
C
#endif            
C      
C
      Do i = 1,nbdim
C
        indtab(i,2,1) = pttab_child(i) + nbtab_child(i) + deb
C        
        indtab(i,2,2) = pttab_child(i) + nbtab_child(i) + fin
C 
        If (posvartab_child(i) .EQ. 1) then
C
            indtab(i,1,1) = pttab_child(i) - fin
C        
            indtab(i,1,2) = pttab_child(i) - deb
C                    
          Else              
C
            indtab(i,1,1) = pttab_child(i) - fin - 1
C
            indtab(i,1,2) = pttab_child(i) - deb - 1  
C
        Endif               
C
        lbglob = lbound(child % var % array2,i)
        ubglob = ubound(child % var % array2,i)
C
#ifdef AGRIF_MPI
C
        lb = lbound(child % var % array2,i)
        ub = ubound(child % var % array2,i)
C
        iming = huge(1)
        imaxg = -huge(1)
C
        Do indloc = lb,ub,ub-lb
C
          Call AGRIF_InvLoc(indloc,AGRIF_ProcRank,i,indglob)
C
          iming = min(iming,indglob)
C
          imaxg = max(imaxg,indglob)
C
        End Do 
C
        Call MPI_ALLREDUCE(iming,lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     MPI_COMM_WORLD,code)  
C
        Call MPI_ALLREDUCE(imaxg,ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     MPI_COMM_WORLD,code)   
C
#endif
C
        indtruetab(i,1,1) = max(indtab(i,1,1),lbglob)
C     
        indtruetab(i,1,2) = max(indtab(i,1,2),lbglob)       
C
        indtruetab(i,2,1) = min(indtab(i,2,1),ubglob)
C
        indtruetab(i,2,2) = min(indtab(i,2,2),ubglob) 
C
      End Do
C 
C
      If (child % var % interpIndex 
     &        /= Agrif_Curgrid % parent % ngridstep) then
C
C     Space interpolation 
C      
      kindex = 1    
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2                
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then    
C        
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                                                     
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i)   
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                                                                             
C
                    End If                        
C      
                End if
C      
              End do
C
              Call Agrif_Interp2D              
     &             (TYPEInterp,parent,child,
     &              ptres(1:nbdim,1),ptres(1:nbdim,2),
     &              pttab_child(1:nbdim),pttab_Parent(1:nbdim),
     &              s_Child(1:nbdim),s_Parent(1:nbdim),
     &              ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &              restore,.FALSE.,nbdim)
C     
C             Safeguard of the values of the grid variable (at times n and n+1 on the parent grid)
C      
              sizetab = 1
C
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array2,i),
     &                                  ubound(child%var%array2,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array2,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array2,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array2,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array2,i)) 
C
                End If                 
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array2(ir,jr)
       End Do
       End Do
       
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array2(ir,jr)
       End Do
       End Do
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#endif
C
              deallocate(tab)     
C
          End if
C
        End Do       
C
      End Do     
C
C
      child % var % interpIndex = Agrif_Curgrid % parent % ngridstep
C
C
      End if              
C
C
C     Calculation of the coefficients c1t and c2t for the temporary interpolation
      If (pweight) then
C
          c1t = weight
C
        Else
C
          c1t = (Real(AGRIF_Nbstepint()) + 1.) / Agrif_Rhot()
C
      End if
C                                  
      c2t = 1. - c1t
C           
C     Time interpolation
C
      kindex = 1
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2      
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then
C           
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                              
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i) 
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                              
C
                    End If                        
C      
                End if
C      
              End do
C      
              sizetab=1
C     
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array2,i),
     &                                  ubound(child%var%array2,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array2,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array2,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array2,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array2,i)) 
C
                End If                 
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array2(ir,jr)
       End Do
       End Do                   
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array2(ir,jr)
       End Do
       End Do
C
#endif      
C                  
              Call timeInterpolation
     &             (child,tab,kindex,c1t,c2t)           
C
#ifdef AGRIF_MPI              
C
C
	nind=0
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              child%var%array2(ir,jr)=tab(nind)
       End Do
       End Do
C
#else
C
	nind=0
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              child%var%array2(ir,jr)=tab(nind)
       End Do
       End Do
C
#endif  
C         
              deallocate(tab)
C 
          End if
C
        End Do
C      
      End Do
C      
C  
      End Subroutine Agrif_Correct2d
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Correct3d
C     **************************************************************************               
C
      Subroutine AGRIF_Correct3d(TypeInterp,parent,child,deb,fin,
     &                           pweight,weight,
     &                           pttab_child,pttab_Parent,
     &                           nbtab_Child,posvartab_Child,
     &                           loctab_Child,
     &                           s_Child,s_Parent,
     &                           ds_Child,ds_Parent,nbdim)
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions for a 3D grid variable on 
CCC   a fine grid by using a space and time interpolations; it is called by the 
CCC   Agrif_CorrectVariable procedure.
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: TypeInterp                          ! Type of interpolation (linear,spline,...)  
      Type(AGRIF_PVariable) :: parent                ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child                 ! Variable on the child grid 
      Integer :: deb,fin                             ! Positions where interpolations are done
      Logical :: pweight                             ! Indicates if weight is used for the temporal interpolation
      Real :: weight                                 ! Coefficient for the temporal interpolation
      Integer                  :: nbdim              ! Number of dimensions of the grid variable
      Integer,Dimension(nbdim) :: pttab_child        ! Index of the first point inside the domain for the parent grid variable              
      Integer,Dimension(nbdim) :: pttab_Parent       ! Index of the first point inside the domain for the child grid variable
      Integer,Dimension(nbdim) :: nbtab_Child        ! Number of cells of the child grid
      Integer,Dimension(nbdim) :: posvartab_Child    ! Position of the grid variable (1 or 2)
      Integer,Dimension(nbdim) :: loctab_Child       ! Indicates if the child grid has a common border with the root grid
      Real,Dimension(nbdim)    :: s_Child,s_Parent   ! Positions of the parent and child grids 
      Real,Dimension(nbdim)    :: ds_Child,ds_Parent ! Space steps of the parent and child grids
C
C     Local variables
      Integer                  :: lbglob,ubglob
      Integer,Dimension(:,:,:),Pointer     :: restore  
      Real    :: c1t,c2t                             ! Coefficients for the time interpolation (c2t=1-c1t) 
      Integer :: i                 
      Integer :: kindex                              ! Index used for safeguard and time interpolation
      Integer,Dimension(nbdim,2,2) :: indtab         ! Arrays indicating the limits of the child     
      Integer,Dimension(nbdim,2,2) :: indtruetab     ! grid variable where boundary conditions are 
      Integer,Dimension(nbdim,2) :: ptres            ! calculated
      Integer :: nb,ndir,n,sizetab
      Real, Dimension(:), Allocatable :: tab       ! Array used for the time interpolation
      Integer nind,ir,jr,kr
C
#ifdef AGRIF_MPI
C
      Integer,Dimension(nbdim) :: ltab,utab
      Integer,Dimension(nbdim) :: ltabtrue,utabtrue
      Integer                  :: lb,ub,iming,imaxg,indloc,indglob
      Integer                  :: code
C
#endif            
	integer j,k
C      
C
      Do i = 1,nbdim
C
        indtab(i,2,1) = pttab_child(i) + nbtab_child(i) + deb
C        
        indtab(i,2,2) = pttab_child(i) + nbtab_child(i) + fin
C 
        If (posvartab_child(i) .EQ. 1) then
C
            indtab(i,1,1) = pttab_child(i) - fin
C        
            indtab(i,1,2) = pttab_child(i) - deb
C                    
          Else              
C
            indtab(i,1,1) = pttab_child(i) - fin - 1
C
            indtab(i,1,2) = pttab_child(i) - deb - 1  
C
        Endif               
C
        lbglob = lbound(child % var % array3,i)
        ubglob = ubound(child % var % array3,i)
C
#ifdef AGRIF_MPI
C
        lb = lbound(child % var % array3,i)
        ub = ubound(child % var % array3,i)
C
        iming = huge(1)
        imaxg = -huge(1)
C
        Do indloc = lb,ub,ub-lb
C
          Call AGRIF_InvLoc(indloc,AGRIF_ProcRank,i,indglob)
C
          iming = min(iming,indglob)
C
          imaxg = max(imaxg,indglob)
C
        End Do 
C
        Call MPI_ALLREDUCE(iming,lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     MPI_COMM_WORLD,code)  
C
        Call MPI_ALLREDUCE(imaxg,ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     MPI_COMM_WORLD,code)  
     
C
#endif
C
        indtruetab(i,1,1) = max(indtab(i,1,1),lbglob)
C     
        indtruetab(i,1,2) = max(indtab(i,1,2),lbglob)       
C
        indtruetab(i,2,1) = min(indtab(i,2,1),ubglob)
C
        indtruetab(i,2,2) = min(indtab(i,2,2),ubglob)        
C
      End Do
C 
C
      If (child % var % interpIndex 
     &        /= Agrif_Curgrid % parent % ngridstep) then
C
C     Space interpolation 
C      
      kindex = 1    
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2                
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then    
C        
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                                                     
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i)   
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                                                                             
C
                    End If                        
C      
                End if
C      
              End do
C
              Call Agrif_Interp3D              
     &             (TYPEInterp,parent,child,
     &              ptres(1:nbdim,1),ptres(1:nbdim,2),
     &              pttab_child(1:nbdim),pttab_Parent(1:nbdim),
     &              s_Child(1:nbdim),s_Parent(1:nbdim),
     &              ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &              restore,.FALSE.,nbdim)
C     
C             Safeguard of the values of the grid variable (at times n and n+1 on the parent grid)
C      
              sizetab = 1
C
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array3,i),
     &                                  ubound(child%var%array3,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array3,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array3,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array3,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array3,i)) 
C
                End If               
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array3(ir,jr,kr)
       End Do
       End Do
       End Do
       
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array3(ir,jr,kr)
       End Do
       End Do
       End Do
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#endif
C
              deallocate(tab)     
C
          End if
C
        End Do       
C
      End Do     
C
C
      child % var % interpIndex = Agrif_Curgrid % parent % ngridstep
C
C
      End if              
C
C
C     Calculation of the coefficients c1t and c2t for the temporary interpolation
      If (pweight) then
C
          c1t = weight
C
        Else
C
          c1t = (Real(AGRIF_Nbstepint()) + 1.) / Agrif_Rhot()
C
      End if
C                                  
      c2t = 1. - c1t
C           
C     Time interpolation
C
      kindex = 1
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2      
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then
C           
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                              
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i) 
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                              
C
                    End If                        
C      
                End if
C      
              End do
C      
              sizetab=1
C     
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array3,i),
     &                                  ubound(child%var%array3,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array3,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array3,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array3,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array3,i))                
C
                End If
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array3(ir,jr,kr)
       End Do
       End Do
       End Do
       
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array3(ir,jr,kr)
       End Do
       End Do
       End Do
C
#endif      
C                  
              Call timeInterpolation
     &             (child,tab,kindex,c1t,c2t)           
C
#ifdef AGRIF_MPI              
C
	nind=0
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              child%var%array3(ir,jr,kr)=tab(nind)
       End Do
       End Do
       End Do
C
#else
C
	nind=0
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              child%var%array3(ir,jr,kr)=tab(nind)
       End Do
       End Do
       End Do
C
#endif  
C         
              deallocate(tab)
C 
          End if
C
        End Do
C      
      End Do
C      
C  
      End Subroutine Agrif_Correct3d 
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Correct4d
C     **************************************************************************               
C
      Subroutine AGRIF_Correct4d(TypeInterp,parent,child,deb,fin,
     &                           pweight,weight,
     &                           pttab_child,pttab_Parent,
     &                           nbtab_Child,posvartab_Child,
     &                           loctab_Child,
     &                           s_Child,s_Parent,
     &                           ds_Child,ds_Parent,nbdim)
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions for a 4D grid variable on 
CCC   a fine grid by using a space and time interpolations; it is called by the 
CCC   Agrif_CorrectVariable procedure.
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: TypeInterp                          ! Type of interpolation (linear,spline,...)  
      Type(AGRIF_PVariable) :: parent                ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child                 ! Variable on the child grid 
      Integer :: deb,fin                             ! Positions where interpolations are done
      Logical :: pweight                             ! Indicates if weight is used for the temporal interpolation
      Real :: weight                                 ! Coefficient for the temporal interpolation
      Integer                  :: nbdim              ! Number of dimensions of the grid variable
      Integer,Dimension(nbdim) :: pttab_child        ! Index of the first point inside the domain for the parent grid variable              
      Integer,Dimension(nbdim) :: pttab_Parent       ! Index of the first point inside the domain for the child grid variable
      Integer,Dimension(nbdim) :: nbtab_Child        ! Number of cells of the child grid
      Integer,Dimension(nbdim) :: posvartab_Child    ! Position of the grid variable (1 or 2)
      Integer,Dimension(nbdim) :: loctab_Child       ! Indicates if the child grid has a common border with the root grid
      Real,Dimension(nbdim)    :: s_Child,s_Parent   ! Positions of the parent and child grids 
      Real,Dimension(nbdim)    :: ds_Child,ds_Parent ! Space steps of the parent and child grids
C
C     Local variables
      Integer                  :: lbglob,ubglob
      Integer,Dimension(:,:,:,:),Pointer     :: restore  
      Real    :: c1t,c2t                             ! Coefficients for the time interpolation (c2t=1-c1t) 
      Integer :: i                 
      Integer :: kindex                              ! Index used for safeguard and time interpolation
      Integer,Dimension(nbdim,2,2) :: indtab         ! Arrays indicating the limits of the child     
      Integer,Dimension(nbdim,2,2) :: indtruetab     ! grid variable where boundary conditions are 
      Integer,Dimension(nbdim,2) :: ptres            ! calculated
      Integer :: nb,ndir,n,sizetab
      Real, Dimension(:), Allocatable :: tab       ! Array used for the time interpolation
      Integer nind,ir,jr,kr,lr
C
#ifdef AGRIF_MPI
C
      Integer,Dimension(nbdim) :: ltab,utab
      Integer,Dimension(nbdim) :: ltabtrue,utabtrue
      Integer                  :: lb,ub,iming,imaxg,indloc,indglob
      Integer                  :: code
C
#endif            
C      
C
      Do i = 1,nbdim
C
        indtab(i,2,1) = pttab_child(i) + nbtab_child(i) + deb
C        
        indtab(i,2,2) = pttab_child(i) + nbtab_child(i) + fin
C 
        If (posvartab_child(i) .EQ. 1) then
C
            indtab(i,1,1) = pttab_child(i) - fin
C        
            indtab(i,1,2) = pttab_child(i) - deb
C                    
          Else              
C
            indtab(i,1,1) = pttab_child(i) - fin - 1
C
            indtab(i,1,2) = pttab_child(i) - deb - 1  
C
        Endif               
C
        lbglob = lbound(child % var % array4,i)
        ubglob = ubound(child % var % array4,i)
C
#ifdef AGRIF_MPI
C
        lb = lbound(child % var % array4,i)
        ub = ubound(child % var % array4,i)
C
        iming = huge(1)
        imaxg = -huge(1)
C
        Do indloc = lb,ub,ub-lb
C
          Call AGRIF_InvLoc(indloc,AGRIF_ProcRank,i,indglob)
C
          iming = min(iming,indglob)
C
          imaxg = max(imaxg,indglob)
C
        End Do 
C
        Call MPI_ALLREDUCE(iming,lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     MPI_COMM_WORLD,code)  
C
        Call MPI_ALLREDUCE(imaxg,ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     MPI_COMM_WORLD,code)   
C
#endif
C
        indtruetab(i,1,1) = max(indtab(i,1,1),lbglob)
C     
        indtruetab(i,1,2) = max(indtab(i,1,2),lbglob)       
C
        indtruetab(i,2,1) = min(indtab(i,2,1),ubglob)
C
        indtruetab(i,2,2) = min(indtab(i,2,2),ubglob)        
C
      End Do
C 
C
      If (child % var % interpIndex 
     &        /= Agrif_Curgrid % parent % ngridstep) then
C
C     Space interpolation 
C      
      kindex = 1    
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2                
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then    
C        
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                                                     
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i)   
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                                                                             
C
                    End If                        
C      
                End if
C      
              End do
C
              Call Agrif_Interp4D              
     &             (TYPEInterp,parent,child,
     &              ptres(1:nbdim,1),ptres(1:nbdim,2),
     &              pttab_child(1:nbdim),pttab_Parent(1:nbdim),
     &              s_Child(1:nbdim),s_Parent(1:nbdim),
     &              ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &              restore,.FALSE.,nbdim)
C     
C             Safeguard of the values of the grid variable (at times n and n+1 on the parent grid)
C      
              sizetab = 1
C
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array4,i),
     &                                  ubound(child%var%array4,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array4,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array4,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array4,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array4,i)) 
C
                End If               
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	Do lr=ltabtrue(4),utabtrue(4)
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array4(ir,jr,kr,lr)
       End Do
       End Do
       End Do
       End Do
       
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do lr=ptres(4,1),ptres(4,2)
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array4(ir,jr,kr,lr)
       End Do
       End Do
       End Do
       End Do

C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#endif
C
              deallocate(tab)     
C
          End if
C
        End Do       
C
      End Do     
C
C
      child % var % interpIndex = Agrif_Curgrid % parent % ngridstep
C
C
      End if              
C
C
C     Calculation of the coefficients c1t and c2t for the temporary interpolation
      If (pweight) then
C
          c1t = weight
C
        Else
C
          c1t = (Real(AGRIF_Nbstepint()) + 1.) / Agrif_Rhot()
C
      End if
C                                  
      c2t = 1. - c1t
C           
C     Time interpolation
C
      kindex = 1
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2      
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then
C           
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                              
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i) 
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                              
C
                    End If                        
C      
                End if
C      
              End do
C      
              sizetab=1
C     
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array4,i),
     &                                  ubound(child%var%array4,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array4,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array4,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array4,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array4,i))                
C
                End If
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C


	nind=0
	Do lr=ltabtrue(4),utabtrue(4)
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array4(ir,jr,kr,lr)
       End Do
       End Do
       End Do
       End Do
       
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	Do lr=ptres(4,1),ptres(4,2)
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array4(ir,jr,kr,lr)
       End Do
       End Do
       End Do
       End Do
C
#endif      
C                  
              Call timeInterpolation
     &             (child,tab,kindex,c1t,c2t)           
C
#ifdef AGRIF_MPI              
C
	nind=0
	Do lr=ltabtrue(4),utabtrue(4)
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              child%var%array4(ir,jr,kr,lr)=tab(nind)
       End Do
       End Do
       End Do
       End Do
C
#else
C
	nind=0
	Do lr=ptres(4,1),ptres(4,2)
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              child%var%array4(ir,jr,kr,lr)=tab(nind)
       End Do
       End Do
       End Do
       End Do
C
#endif  
C         
              deallocate(tab)
C 
          End if
C
        End Do
C      
      End Do
C      
C  
      End Subroutine Agrif_Correct4d 
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Correct5d
C     **************************************************************************               
C
      Subroutine AGRIF_Correct5d(TypeInterp,parent,child,deb,fin,
     &                           pweight,weight,
     &                           pttab_child,pttab_Parent,
     &                           nbtab_Child,posvartab_Child,
     &                           loctab_Child,
     &                           s_Child,s_Parent,
     &                           ds_Child,ds_Parent,nbdim)
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions for a 5D grid variable on 
CCC   a fine grid by using a space and time interpolations; it is called by the 
CCC   Agrif_CorrectVariable procedure.
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: TypeInterp                          ! Type of interpolation (linear,spline,...)  
      Type(AGRIF_PVariable) :: parent                ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child                 ! Variable on the child grid 
      Integer :: deb,fin                             ! Positions where interpolations are done
      Logical :: pweight                             ! Indicates if weight is used for the temporal interpolation
      Real :: weight                                 ! Coefficient for the temporal interpolation
      Integer                  :: nbdim              ! Number of dimensions of the grid variable
      Integer,Dimension(nbdim) :: pttab_child        ! Index of the first point inside the domain for the parent grid variable              
      Integer,Dimension(nbdim) :: pttab_Parent       ! Index of the first point inside the domain for the child grid variable
      Integer,Dimension(nbdim) :: nbtab_Child        ! Number of cells of the child grid
      Integer,Dimension(nbdim) :: posvartab_Child    ! Position of the grid variable (1 or 2)
      Integer,Dimension(nbdim) :: loctab_Child       ! Indicates if the child grid has a common border with the root grid
      Real,Dimension(nbdim)    :: s_Child,s_Parent   ! Positions of the parent and child grids 
      Real,Dimension(nbdim)    :: ds_Child,ds_Parent ! Space steps of the parent and child grids
C
C     Local variables
      Integer                  :: lbglob,ubglob
      Integer,Dimension(:,:,:,:,:),Pointer     :: restore  
      Real    :: c1t,c2t                             ! Coefficients for the time interpolation (c2t=1-c1t) 
      Integer :: i                 
      Integer :: kindex                              ! Index used for safeguard and time interpolation
      Integer,Dimension(nbdim,2,2) :: indtab         ! Arrays indicating the limits of the child     
      Integer,Dimension(nbdim,2,2) :: indtruetab     ! grid variable where boundary conditions are 
      Integer,Dimension(nbdim,2) :: ptres            ! calculated
      Integer :: nb,ndir,n,sizetab
      Real, Dimension(:), Allocatable :: tab       ! Array used for the time interpolation
      Integer nind,ir,jr,kr,lr,mr
C
#ifdef AGRIF_MPI
C
      Integer,Dimension(nbdim) :: ltab,utab
      Integer,Dimension(nbdim) :: ltabtrue,utabtrue
      Integer                  :: lb,ub,iming,imaxg,indloc,indglob
      Integer                  :: code
C
#endif            
C      
C
      Do i = 1,nbdim
C
        indtab(i,2,1) = pttab_child(i) + nbtab_child(i) + deb
C        
        indtab(i,2,2) = pttab_child(i) + nbtab_child(i) + fin
C 
        If (posvartab_child(i) .EQ. 1) then
C
            indtab(i,1,1) = pttab_child(i) - fin
C        
            indtab(i,1,2) = pttab_child(i) - deb
C                    
          Else              
C
            indtab(i,1,1) = pttab_child(i) - fin - 1
C
            indtab(i,1,2) = pttab_child(i) - deb - 1  
C
        Endif               
C
        lbglob = lbound(child % var % array5,i)
        ubglob = ubound(child % var % array5,i)
C
#ifdef AGRIF_MPI
C
        lb = lbound(child % var % array5,i)
        ub = ubound(child % var % array5,i)
C
        iming = huge(1)
        imaxg = -huge(1)
C
        Do indloc = lb,ub,ub-lb
C
          Call AGRIF_InvLoc(indloc,AGRIF_ProcRank,i,indglob)
C
          iming = min(iming,indglob)
C
          imaxg = max(imaxg,indglob)
C
        End Do 
C
        Call MPI_ALLREDUCE(iming,lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     MPI_COMM_WORLD,code)  
C
        Call MPI_ALLREDUCE(imaxg,ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     MPI_COMM_WORLD,code)   
C
#endif
C
        indtruetab(i,1,1) = max(indtab(i,1,1),lbglob)
C     
        indtruetab(i,1,2) = max(indtab(i,1,2),lbglob)       
C
        indtruetab(i,2,1) = min(indtab(i,2,1),ubglob)
C
        indtruetab(i,2,2) = min(indtab(i,2,2),ubglob)        
C
      End Do
C 
C
      If (child % var % interpIndex 
     &        /= Agrif_Curgrid % parent % ngridstep) then
C
C     Space interpolation 
C      
      kindex = 1    
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2                
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then    
C        
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                                                     
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i)   
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                                                                             
C
                    End If                        
C      
                End if
C      
              End do
C
              Call Agrif_Interp5D              
     &             (TYPEInterp,parent,child,
     &              ptres(1:nbdim,1),ptres(1:nbdim,2),
     &              pttab_child(1:nbdim),pttab_Parent(1:nbdim),
     &              s_Child(1:nbdim),s_Parent(1:nbdim),
     &              ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &              restore,.FALSE.,nbdim)
C     
C             Safeguard of the values of the grid variable (at times n and n+1 on the parent grid)
C      
              sizetab = 1
C
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array5,i),
     &                                  ubound(child%var%array5,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array5,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array5,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array5,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array5,i)) 
C
                End If               
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	DO mr=ltabtrue(5),utabtrue(5)
	Do lr=ltabtrue(4),utabtrue(4)
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array5(ir,jr,kr,lr,mr)
       End Do
       End Do
       End Do
       End Do
       End Do
       
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	DO mr=ptres(5,1),ptres(5,2)
	Do lr=ptres(4,1),ptres(4,2)
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array5(ir,jr,kr,lr,mr)
       End Do
       End Do
       End Do
       End Do
       End Do
C     
              Call saveAfterInterp
     &             (child,tab,kindex)
C
#endif
C
              deallocate(tab)     
C
          End if
C
        End Do       
C
      End Do     
C
C
      child % var % interpIndex = Agrif_Curgrid % parent % ngridstep
C
C
      End if              
C
C
C     Calculation of the coefficients c1t and c2t for the temporary interpolation
      If (pweight) then
C
          c1t = weight
C
        Else
C
          c1t = (Real(AGRIF_Nbstepint()) + 1.) / Agrif_Rhot()
C
      End if
C                                  
      c2t = 1. - c1t
C           
C     Time interpolation
C
      kindex = 1
C
      Do nb = 1,nbdim
C
        Do ndir = 1,2      
C
          If (loctab_child(nb) /= (-ndir) 
     &        .AND. loctab_child(nb) /= -3) then
C           
              Do n = 1,2
C
                ptres(nb,n) = indtruetab(nb,ndir,n)  
C
              End Do              
C
              Do i = 1,nbdim
C     
                If (i .NE. nb) then      
C
                    If (loctab_child(i) == -1 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        ptres(i,1) = pttab_child(i)
C
                      Else
C
                        ptres(i,1) = indtruetab(i,1,1)                                              
C
                    End If
C
                    If (loctab_child(i) == -2 
     &                                 .OR. loctab_child(i) == -3) Then
C
                        If (posvartab_child(i) == 1) Then
C
                            ptres(i,2) = pttab_child(i) 
     &                                                 + nbtab_child(i) 
C
                          Else
C
                            ptres(i,2) = pttab_child(i) 
     &                                             + nbtab_child(i) - 1   
C
                        End If                             
C
                      Else
C
                        ptres(i,2) = indtruetab(i,2,2)                                              
C
                    End If                        
C      
                End if
C      
              End do
C      
              sizetab=1
C     
#ifdef AGRIF_MPI 
C
              Do i = 1,nbdim
C
                Call GetLocalBoundaries(ptres(i,1),ptres(i,2),i,
     &                                  lbound(child%var%array5,i),
     &                                  ubound(child%var%array5,i),
     &                                  ltab(i),utab(i))
C
                ltabtrue(i) = max(ltab(i),
     &                                lbound(child%var%array5,i)) 
C
                utabtrue(i) = min(utab(i),
     &                                ubound(child%var%array5,i))
C
                If ((i == nb) .AND. (ndir == 1)) Then 
C                  
                    utabtrue(i) = max(utab(i),
     &                                lbound(child%var%array5,i))
C
                  Else if ((i == nb) .AND. (ndir == 2)) Then
C
                    ltabtrue(i) = min(ltab(i),
     &                                ubound(child%var%array5,i))                
C
                End If
C
                sizetab = sizetab * (utabtrue(i) - ltabtrue(i) + 1)
C
              End Do              
C
              allocate(tab(sizetab))
C
	nind=0
	DO mr=ltabtrue(5),utabtrue(5)
	Do lr=ltabtrue(4),utabtrue(4)
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              tab(nind) = child%var%array5(ir,jr,kr,lr,mr)
       End Do
       End Do
       End Do
       End Do
       End Do
C
#else      
C
              Do i = 1,nbdim
C	  
                sizetab = sizetab * (ptres(i,2)-ptres(i,1)+1)
C      
              End Do
C              
              allocate(tab(sizetab))
C
	nind=0
	DO mr=ptres(5,1),ptres(5,2)
	Do lr=ptres(4,1),ptres(4,2)
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              tab(nind) = child%var%array5(ir,jr,kr,lr,mr)
       End Do
       End Do
       End Do
       End Do
       End Do
C
#endif      
C                  
              Call timeInterpolation
     &             (child,tab,kindex,c1t,c2t)           
C
#ifdef AGRIF_MPI              
C
	nind=0
	DO mr=ltabtrue(5),utabtrue(5)
	Do lr=ltabtrue(4),utabtrue(4)
	Do kr=ltabtrue(3),utabtrue(3)
	Do jr=ltabtrue(2),utabtrue(2)
	Do ir=ltabtrue(1),utabtrue(1)
	nind=nind+1
              child%var%array5(ir,jr,kr,lr,mr)=tab(nind)
       End Do
       End Do
       End Do
       End Do
       End Do         
C
#else
C
	nind=0
	DO mr=ptres(5,1),ptres(5,2)
	Do lr=ptres(4,1),ptres(4,2)
	Do kr=ptres(3,1),ptres(3,2)
	Do jr=ptres(2,1),ptres(2,2)
	Do ir=ptres(1,1),ptres(1,2)
	nind=nind+1
              child%var%array5(ir,jr,kr,lr,mr)=tab(nind)
       End Do
       End Do
       End Do
       End Do
       End Do
C
#endif  
C         
              deallocate(tab)
C 
          End if
C
        End Do
C      
      End Do
C      
C  
      End Subroutine Agrif_Correct5d            
C
C
C
C     **************************************************************************
CCC   Subroutine saveAfterInterp
C     **************************************************************************
C
      Subroutine saveAfterInterp(child,tab,kindex)
C
CCC   Descritpion:
CCC   Subroutine used to save the values of the grid variable on the fine grid 
CCC   after the space interpolation. 
C
C     Declarations:
C
      Implicit none
C
C     Type argument
      Type (AGRIF_PVariable) :: child   ! The fine grid variable
      Real, Dimension(:) :: tab         ! Values on the fine grid variable after the space interpolation  
C
C     Scalar arguments
      Integer :: kindex                 ! Index indicating where this safeguard 
                                        ! is done on the fine grid
C
C     Local scalars
      Integer :: newsize                ! Size of the domain where boundary onditions are calculated
      Integer :: i
C
C
C     Allocation of the array oldvalues2d
      newsize = size(tab)
C
      if (newsize .LE. 0) return
C
      Call checkSize
     &     (child,kindex+newsize)  
C
C
C     Safeguard in the oldvalues2d array       
C            
      Do i = 1,newsize
C
        child % var % oldvalues2d(kindex,1) = child % var %
     &                                        oldvalues2d(kindex,2) 
C
        child % var % oldvalues2d(kindex,2) = tab(i)
C
        kindex = kindex + 1
C
      End do
C
C                                                  
      End subroutine saveAfterInterp
C
C
C
C     **************************************************************************
CCC   Subroutine timeInterpolation
C     **************************************************************************
C
      Subroutine timeInterpolation(child,tab,kindex,c1t,c2t) 
C
CCC   Descritpion:
CCC   Subroutine for a linear time interpolation on the child grid. 
C
C     Declarations:
C
      Implicit none
C
C     Type argument
      Type (AGRIF_PVariable) :: child  ! The fine grid variable
      Real, Dimension(:) :: tab
C
C     Scalar arguments      
      Integer :: kindex                ! Index indicating the values of the fine 
                                       ! grid got before and after the space 
                                       ! interpolation and used for the time 
                                       ! interpolation
      Real :: c1t,c2t                  ! coefficients for the time interpolation    
                                       ! (c2t=1-c1t)  
C
C     Local aruments      
      Integer :: i
C
C
      Do i = 1,size(tab)
C
        tab(i) = c2t*child % var % oldvalues2d(kindex,1)   
     &         + c1t*child % var % oldvalues2d(kindex,2)        
C 
        kindex = kindex + 1        
C
      End do                                             
C
C
      End subroutine timeInterpolation      
C
C
C
C     **************************************************************************
CCC   Subroutine checkSize
C     **************************************************************************
C
      Subroutine checkSize(child,newsize)
C
CCC   Descritpion:
CCC   Subroutine used in the saveAfterInterp procedure to allocate the 
CCC   oldvalues2d array. 
C
C     Declarations:
C
      Implicit none
C      
C     Type argument
      Type (AGRIF_PVariable) :: child  ! The fine grid variable
C 
C     Scalar arguments
      Integer :: newsize               ! Size of the domains where the boundary 
                                       ! conditions are calculated  
      Integer :: k             
C
C     Local arrays
      Real, dimension(:,:), Allocatable :: tempoldvalues ! Temporary array                                        
C                
C
      If (.not. associated(child % var % oldvalues2d)) then
C
          allocate(child % var % oldvalues2d(newsize,2))
C  
          child % var % oldvalues2d=0.
C
        Else
C
          If (SIZE(child % var % oldvalues2d,1) < newsize) then   
C
              allocate(tempoldvalues(SIZE(child % var %
     &                                    oldvalues2d,1),2))
C
              tempoldvalues = child % var % oldvalues2d
C
              deallocate(child % var % oldvalues2d)
C
              allocate(child % var % oldvalues2d(newsize,2))
C            
              child%var%oldvalues2d=0.
C
              child % var % oldvalues2d(1:SIZE(tempoldvalues,1),:) = 
     &        tempoldvalues(:,:)
C
              deallocate(tempoldvalues)
C
          End if 
C
      End if                                                    
C
C
      End  Subroutine checkSize
C
C
C
#ifdef AGRIF_MPI
C
C     **************************************************************************
CCC   Subroutine GetLocalBoundaries
C     **************************************************************************
C
      Subroutine GetLocalBoundaries(tab1,tab2,i,lb,ub,deb,fin)
C
CCC   Descritpion:
C 
C
C     Declarations:
C
      Implicit none 
C
#include "mpif.h"
C
C     Scalar arguments
      Integer  ::  tab1,tab2
      Integer  ::  i
      Integer  ::  lb,ub
      Integer  ::  deb,fin
C
C     Local scalars
      Integer  ::  imin,imax
      Integer  ::  i1,i2  
C
C
      Call AGRIF_InvLoc(lb,AGRIF_ProcRank,i,imin)
C
      Call AGRIF_InvLoc(ub,AGRIF_ProcRank,i,imax)
C
C
      If (imin > tab2) Then
C
          i1 = imax - imin
C
        Else
C
          i1 = max(tab1 - imin,0)
C
      End If
C
      If (imax < tab1) Then
C
          i2 = -(imax - imin)
C
        Else
C
          i2 = min(tab2 - imax,0)
C
      End If 
C
      deb = lb + i1
C
      fin = ub + i2      
C
C
      End Subroutine GetLocalBoundaries 
C
#endif
C
C
C      
      End Module AGRIF_boundary

