!
! $Id: modcluster.F,v 1.3 2004/11/29 13:26:07 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module AGRIF_Clustering  
C
#include "keys_agrif.h"
C
      Module Agrif_Clustering 
C
CCC   Description:
CCC   Module to create and initialize the grid hierarchy from the 
CCC   AGRIF_FixedGrids.in file.
C
C     Modules used:
C  
      Use Agrif_Curgridfunctions 
      Use Agrif_Init_Vars
      Use Agrif_Save
C             
C 
#include "clustering_agrif.h"
C        
      Contains 
C     Define procedures contained in this module
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Cluster_All  
C     **************************************************************************
C
      Recursive Subroutine Agrif_Cluster_All(g,coarsegrid)
C
CCC   Description:
CCC   Subroutine for the clustering. A temporary grid hierarchy, pointed by 
CCC   coarsegrid, is created.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Pointer arguments    
      Type(AGRIF_grid),pointer   :: g        ! Pointer on the current grid
      Type(AGRIF_rectangle),pointer   :: coarsegrid
C      
C     Local pointer         
      Type(AGRIF_lrectangle),pointer  :: parcours 
      Type(AGRIF_grid),pointer   :: newgrid 
      Real :: g_eps
C      
C        
      g_eps = g%Agrif_dx
C          
      If (Agrif_Probdim.GE.2) Then
C          
          g_eps = min(g_eps,g%Agrif_dy)
C            
          If (Agrif_Probdim.GE.3) Then
C              
              g_eps = min(g_eps,g%Agrif_dz)      
C            
          End If
C          
      End If
C
      g_eps = g_eps/100.                  
C
C     Necessary condition for clustering 
      If (g%Agrif_dx/Agrif_coeffrefx.LT.(Agrif_mindx-g_eps))
     &   Return
C      
      If (Agrif_Probdim.GE.2) Then
C        
          If (g%Agrif_dy/Agrif_coeffrefy.LT.(Agrif_mindy-g_eps))
     &        Return       
C          
          If (Agrif_Probdim.GE.3) Then
C            
              If (g%Agrif_dz/Agrif_coeffrefz.LT.
     &            (Agrif_mindz-g_eps)) Return        
C          
          End If         
C      
      End If
C
      Nullify(coarsegrid%childgrids)                                         
C
#ifdef Agrif_Probdim_1D
C
      Call Agrif_ClusterGrid1D(g,coarsegrid)  
C
#endif
C
#ifdef Agrif_Probdim_2D
C
      Call Agrif_ClusterGrid2D(g,coarsegrid) 
C
#endif
C
#ifdef Agrif_Probdim_3D
C
      Call Agrif_ClusterGrid3D(g,coarsegrid)
C
#endif
C 
C
      parcours => coarsegrid % childgrids                   
C      
      Do while (associated(parcours))
C
C       Newgrid is created. It is a copy of a fine grid created previously by clustering.      
        Allocate(newgrid)
C      
        Nullify(newgrid%child_grids)
C                        
        newgrid % nbx = (parcours % r % imax - parcours % r %
     &       imin)*Agrif_Coeffrefx
C        
        newgrid % Agrif_x = g%Agrif_x + (parcours %r % imin -1) *
     &    g%Agrif_dx
C 
        newgrid % Agrif_dx = g%Agrif_dx / Agrif_Coeffrefx
C
        If (Agrif_Probdim .GE. 2) Then
C            
            newgrid % nby = (parcours % r % jmax - parcours % r %
     &                       jmin)*Agrif_Coeffrefy
C 
            newgrid % Agrif_y = g%Agrif_y + (parcours %r % jmin -1) *
     &                                       g%Agrif_dy
C 
            newgrid % Agrif_dy = g%Agrif_dy / Agrif_Coeffrefy
C
        End If          
C
        If (Agrif_Probdim .EQ. 3) Then            
C      
            newgrid % nbz = (parcours % r % kmax - parcours % r %
     &                       kmin)*Agrif_Coeffrefz
C            
            newgrid % Agrif_z = g%Agrif_z + (parcours %r % kmin -1) *
     &                                       g%Agrif_dz 
C            
            newgrid % Agrif_dz = g%Agrif_dz / Agrif_Coeffrefz      
C
        End If 
      
C
#ifdef Agrif_Probdim_1D
C        
        allocate(newgrid%tabpoint1D(newgrid%nbx+1))
C
        newgrid%tabpoint1D = 0
C 
C       Points detection on newgrid         
        Call Agrif_Create_Tabpoints1D(Agrif_mygrid,newgrid)
C
#endif      
C
#ifdef Agrif_Probdim_2D
C
        allocate(newgrid%tabpoint2D(newgrid%nbx+1,newgrid%nby+1))                 
C
        newgrid%tabpoint2D = 0      
C
C       Points detection on newgrid          
        Call Agrif_Create_Tabpoints2D(Agrif_mygrid,newgrid) 
C
#endif
C
#ifdef Agrif_Probdim_3D                           
C
        allocate(newgrid%tabpoint3D(newgrid%nbx+1,
     &                      newgrid%nby+1,newgrid%nbz+1))
C
        newgrid%tabpoint3D = 0
C
C       Points detection on newgrid
        Call Agrif_Create_Tabpoints3D(Agrif_mygrid,newgrid) 
C
#endif
C
C       Recursive call to Agrif_Cluster_All            
        Call Agrif_Cluster_All
     &           (newgrid, parcours % r)       
C
        parcours => parcours % next 
C
#ifdef Agrif_Probdim_1D
C      
        Deallocate(newgrid%tabpoint1D)
C
#endif
C
#ifdef Agrif_Probdim_2D
C
        Deallocate(newgrid%tabpoint2D)
#endif
C
#ifdef Agrif_Probdim_3D
C
        Deallocate(newgrid%tabpoint3D)
C
#endif      
C
        Deallocate(newgrid)   
C
      End Do
C
C      
      End Subroutine Agrif_Cluster_All
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Create_Tabpoints1D
C     **************************************************************************
C
      Recursive Subroutine Agrif_Create_Tabpoints1D(g,newgrid)
C
CCC   Description:
CCC   Copy on newgrid of points detected on the grid hierarchy pointed by g. 
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Type(Agrif_Grid),pointer   :: g,newgrid
C
C     Local variables      
      Integer :: i,i0
      Type(Agrif_Pgrid),pointer  :: parcours
      Real :: g_eps,newgrid_eps,eps
      Real :: xgmin,xgmax
      Real :: xnewmin,xnewmax
      Real :: xmin,xmax
      Integer :: igmin,inewmin,igmax,inewmax            
C
C     
      parcours => g % child_grids
C
      Do While(associated(parcours))
C
        Call Agrif_Create_Tabpoints1D(parcours%gr,newgrid)
C
        parcours => parcours % next
C
      End Do      
C
      g_eps = g%Agrif_dx
C      
      newgrid_eps = newgrid%Agrif_dx
C
      eps = min(g_eps,newgrid_eps)/100.                  
C                   
      If ((newgrid%Agrif_dx.LT.(g%Agrif_dx-eps))) Return
C                   
      xgmin = g%Agrif_x
      xgmax = g%Agrif_x + g%nbx * g%Agrif_dx            
C 
      xnewmin = newgrid%Agrif_x
      xnewmax = newgrid%Agrif_x + newgrid%nbx * newgrid%Agrif_dx      
C      
      If ((xgmax.LT.xnewmin)) Return
C
      If ((xgmin.GT.xnewmax)) Return
C           
      inewmin = 1 - floor(-(max(xgmin,xnewmin)-xnewmin)
     &                   /newgrid%Agrif_dx) 
C          
      xmin = newgrid%Agrif_x + (inewmin-1)*newgrid%Agrif_dx          
C        
      igmin = 1 + nint((xmin-xgmin)/g%Agrif_dx)      
C
      inewmax = 1 + int((min(xgmax,xnewmax)-xnewmin)/newgrid%Agrif_dx)
C         
      xmax = newgrid%Agrif_x + (inewmax-1)*newgrid%Agrif_dx 
C          
      igmax = 1 + nint((xmax-xgmin)/g%Agrif_dx)
C        
      i0 = igmin
C          
      Do i = inewmin,inewmax
C
        newgrid%tabpoint1D(i) = max(newgrid%tabpoint1D(i),
     &                                                g%tabpoint1D(i0))
C 
        i0 = i0 + int(newgrid%Agrif_dx/g%Agrif_dx)
C                  
      End Do                                        
C
      Return
C
C
      End Subroutine Agrif_Create_Tabpoints1D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Create_Tabpoints2D
C     **************************************************************************
C
      Recursive Subroutine Agrif_Create_Tabpoints2D(g,newgrid)
C
CCC   Description:
CCC   Copy on newgrid of points detected on the grid hierarchy pointed by g.   
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Arguments 
      Type(Agrif_Grid),pointer   :: g,newgrid
C
C     Local variables       
      Integer :: i,j,i0,j0
      Type(Agrif_Pgrid),pointer  :: parcours
      Real :: g_eps,newgrid_eps,eps
      Real :: xgmin,xgmax,ygmin,ygmax
      Real :: xnewmin,xnewmax,ynewmin,ynewmax
      Real :: xmin,xmax,ymin,ymax
      Integer :: igmin,inewmin,igmax,inewmax
      Integer :: jgmin,jnewmin,jgmax,jnewmax            
C
C     
      parcours => g % child_grids
C
      Do While(associated(parcours))
C
        Call Agrif_Create_Tabpoints2D(parcours%gr,newgrid)
C
        parcours => parcours % next
C
      End Do      
C
      g_eps = min(g%Agrif_dx,g%Agrif_dy)
C      
      newgrid_eps = min(newgrid%Agrif_dx,newgrid%Agrif_dy)
C
      eps = min(g_eps,newgrid_eps)/100.                  
C                   
      If ((newgrid%Agrif_dx .LT. (g%Agrif_dx-eps)) .OR.
     &    (newgrid%Agrif_dy .LT. (g%Agrif_dy-eps))) Return
C                   
      xgmin = g%Agrif_x
      xgmax = g%Agrif_x + g%nbx * g%Agrif_dx            
C 
      xnewmin = newgrid%Agrif_x
      xnewmax = newgrid%Agrif_x + newgrid%nbx * newgrid%Agrif_dx
C      
      ygmin = g%Agrif_y
      ygmax = g%Agrif_y + g%nby * g%Agrif_dy            
C 
      ynewmin = newgrid%Agrif_y
      ynewmax = newgrid%Agrif_y + newgrid%nby * newgrid%Agrif_dy
C      
      If ((xgmax .LT. xnewmin) .OR.
     &        (ygmax .LT. ynewmin)) Return
C
      If ((xgmin .GT. xnewmax) .OR.
     &        (ygmin .GT. ynewmax)) Return
C           
      inewmin = 1 - floor(-(max(xgmin,xnewmin)-xnewmin)
     &                   /newgrid%Agrif_dx) 
C          
      xmin = newgrid%Agrif_x + (inewmin-1)*newgrid%Agrif_dx          
C        
      igmin = 1 + nint((xmin-xgmin)/g%Agrif_dx)      
C
      inewmax = 1 + int((min(xgmax,xnewmax)-xnewmin)/newgrid%Agrif_dx)
C         
      xmax = newgrid%Agrif_x + (inewmax-1)*newgrid%Agrif_dx 
C          
      igmax = 1 + nint((xmax-xgmin)/g%Agrif_dx)
C
      jnewmin = 1 - floor(-(max(ygmin,ynewmin)-ynewmin)
     &                   /newgrid%Agrif_dy) 
C          
      ymin = newgrid%Agrif_y + (jnewmin-1)*newgrid%Agrif_dy
C      
      jgmin = 1 + nint((ymin-ygmin)/g%Agrif_dy)      
C
      jnewmax = 1 + int((min(ygmax,ynewmax)-ynewmin)/newgrid%Agrif_dy)
C
      ymax = newgrid%Agrif_y + (jnewmax-1)*newgrid%Agrif_dy
C      
      jgmax = 1 + nint((ymax-ygmin)/g%Agrif_dy)
C          
      j0 = jgmin
C      
      Do j = jnewmin,jnewmax
C        
        i0 = igmin
C          
        Do i = inewmin,inewmax
C
          newgrid%tabpoint2D(i,j) = max(newgrid%tabpoint2D(i,j),
     &                                             g%tabpoint2D(i0,j0))
C 
          i0 = i0 + int(newgrid%Agrif_dx/g%Agrif_dx)
C                  
        End Do
C          
        j0 = j0 + int(newgrid%Agrif_dy/g%Agrif_dy)
C          
      End Do                                        
C
      Return
C
C
      End Subroutine Agrif_Create_Tabpoints2D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Create_Tabpoints3D
C     **************************************************************************
C
      Recursive Subroutine Agrif_Create_Tabpoints3D(g,newgrid)
C
CCC   Description:
CCC   Copy on newgrid of points detected on the grid hierarchy pointed by g.   
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Type(Agrif_Grid),pointer   :: g,newgrid
C
C     Local variables      
      Integer :: i,j,k,i0,j0,k0
      Type(Agrif_Pgrid),pointer  :: parcours
      Real :: g_eps,newgrid_eps,eps
      Real :: xgmin,xgmax,ygmin,ygmax,zgmin,zgmax
      Real :: xnewmin,xnewmax,ynewmin,ynewmax,znewmin,znewmax
      Real :: xmin,xmax,ymin,ymax,zmin,zmax
      Integer :: igmin,inewmin,igmax,inewmax
      Integer :: jgmin,jnewmin,jgmax,jnewmax
      Integer :: kgmin,knewmin,kgmax,knewmax            
C
C     
      parcours => g % child_grids
C
      Do While(associated(parcours))
C
        Call Agrif_Create_Tabpoints3D(parcours%gr,newgrid)
C
        parcours => parcours % next
C
      End Do      
C
      g_eps = min(g%Agrif_dx,g%Agrif_dy,g%Agrif_dz)
C      
      newgrid_eps = min(newgrid%Agrif_dx,newgrid%Agrif_dy,
     &                  newgrid%Agrif_dz)
C
      eps = min(g_eps,newgrid_eps)/100.                  
C                   
      If ((newgrid%Agrif_dx .LT. (g%Agrif_dx-eps)) .OR.
     &    (newgrid%Agrif_dy .LT. (g%Agrif_dy-eps)) .OR.
     &    (newgrid%Agrif_dz .LT. (g%Agrif_dz-eps))) Return
C                   
      xgmin = g%Agrif_x
      xgmax = g%Agrif_x + g%nbx * g%Agrif_dx            
C 
      xnewmin = newgrid%Agrif_x
      xnewmax = newgrid%Agrif_x + newgrid%nbx * newgrid%Agrif_dx
C      
      ygmin = g%Agrif_y
      ygmax = g%Agrif_y + g%nby * g%Agrif_dy            
C 
      ynewmin = newgrid%Agrif_y
      ynewmax = newgrid%Agrif_y + newgrid%nby * newgrid%Agrif_dy
C      
      zgmin = g%Agrif_z
      zgmax = g%Agrif_z + g%nbz * g%Agrif_dz            
C 
      znewmin = newgrid%Agrif_z
      znewmax = newgrid%Agrif_z + newgrid%nbz * newgrid%Agrif_dz
C      
      If ((xgmax .LT. xnewmin) .OR.
     &        (ygmax .LT. ynewmin) .OR. (zgmax .LT. znewmin)) Return
C
      If ((xgmin .GT. xnewmax) .OR.
     &        (ygmin .GT. ynewmax) .OR. (zgmin .GT. znewmax)) Return
C           
      inewmin = 1 - floor(-(max(xgmin,xnewmin)-xnewmin)
     &                   /newgrid%Agrif_dx) 
C          
      xmin = newgrid%Agrif_x + (inewmin-1)*newgrid%Agrif_dx          
C
      igmin = 1 + nint((xmin-xgmin)/g%Agrif_dx)      
C
      inewmax = 1 + int((min(xgmax,xnewmax)-xnewmin)/newgrid%Agrif_dx)
C         
      xmax = newgrid%Agrif_x + (inewmax-1)*newgrid%Agrif_dx 
C          
      igmax = 1 + nint((xmax-xgmin)/g%Agrif_dx)
C
      jnewmin = 1 - floor(-(max(ygmin,ynewmin)-ynewmin)
     &                   /newgrid%Agrif_dy) 
C          
      ymin = newgrid%Agrif_y + (jnewmin-1)*newgrid%Agrif_dy
C      
      jgmin = 1 + nint((ymin-ygmin)/g%Agrif_dy)
C
      jnewmax = 1 + int((min(ygmax,ynewmax)-ynewmin)/newgrid%Agrif_dy)
C
      ymax = newgrid%Agrif_y + (jnewmax-1)*newgrid%Agrif_dy
C      
      jgmax = 1 + nint((ymax-ygmin)/g%Agrif_dy)
C      
      knewmin = 1 - floor(-(max(zgmin,znewmin)-znewmin)
     &                   /newgrid%Agrif_dz) 
C   
      zmin = newgrid%Agrif_z + (knewmin-1)*newgrid%Agrif_dz
C      
      kgmin = 1 + nint((zmin-zgmin)/g%Agrif_dz)      
C
      knewmax = 1 + int((min(zgmax,znewmax)-znewmin)/newgrid%Agrif_dz)
C      
      zmax = newgrid%Agrif_z + (knewmax-1)*newgrid%Agrif_dz
C      
      kgmax = 1 + nint((zmax-zgmin)/g%Agrif_dz)
C          
      k0 = kgmin
C      
      Do k = knewmin,knewmax
C      
        j0 = jgmin
C      
        Do j = jnewmin,jnewmax
C        
          i0 = igmin
C          
          Do i = inewmin,inewmax
C
            newgrid%tabpoint3D(i,j,k) = max(newgrid%tabpoint3D(i,j,k),
     &                                          g%tabpoint3D(i0,j0,k0))
C 
            i0 = i0 + int(newgrid%Agrif_dx/g%Agrif_dx)
C                  
          End Do
C          
          j0 = j0 + int(newgrid%Agrif_dy/g%Agrif_dy)
C          
        End Do
C        
        k0 = k0 + int(newgrid%Agrif_dz/g%Agrif_dz)
C        
      End Do                                        
C
      Return
C
C
      End Subroutine Agrif_Create_Tabpoints3D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_ClusterGrid1D
C     **************************************************************************      
C      
      Subroutine Agrif_ClusterGrid1D(g,coarsegrid)
C
CCC   Description:
CCC   Clustering on the grid pointed by g.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C                 
C     Pointer arguments    
      Type(AGRIF_grid),pointer       :: g        ! Pointer on the current grid
      Type(AGRIF_rectangle),pointer  :: coarsegrid
C
C     Local variables      
      type(Agrif_rectangle) :: newrect,zone
      integer :: i,l,nbrectangle,sx,sy
      integer :: bufferwidth,flagpoints
      integer,dimension(:),allocatable :: newflag
      integer :: m1,m2
C
C      
      bufferwidth = int(Agrif_Minwidth/5.)
C       
      sx = g % nbx + 1
C                  
      allocate(newflag(sx))
C
      newflag = 0      
C
      flagpoints=0 
C      
      If (bufferwidth>0) Then 
C            
          Do i = bufferwidth,sx-bufferwidth+1      
C            
            If (g%tabpoint1D(i) .eq. 1) Then
C                  
                m1 = i - bufferwidth + 1
                m2 = i + bufferwidth - 1                  
C                  
                flagpoints = flagpoints+1
C      
                newflag(m1:m2) = 1 
C            
             End If 
C            
          End Do 
C      
        Else 
C
          flagpoints = 1      
C      
          newflag = g % tabpoint1D
C      
      End If
C       
      If (flagpoints .eq. 0) Then      
C      
          deallocate(newflag) 
C            
          Return 
C
      End If 
C      
      newrect % imin = 1
      newrect % imax = sx
C      
      Call Agrif_Cluster1d(newflag,coarsegrid%childgrids,newrect) 
C      
      deallocate(newflag)
C     
C 
      End Subroutine Agrif_ClusterGrid1D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_ClusterGrid2D
C     **************************************************************************      
C      
      Subroutine Agrif_ClusterGrid2D(g,coarsegrid)
C
CCC   Description:
CCC   Clustering on the grid pointed by g.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C                 
C     Pointer arguments    
      Type(AGRIF_grid),pointer       :: g        ! Pointer on the current grid
      Type(AGRIF_rectangle),pointer  :: coarsegrid
C
C     Local variables      
      type(Agrif_rectangle) :: newrect,zone
      integer :: i,j,l,nbrectangle,sx,sy
      integer :: bufferwidth,flagpoints
      integer,dimension(:,:),allocatable :: newflag
      integer :: n1,n2,m1,m2
C
C      
      bufferwidth = int(Agrif_Minwidth/5.)
C       
      sx = g % nbx + 1
      sy = g % nby + 1
C                  
      allocate(newflag(sx,sy))
C
      newflag = 0      
C
      flagpoints = 0 
C      
      If (bufferwidth>0) Then 
C      
          Do j = bufferwidth,sy-bufferwidth+1
C            
            Do i = bufferwidth,sx-bufferwidth+1      
C            
              If (g % tabpoint2D(i,j) .eq. 1) Then
C                  
                  n1 = j - bufferwidth + 1
                  n2 = j + bufferwidth - 1
                  m1 = i - bufferwidth + 1
                  m2 = i + bufferwidth - 1                  
C                  
                  flagpoints = flagpoints + 1
C      
                  newflag(m1:m2,n1:n2)=1 
C            
                End If 
C            
              End Do 
C            
            End Do
C      
        Else
C
          flagpoints = 1       
C      
          newflag = g % tabpoint2D
C      
      End If
C       
      If (flagpoints .eq. 0) Then       
C      
          deallocate(newflag) 
C            
          Return 
C
      End If       
C      
      newrect % imin = 1
      newrect % imax = sx
      newrect % jmin = 1
      newrect % jmax = sy
C      
      Call Agrif_Cluster2d(newflag,coarsegrid%childgrids,newrect) 
C      
      deallocate(newflag)
C  
C    
      End Subroutine Agrif_ClusterGrid2D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_ClusterGrid3D
C     **************************************************************************      
C      
      Subroutine Agrif_ClusterGrid3D(g,coarsegrid)
C
CCC   Description:
CCC   Clustering on the grid pointed by g.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C                 
C     Pointer arguments    
      Type(AGRIF_grid),pointer   :: g        ! Pointer on the current grid
      Type(AGRIF_rectangle),pointer   :: coarsegrid
C
C     Local variables      
      type(Agrif_rectangle) :: newrect,zone
      integer :: i,j,k,l,nbrectangle,sx,sy,sz
      integer :: bufferwidth,flagpoints
      integer,dimension(:,:,:),allocatable :: newflag
      integer :: n1,n2,m1,m2,o1,o2
C
C      
      bufferwidth = int(Agrif_Minwidth/5.)
C       
      sx = g % nbx + 1
      sy = g % nby + 1
      sz = g % nbz + 1
C                  
      allocate(newflag(sx,sy,sz))
C
      newflag = 0      
C
      flagpoints=0 
C      
      If (bufferwidth > 0) Then 
C      
          Do k = bufferwidth,sz-bufferwidth+1
C
            Do j = bufferwidth,sy-bufferwidth+1
C            
              Do i = bufferwidth,sx-bufferwidth+1      
C            
                If (g % tabpoint3D(i,j,k) .eq. 1) Then
C                  
                    n1 = j - bufferwidth + 1
                    n2 = j + bufferwidth - 1
                    m1 = i - bufferwidth + 1
                    m2 = i + bufferwidth - 1
                    o1 = k - bufferwidth + 1
                    o2 = k + bufferwidth - 1                  
C                  
                    flagpoints = flagpoints + 1
C      
                    newflag(m1:m2,n1:n2,o1:o2) = 1 
C            
                End If 
C            
              End Do 
C            
            End Do
C            
          End Do
C      
        Else 
C
          flagpoints = 1      
C      
          newflag = g % tabpoint3D
C      
      End If
C       
      If (flagpoints .eq. 0) Then       
C      
          deallocate(newflag) 
C            
          Return 
C
      End If
C      
      newrect % imin = 1
      newrect % imax = sx
      newrect % jmin = 1
      newrect % jmax = sy
      newrect % kmin = 1
      newrect % kmax = sz
C
      Call Agrif_Cluster3d(newflag,coarsegrid%childgrids,newrect)
C      
      deallocate(newflag) 
C
C      
      End Subroutine Agrif_ClusterGrid3D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Cluster1D
C     **************************************************************************      
C
      Recursive subroutine Agrif_Cluster1d(flag,boxlib,oldB) 
C
CCC   Description:
CCC   Clustering on the grid pointed by oldB.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C      
C     Arguments
      Type(Agrif_rectangle) ::  oldB
      Integer,dimension(oldB%imin:oldB%imax) :: flag
      Type(Agrif_lrectangle),pointer :: boxlib      
C
C     Local variables      
      Type(Agrif_lrectangle),pointer :: tempbox,parcbox,parcbox2
      integer :: i
      integer,dimension(oldB%imin:oldB%imax) :: i_sig       
      integer :: ipu,ipl,istr,islice
      REAL :: cureff
      REAL :: neweff
      Integer :: nbpoints,nbpointsflag      
      type(Agrif_rectangle) :: newB,newB2
      Logical :: add
C
C          
      If ((SIZE(i_sig).lt.Agrif_Minwidth)) then
C     
          return
C        
      End if
C      
      i_sig = flag      
C                    
      ipl = oldB%imin
      ipu = oldB%imax
C       
      Call Agrif_Clusterprune(i_sig,ipl,ipu)      
C      
      If (.not.((ipl.eq.oldB%imin).and.     
     &              (ipu.eq.oldB%imax))) Then 
C      
          newB%imin = ipl
          newB%imax = ipu
C        
          nbpoints=SUM(flag(newB%imin:newB%imax))   
C   
          neweff = real(nbpoints)/SIZE(flag(newB%imin:newB%imax))
C     
          If (nbpoints.GT.0) Then
C        
              If ((neweff .GT .Agrif_efficiency)) Then
C           
                  Call Agrif_Add_Rectangle(newB,boxlib)
C           
                  Return
C              
                Else
C            
                  tempbox => boxlib
C           
                  newB2 = newB
C           
                  Call Agrif_Cluster1d(flag(newB%imin:newB%imax),
     &                      boxlib,newB)
C     
C                 Compute new efficiency
C
                  cureff = neweff
C            
                  parcbox2 => boxlib
C
                  nbpoints = 0
C
                  nbpointsflag = 0
C            
                  Do While (associated(parcbox2))
C              
                    If (associated(parcbox2,tempbox)) Exit
C            
                    newB = parcbox2%r
C     
                    nbpointsflag = nbpointsflag + 
     &                             SUM(flag(newB%imin:newB%imax))
C   
                    nbpoints = nbpoints + 
     &                         SIZE(flag(newB%imin:newB%imax))   
C              
                        parcbox2 => parcbox2%next  
C            
                  End Do  
C            
                  If  (real(nbpointsflag)/real(nbpoints)
     &                 .LT.(1.15*cureff)) Then
C            
                      parcbox2 => boxlib            
C            
                      Do While (associated(parcbox2))
C              
                        If (associated(parcbox2,tempbox)) Exit
C            
                        deallocate(parcbox2%r)
C              
                        parcbox2=>parcbox2%next             
C
                      End Do      
C                      
                      boxlib => tempbox
C            
                      Call Agrif_Add_Rectangle(newB2,boxlib)
C            
                      Return
C
                  End If
C            
              End If         
C      
          End If
C      
          Return
C      
      End If 
C       
      istr = oldB%imax   
C         
      islice = oldB%imin      
C      
      Call Agrif_Clusterslice(i_sig,islice,istr)
C      
      If ((islice).eq.-1) Then 
C      
          Call Agrif_Add_Rectangle(oldB,boxlib) 
C        
          Return 
C      
      End If 
C       
      nullify(tempbox)
C      
      tempbox => boxlib
C      
      cureff = SIZE(flag(oldB%imin:oldB%imax))
C      
      Nullify(parcbox)            
C            
      newB%imin = oldB%imin
      newB%imax = islice
C
      Call Agrif_Add_Rectangle(newB,parcbox)
C            
      newB%imin = islice+1
      newB%imax = oldB%imax
        
      Call Agrif_Add_Rectangle(newB,parcbox)      
C      
      Do While (associated(parcbox))
C      
        newB = parcbox%r            
C    
        nbpoints = SUM(flag(newB%imin:newB%imax))   
C      
        neweff = real(nbpoints)/SIZE(flag(newB%imin:newB%imax))
C     
        If (nbpoints.GT.0) Then
C      
            If ((neweff .GT .Agrif_efficiency)) Then
C          
                Call Agrif_Add_Rectangle(newB,boxlib)
C         
              Else
C           
                tempbox => boxlib
C           
                newB2 = newB
C           
                Call Agrif_Cluster1d(flag(newB%imin:newB%imax),
     &                      boxlib,newB)
C
C               Compute new efficiency
C
                cureff = neweff
C            
                parcbox2 => boxlib
C  
                nbpoints = 0
C            
                nbpointsflag = 0
C
                Do While (associated(parcbox2))
C              
                  If (associated(parcbox2,tempbox)) Exit
C            
                  newB = parcbox2%r
C            
                  nbpointsflag = nbpointsflag +
     &                         SUM(flag(newB%imin:newB%imax))
C
                  nbpoints = nbpoints + SIZE(flag(newB%imin:newB%imax))   
C              
                  parcbox2 => parcbox2%next  
C            
                End Do   
C            
                If  (real(nbpointsflag)/real(nbpoints)
     &               .LT.(1.15*cureff)) Then
C            
                    parcbox2 => boxlib            
C            
                    Do While (associated(parcbox2))
C              
                      If (associated(parcbox2,tempbox)) Exit
C            
                      deallocate(parcbox2%r)
C              
                      parcbox2 => parcbox2%next             
C            
                    End Do
C                              
                    boxlib => tempbox
C            
                    Call Agrif_Add_Rectangle(newB2,boxlib)
C          
                End If
C         
            End If
C                   
        End If
C        
        parcbox => parcbox%next
C       
      End Do
C      
C      
      Return
C      
C
      End Subroutine Agrif_Cluster1d 
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Cluster2D
C     **************************************************************************      
C
      Recursive subroutine Agrif_Cluster2d(flag,boxlib,oldB) 
C
CCC   Description:
CCC   Clustering on the grid pointed by oldB.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Arguments  
      Type(Agrif_rectangle) ::  oldB    
      Integer,dimension(oldB%imin:oldB%imax,
     &                  oldB%jmin:oldB%jmax) :: flag
      Type(Agrif_lrectangle),pointer :: boxlib      
C
C     Local variables      
      Type(Agrif_lrectangle),pointer :: tempbox,parcbox,parcbox2       
      integer :: i,j
      integer,dimension(oldB%imin:oldB%imax) :: i_sig 
      integer,dimension(oldB%jmin:oldB%jmax) :: j_sig       
      integer :: ipu,ipl,istr,islice
      Integer :: jpu,jpl,jstr,jslice
      REAL :: cureff
      REAL :: neweff
      Integer :: nbpoints,nbpointsflag      
      type(Agrif_rectangle) :: newB,newB2
      Logical :: add
C
C          
      If ((SIZE(i_sig).lt.Agrif_Minwidth)
     &       .or.(SIZE(j_sig).lt.Agrif_Minwidth)) then
        Return
C        
      End if
C      
      Do i = oldB%imin,oldB%imax
C  
        i_sig(i) = SUM(flag(i,oldB%jmin:oldB%jmax))
C      
      End Do
C      
      Do j = oldB%jmin,oldB%jmax
C      
        j_sig(j) = SUM(flag(oldB%imin:oldB%imax,j))
C      
      End Do      
C                    
      ipl = oldB%imin
      ipu = oldB%imax
      jpl = oldB%jmin
      jpu = oldB%jmax
              
C       
      Call Agrif_Clusterprune(i_sig,ipl,ipu) 
      Call Agrif_Clusterprune(j_sig,jpl,jpu)      
C      
      If (.not.((ipl.eq.oldB%imin).and.
     &              (jpl.eq.oldB%jmin).and.     
     &              (ipu.eq.oldB%imax).and.
     &              (jpu.eq.oldB%jmax))) Then 
C      
          newB%imin = ipl
          newB%imax = ipu
          newB%jmin = jpl
          newB%jmax = jpu      
C        
          nbpoints = SUM(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax))   
C   
          neweff = real(nbpoints)/SIZE(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax))
C     
          If (nbpoints.GT.0) Then
C        
              If ((neweff .GT .Agrif_efficiency)) Then
C           
                  Call Agrif_Add_Rectangle(newB,boxlib)
C           
                  Return
C              
                Else
C            
                  tempbox => boxlib
C           
                  newB2 = newB
C           
                  Call Agrif_Cluster2d(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax),
     &                      boxlib,newB)
C     
C                 Compute new efficiency
C
                  cureff = neweff
C            
                  parcbox2 => boxlib
C
                  nbpoints = 0
C
                  nbpointsflag = 0
C            
                  Do While (associated(parcbox2))
C              
                    If (associated(parcbox2,tempbox)) Exit
C            
                    newB = parcbox2%r
C     
                    nbpointsflag = nbpointsflag 
     &                           + SUM(flag(newB%imin:newB%imax,
     &                                      newB%jmin:newB%jmax))
C   
                    nbpoints = nbpoints 
     &                       + SIZE(flag(newB%imin:newB%imax,
     &                                   newB%jmin:newB%jmax))   
C              
                    parcbox2 => parcbox2%next  
C            
                  End Do  
C            
                  If  (real(nbpointsflag)/real(nbpoints)
     &                 .LT.(1.15*cureff)) Then
C            
                      parcbox2 => boxlib            
C            
                      Do While (associated(parcbox2))
C              
                        If (associated(parcbox2,tempbox)) Exit
C            
                        deallocate(parcbox2%r)
C              
                        parcbox2 => parcbox2%next             
C
                      End Do      
C                      
                      boxlib => tempbox
C            
                      Call Agrif_Add_Rectangle(newB2,boxlib)
C            
                      Return
C
                  End If
C            
              End If         
C      
          End If
C      
          Return
C      
      End If 
C       
      istr = oldB%imax
      jstr = oldB%jmax      
      islice = oldB%imin
      jslice = oldB%jmin      
C      
      Call Agrif_Clusterslice(i_sig,islice,istr) 
      Call Agrif_Clusterslice(j_sig,jslice,jstr) 
C      
      If ((islice+jslice).eq.-2) Then 
C      
          Call Agrif_Add_Rectangle(oldB,boxlib) 
C        
          Return 
C      
      End If 
C       
      nullify(tempbox)
C      
      tempbox => boxlib
C      
      cureff = SIZE(flag(oldB%imin:oldB%imax,oldB%jmin:oldB%jmax))
C      
      Nullify(parcbox)
      
      If (istr.EQ.Max(istr,jstr)) Then
C      
          newB%jmin = oldB%jmin            
          newB%jmax = oldB%jmax            
C            
          newB%imin = oldB%imin
          newB%imax = islice
C
        Call Agrif_Add_Rectangle(newB,parcbox)                   
C            
          newB%imin = islice+1
          newB%imax = oldB%imax
C        
        Call Agrif_Add_Rectangle(newB,parcbox)
C            
C      
        Else If (jstr.EQ.Max(istr,jstr)) Then
C          
          newB%imin = oldB%imin
          newB%imax = oldB%imax            
C             
          newB%jmin = oldB%jmin        
          newB%jmax = jslice
C            
      Call Agrif_Add_Rectangle(newB,parcbox)
C              
          newB%jmin = jslice+1
          newB%jmax = oldB%jmax
C
      Call Agrif_Add_Rectangle(newB,parcbox)            
C
      End If
C      
      Do While (associated(parcbox))
C      
        newB = parcbox%r            
C    
        nbpoints = SUM(flag(newB%imin:newB%imax,newB%jmin:newB%jmax))   
C      
        neweff = real(nbpoints)/SIZE(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax))
C     
        If (nbpoints .GT. 0) Then
C      
            If ((neweff .GT .Agrif_efficiency)) Then
C          
                Call Agrif_Add_Rectangle(newB,boxlib)
C         
              Else
C           
                tempbox => boxlib
C           
                newB2 = newB
C           
                Call Agrif_Cluster2d(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax),
     &                      boxlib,newB)
C
C               Compute new efficiency
C
                cureff = neweff
C            
                parcbox2 => boxlib
C  
                nbpoints = 0
C            
                nbpointsflag = 0
C
                Do While (associated(parcbox2))
C              
                  If (associated(parcbox2,tempbox)) Exit
C            
                  newB = parcbox2%r
C            
                  nbpointsflag = nbpointsflag +
     &                         SUM(flag(newB%imin:newB%imax,
     &                                  newB%jmin:newB%jmax))
C
                  nbpoints = nbpoints 
     &                     + SIZE(flag(newB%imin:newB%imax,
     &                                 newB%jmin:newB%jmax))   
C              
                  parcbox2 => parcbox2%next  
C            
                End Do   
C            
                If  (real(nbpointsflag)/real(nbpoints)
     &               .LT.(1.15*cureff)) Then
C            
                    parcbox2 => boxlib            
C            
                    Do While (associated(parcbox2))
C              
                      If (associated(parcbox2,tempbox)) Exit
C            
                      deallocate(parcbox2%r)
C              
                      parcbox2 => parcbox2%next             
C            
                    End Do
C                              
                    boxlib => tempbox
C            
                    Call Agrif_Add_Rectangle(newB2,boxlib)
C          
                End If
C         
            End If
C                   
        End If
C        
        parcbox => parcbox%next
C       
      End Do
C      
C      
      Return
C      
      End Subroutine Agrif_Cluster2d 
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Cluster3D
C     **************************************************************************      
C
      Recursive subroutine Agrif_Cluster3d(flag,boxlib,oldB) 
C
CCC   Description:
CCC   Clustering on the grid pointed by oldB.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C      
C     Arguments
      Type(Agrif_rectangle) ::  oldB
      Integer,dimension(oldB%imin:oldB%imax,oldB%jmin:oldB%jmax,
     &            oldB%kmin:oldB%kmax) :: flag
      Type(Agrif_lrectangle),pointer :: boxlib      
C
C     Local variables      
      Type(Agrif_lrectangle),pointer :: tempbox,parcbox,parcbox2
      integer :: i,j,k
      integer,dimension(oldB%imin:oldB%imax) :: i_sig 
      integer,dimension(oldB%jmin:oldB%jmax) :: j_sig 
      integer,dimension(oldB%kmin:oldB%kmax) :: k_sig       
      integer :: ipu,ipl,istr,islice
      Integer :: jpu,jpl,jstr,jslice
      integer :: kpu,kpl,kstr,kslice 
      REAL :: cureff
      REAL :: neweff
      Integer :: nbpoints,nbpointsflag      
      type(Agrif_rectangle) :: newB,newB2
      Logical :: add
C
C          
      If ((SIZE(i_sig).lt.Agrif_Minwidth)
     &      .or.(SIZE(j_sig).lt.Agrif_Minwidth).or.
     &       (SIZE(k_sig).lt.Agrif_Minwidth)) then
        return
C        
      End if
C      
      Do i = oldB%imin,oldB%imax
C  
        i_sig(i) = SUM(flag(i,oldB%jmin:oldB%jmax,oldB%kmin:oldB%kmax))
C      
      End Do
C      
      Do j = oldB%jmin,oldB%jmax
C      
        j_sig(j) = SUM(flag(oldB%imin:oldB%imax,j,oldB%kmin:oldB%kmax))
C      
      End Do
C      
      Do k = oldB%kmin,oldB%kmax
C      
        k_sig(k) = SUM(flag(oldB%imin:oldB%imax,oldB%jmin:oldB%jmax,k))
C      
      End Do      
C                    
      ipl = oldB%imin
      ipu = oldB%imax
      jpl = oldB%jmin
      jpu = oldB%jmax
      kpl = oldB%kmin
      kpu = oldB%kmax              
C       
      Call Agrif_Clusterprune(i_sig,ipl,ipu) 
      Call Agrif_Clusterprune(j_sig,jpl,jpu) 
      Call Agrif_Clusterprune(k_sig,kpl,kpu)      
C      
      If (.not.((ipl.eq.oldB%imin).and.
     &              (jpl.eq.oldB%jmin).and.
     &              (kpl.eq.oldB%kmin).and.     
     &              (ipu.eq.oldB%imax).and.
     &              (jpu.eq.oldB%jmax).and.
     &              (kpu.eq.oldB%kmax))) Then 
C      
          newB%imin = ipl
          newB%imax = ipu
          newB%jmin = jpl
          newB%jmax = jpu
          newB%kmin = kpl
          newB%kmax = kpu      
C        
          nbpoints = SUM(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))   
C   
          neweff = real(nbpoints)/SIZE(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))
C     
          If (nbpoints .GT. 0) Then
C        
              If ((neweff .GT .Agrif_efficiency)) Then
C           
                  Call Agrif_Add_Rectangle(newB,boxlib)
C           
                  Return
C              
                Else
C            
                  tempbox => boxlib
C           
                  newB2 = newB
C           
                  Call Agrif_Cluster3d(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax),
     &                      boxlib,newB)
C     
C                 Compute new efficiency
C
                  cureff = neweff
C            
                  parcbox2 => boxlib
C
                  nbpoints = 0
C
                  nbpointsflag = 0
C            
                  Do While (associated(parcbox2))
C              
                    If (associated(parcbox2,tempbox)) Exit
C            
                    newB = parcbox2%r
C     
                    nbpointsflag = nbpointsflag+
     &                           SUM(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))
C   
                    nbpoints = nbpoints+SIZE(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))   
C              
                    parcbox2 => parcbox2%next  
C            
                  End Do  
C            
                  If  (real(nbpointsflag)/real(nbpoints)
     &                 .LT.(1.15*cureff)) Then
C            
                      parcbox2 => boxlib            
C            
                      Do While (associated(parcbox2))
C              
                        If (associated(parcbox2,tempbox)) Exit
C            
                        deallocate(parcbox2%r)
C              
                        parcbox2=>parcbox2%next             
C
                      End Do      
C                      
                      boxlib => tempbox
C            
                      Call Agrif_Add_Rectangle(newB2,boxlib)
C            
                      Return
C
                  End If
C            
              End If         
C      
          End If
C      
          Return
C      
      End If 
C       
      istr = oldB%imax
      jstr = oldB%jmax
      kstr = oldB%kmax      
      islice = oldB%imin
      jslice = oldB%jmin      
      kslice = oldB%kmin      
C      
      Call Agrif_Clusterslice(i_sig,islice,istr) 
      Call Agrif_Clusterslice(j_sig,jslice,jstr) 
      Call Agrif_Clusterslice(k_sig,kslice,kstr) 
C      
      If ((islice+jslice+kslice) .eq. -3) Then 
C      
          Call Agrif_Add_Rectangle(oldB,boxlib) 
C        
          Return 
C      
      End If 
C       
      nullify(tempbox)
C      
      tempbox => boxlib
C      
      cureff = SIZE(flag(oldB%imin:oldB%imax,oldB%jmin:oldB%jmax,
     &               oldB%kmin:oldB%kmax))
C      
      Nullify(parcbox)
      
      If (istr .EQ. Max(istr,jstr,kstr)) Then
C      
          newB%jmin = oldB%jmin            
          newB%jmax = oldB%jmax
C            
          newB%kmin = oldB%kmin
          newB%kmax = oldB%kmax            
C            
          newB%imin = oldB%imin
          newB%imax = islice
C
        Call Agrif_Add_Rectangle(newB,parcbox)                   
c          CALL Agrif_Cluster3d(flag,boxlib,newB)
C            
          newB%imin = islice + 1
          newB%imax = oldB%imax
        
        Call Agrif_Add_Rectangle(newB,parcbox)
C            
c          CALL Agrif_Cluster3d(flag,boxlib,newB)      
C      
        Else If (jstr .EQ. Max(istr,jstr,kstr)) Then
C          
          newB%imin = oldB%imin
          newB%imax = oldB%imax
C            
          newB%kmin = oldB%kmin
          newB%kmax = oldB%kmax            
C             
          newB%jmin = oldB%jmin        
          newB%jmax = jslice
C            
          Call Agrif_Add_Rectangle(newB,parcbox)
c          CALL Agrif_Cluster3d(flag,boxlib,newB)            
C              
          newB%jmin = jslice + 1
          newB%jmax = oldB%jmax
C
          Call Agrif_Add_Rectangle(newB,parcbox)            
c          CALL Agrif_Cluster3d(flag,boxlib,newB)
C      
        Else
C      
          newB%imin = oldB%imin
          newB%imax = oldB%imax
C                    
          newB%jmin = oldB%jmin
          newB%jmax = oldB%jmax
C            
          newB%kmin = oldB%kmin
          newB%kmax = kslice
C
          Call Agrif_Add_Rectangle(newB,parcbox)                   
c          CALL Agrif_Cluster3d(flag,boxlib,newB)
C          
          newB%kmin = kslice + 1
          newB%kmax = oldB%kmax
C      
          Call Agrif_Add_Rectangle(newB,parcbox)      
c          CALL Agrif_Cluster3d(flag,boxlib,newB)      
C
      End If
C      
      Do While (associated(parcbox))
C      
        newB = parcbox%r            
C    
        nbpoints = SUM(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))   
C    
        neweff = real(nbpoints)/SIZE(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))
C     

        If (nbpoints .GT. 0) Then
C      
            If ((neweff .GT .Agrif_efficiency)) Then
C          
                Call Agrif_Add_Rectangle(newB,boxlib)
C         
              Else
C           
                tempbox => boxlib
C           
                newB2 = newB
C           
                Call Agrif_Cluster3d(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax),
     &                      boxlib,newB)
C
C               Compute new efficiency
C
                cureff = neweff
C            
                parcbox2 => boxlib
C  
                nbpoints = 0
C            
                nbpointsflag = 0
C
                Do While (associated(parcbox2))
C              
                  If (associated(parcbox2,tempbox)) Exit
C            
                  newB = parcbox2%r
C            
                  nbpointsflag = nbpointsflag +
     &                         SUM(flag(newB%imin:newB%imax,
     &                       newB%jmin:newB%jmax,newB%kmin:newB%kmax))
C
                  nbpoints = nbpoints + SIZE(flag(newB%imin:newB%imax,
     &                      newB%jmin:newB%jmax,newB%kmin:newB%kmax))   
C              
                  parcbox2 => parcbox2%next  
C            
                End Do   
C            
                If  (real(nbpointsflag)/real(nbpoints)
     &               .LT.(1.15*cureff)) Then
C            
                    parcbox2 => boxlib            
C            
                    Do While (associated(parcbox2))
C              
                      If (associated(parcbox2,tempbox)) Exit
C            
                      deallocate(parcbox2%r)
C              
                      parcbox2 => parcbox2%next             
C            
                    End Do
C                              
                    boxlib => tempbox
C            
                    Call Agrif_Add_Rectangle(newB2,boxlib)
C          
                End If
C         
            End If
C                   
        End If
C        
        parcbox => parcbox%next
C       
      End Do
C      
C      
      Return
C      
      End Subroutine Agrif_Cluster3d  
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Clusterslice
C     **************************************************************************
C                           
      Subroutine Agrif_Clusterslice(sig,slice,str) 
C
C
CCC   Description:
CCC   
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Arguments 
      Integer :: slice,str
      Integer,Dimension(slice:str) :: sig 
C
C     Local variables      
      Integer :: ideb,ifin
      Integer :: i,t,a,di,ts 
      Integer,Dimension(slice:str) :: lap 
C
C    
      ideb = slice
      ifin = str
C    
      If (SIZE(sig) <= 2*Agrif_Minwidth) Then
C           
          str = -1 
C        
          slice = -1 
C        
          Return
C         
      End If
C      
      t = -1 
C
      a = -1 
C
      Do i = ideb+Agrif_Minwidth-1,ifin-Agrif_Minwidth
C      
          If (sig(i) .EQ. 0) Then
C         
              If ((i-ideb) < (ifin-i)) Then
C             
                  di = i - ideb
C                
                Else 
C            
                  di = ifin - i 
C                
              END IF
C            
              If (di > t) Then
C             
                a = i 
C              
                t = di 
C            
              End If
C            
         End If
C       
      End Do
C      
      If (a .NE. (-1)) Then
C       
          slice = a 
C        
          str = t 
C        
          Return
C         
      End If
C      
      t = -1 
C      
      a = -1 
C      
      Do i = ideb+1,ifin-1 
C        
        lap(i) = sig(i+1) + sig(i-1) - 2*sig(i) 
C      
      End Do
C      
      Do i = ideb + Agrif_Minwidth-1,ifin-Agrif_Minwidth
C      
        If ((lap(i+1)*lap(i)) .LE. 0) Then
C       
            ts = ABS(lap(i+1) - lap(i)) 
C          
            If (ts > t) Then
C           
                t = ts 
C              
                a = i
C             
            End If
C          
       End If
C       
      End Do
C      
      If (a .EQ. (ideb + Agrif_Minwidth - 1)) Then
C      
          a = -1
C        
          t = -1
C        
      End If
C      
      slice = a 
C      
      str = t 
C
C      
      End Subroutine Agrif_Clusterslice       
C
C
C    
C     **************************************************************************
CCC   Subroutine Agrif_Clusterprune
C     **************************************************************************      
C      
      Subroutine Agrif_Clusterprune(sig,pl,pu) 
C
C
CCC   Description:
CCC   
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C            
C     Arguments
      integer :: pl,pu 
      integer,dimension(pl:pu) :: sig
C
C     Local variables      
      integer :: ideb,ifin       
      integer :: diff,addl,addu,udist,ldist  
C
C      
      ideb = pl
C      
      ifin = pu
C      
      If (SIZE(sig) <= Agrif_Minwidth) Then
C       
          return 
C      
      End If 
C       
      Do While ((sig(pl) .eq. 0) .and. (pl < ifin)) 
C      
        pl = pl + 1 
C      
      End Do 
C       
      Do While ((sig(pu) .eq. 0) .and. (pu > ideb)) 
C
        pu = pu - 1 
C      
      End Do 
C       
      If ((pu-pl) < Agrif_Minwidth) Then
C       
          diff = Agrif_Minwidth - (pu - pl + 1) 
C      
          udist = ifin - pu 
C      
          ldist = pl - ideb
C      
          addl = diff / 2 
C      
          addu = diff - addl 
C      
          If (addu > udist) Then
C         
              addu = udist 
C            
              addl = diff - addu 
C      
          End If
C         
          If (addl > ldist) Then
C         
              addl = ldist 
C            
              addu = diff - addl
C      
          End If
C         
          pu = pu + addu 
C      
          pl = pl - addl         
C      
      End If 
C       
C
      End Subroutine Agrif_Clusterprune
C
C
C              
C     **************************************************************************                      
CCC   Subroutine Agrif_Add_Rectangle 
C     **************************************************************************              
C        
      Subroutine Agrif_Add_Rectangle(R,LR)
C
CCC   Description:
CCC   Subroutine to add the Agrif_Rectangle R in a list managed by LR.             
C
C     Declarations:
C
      Implicit none 
C  
C     Arguments               
      Type(AGRIF_rectangle)           :: R  
      Type(AGRIF_lrectangle), Pointer :: LR
C
C     Local variable        
      Type(AGRIF_lrectangle), Pointer :: newrect  
C
C        
      allocate(newrect)
C      
      allocate(newrect % r)
C
      newrect % r = R
C      
cc      newrect%r%israfx = '.true.'      
cc      newrect%r%israfy = '.true.'      
cc      newrect%r%israfz = '.true.'
C
      newrect % r % spacerefx = Agrif_Coeffrefx
C
      newrect % r % timerefx = Agrif_Coeffreftx
C
      If (Agrif_Probdim >= 2) Then
C      
          newrect % r % spacerefy = Agrif_Coeffrefy
C      
          newrect % r % timerefy = Agrif_Coeffrefty
C
      End If
C      
      If (Agrif_Probdim == 3) Then
C      
          newrect % r % spacerefz = Agrif_Coeffrefz
C      
          newrect % r % timerefz = Agrif_Coeffreftz
C
      End If
C              
      newrect % r % number = -1            
C      
      Nullify(newrect % r % childgrids)
C      
      newrect % next => LR  
C      
      LR => newrect  
C
C      
      End Subroutine Agrif_Add_Rectangle   
C  
C
C  
C     **************************************************************************  
CCC   Subroutine Agrif_Read_Fixed_Grids  
C     **************************************************************************        
C      
      Recursive Subroutine Agrif_Read_Fixed_Grids(coarsegrid,j,nunit)
C
CCC   Description:
CCC   Subroutine to create the grid hierarchy from the reading of the 
CCC   AGRIF_FixedGrids.in file.
C
CC    Method:
CC    Recursive subroutine and creation of a first grid hierarchy from the
CC    reading of the AGRIF_FixedGrids.in file.        
C
C     Declarations:
C
      Implicit none
C
C     Pointer argument      
      Type(AGRIF_rectangle), Pointer   :: coarsegrid ! Pointer on the first grid 
                                                     ! of the grid hierarchy
C                                    
C     Scalar arguments                                    
      Integer                          :: j          ! Number of the new grid
      Integer                          :: nunit      ! unit associated with the file
C      
C     Local variables      
      Type(AGRIF_rectangle)            :: newrect    ! Pointer on a new grid   
      Type(AGRIF_lrectangle), Pointer  :: parcours   ! Pointer for the recursive procedure
      Type(AGRIF_lrectangle), Pointer  :: newlrect
      Type(AGRIF_lrectangle), Pointer  :: end_list       
      Integer                          :: i          ! for each child grid
      Integer                          :: nb_grids   ! Number of child grids       
C          
C
      Nullify(newrect%childgrids)
C      
C     Reading of the number of child grids        
      read(nunit,*) nb_grids
C      
C     coarsegrid%nbgridchild = nb_grids
C      
      allocate(end_list)
C
      nullify(end_list % r)
C
      nullify(end_list % next)
C      
      coarsegrid % childgrids => end_list
C      
C
C     Reading of imin,imax,jmin,jmax,kmin,kmax, and space and time refinement factors for each 
C     child grid.
C     Creation and addition of the new grid to the grid hierarchy. 
C
      Do i = 1,nb_grids
C      
        allocate(newlrect)      
C      
C       Number of the grid
        newrect % number = j   
C
        If (Agrif_Probdim == 3) then  
C            
            read(nunit,*) newrect % imin, newrect % imax,
     &                    newrect % jmin, newrect % jmax,
     &                    newrect % kmin, newrect % kmax,
     &                    newrect % spacerefx,newrect % spacerefy,
     &                    newrect % spacerefz,
     &                    newrect % timerefx,newrect % timerefy,
     &                    newrect % timerefz             
C
          Else if (Agrif_Probdim == 2) then            
C
           read(nunit,*) newrect % imin,newrect % imax,
     &	                  newrect % jmin,newrect % jmax,
     &                    newrect % spacerefx,newrect % spacerefy,
     &                    newrect % timerefx,newrect % timerefy
C
C
          Else if (Agrif_Probdim == 1) then
C
            read(nunit,*) newrect % imin, newrect % imax,
     &                    newrect % spacerefx,
     &                    newrect % timerefx      
C
        End If
C
C
C       Addition to the grid hierarchy
C
        nullify(newrect % childgrids)
C            
        j = j + 1
C            
        Allocate(newlrect%r)
C
        newlrect % r = newrect 
C      
        nullify(newlrect % next)
C      
        end_list % next => newlrect
C      
        end_list => end_list % next
C
      End Do
C      
      coarsegrid % childgrids => coarsegrid % childgrids % next
C      
      parcours => coarsegrid % childgrids
C
C     Recursive operation to create the grid hierarchy branch by branch
      Do while (associated(parcours))
C      
        call Agrif_Read_Fixed_Grids
     &           (parcours % r,j,nunit)
C      
        parcours => parcours % next
C
      End do
C      
C
      End Subroutine Agrif_Read_Fixed_Grids        
C        
C 
C
C     **************************************************************************  
CCC   Subroutine Agrif_Create_Grids  
C     **************************************************************************  
C
      Recursive Subroutine Agrif_Create_Grids(g,coarsegrid)
C
CCC   Description:
CCC   Subroutine to create the grid hierarchy (g) from the one created with the
CCC   Agrif_Read_Fixed_Grids or Agrif_Cluster_All procedures (coarsegrid).
C
CC    Method:
CC    Recursive subroutine.        
C
C     Declarations:
C
      Implicit none  
C
C     Pointer arguments        
      Type(AGRIF_grid), Pointer       :: g          ! Pointer on the root coarse
                                                    ! grid
      Type(AGRIF_rectangle), Pointer  :: coarsegrid ! Pointer on the root coarse 
                                                    ! grid of the grid hierarchy
                                                    ! created with the
                                                    ! Agrif_Read_Fixed_Grids 
                                                    ! subroutine                                                                                                                            
C      
C     Local pointers      
      Type(Agrif_grid), Pointer       :: newgrid    ! New grid
      Type(Agrif_pgrid), Pointer      :: newpgrid
      Type(Agrif_pgrid), Pointer      :: parcours2
      Type(Agrif_lrectangle), Pointer :: parcours
      Type(Agrif_pgrid), Pointer      :: end_list
      Type(Agrif_pgrid), Pointer      :: parcours3
C
C     Local scalars      
      Integer                         :: irhox      ! Space refinement factors
      Integer                         :: irhoy      
      Integer                         :: irhoz      
      Integer                         :: res        ! Time refinement factor
      Logical                         :: nullliste
      
C
C
      parcours3 => g % child_grids
C     
      If (associated(parcours3)) Then
C     
          Do While (associated(parcours3 % next))
C          
            parcours3 => parcours3 % next
C        
          End Do
C      
          end_list => parcours3
C      
          nullliste=.FALSE.
C      
        Else
C      
          allocate(end_list)
C      
          nullify(end_list % gr)
C      
          nullify(end_list % next)      
C      
          g % child_grids => end_list 
C      
          parcours3 => end_list      
C             
          nullliste=.TRUE.
C      
      End If
C  
C      
      parcours => coarsegrid % childgrids
C      
C     Creation of the grid hierarchy from the one created by using the 
C     Agrif_Read_Fixed_Grids subroutine 
      Do while (associated(parcours))
C      
        allocate(newgrid)        
C
        newgrid % spacerefx = parcours % r % spacerefx
C
        newgrid % timerefx = parcours % r % timerefx
C
        If (Agrif_Probdim >= 2) Then
C
            newgrid % spacerefy = parcours % r % spacerefy
C
            newgrid % timerefy = parcours % r % timerefy  
C
        End If
C
        If (Agrif_Probdim == 3) Then
C
            newgrid % spacerefz = parcours % r % spacerefz
C
            newgrid % timerefz = parcours % r % timerefz 
C
        End If                            
C        
C            
        newgrid % nbx = (parcours % r % imax 
     &                   - parcours % r % imin) 
     &                   * parcours % r % spacerefx       
C     
        newgrid % ix = parcours % r % imin                  
C
        newgrid % Agrif_dx = g % Agrif_dx / Real(newgrid % spacerefx)
C
        newgrid % Agrif_x = g % Agrif_x +
     &                      (parcours % r % imin - 1)* g % Agrif_dx
C      
        If (Agrif_Probdim .GE. 2) Then
C            
            newgrid % nby = (parcours % r % jmax
     &                       - parcours % r % jmin) 
     &                       * parcours % r % spacerefy
C            
            newgrid % iy = parcours % r % jmin 
C      
            newgrid % Agrif_dy 
     &      = g % Agrif_dy / Real(newgrid % spacerefy)
C      
            newgrid % Agrif_y = g % Agrif_y +
     &                          (parcours % r % jmin - 1)* g % Agrif_dy
C
        End If      
C
        If (Agrif_Probdim .EQ. 3) Then
C               
            newgrid % nbz = (parcours % r % kmax
     &                       - parcours % r % kmin) 
     &                       * parcours % r % spacerefz
C          
            newgrid % iz = parcours % r % kmin 
C    
            newgrid % Agrif_dz 
     &      = g % Agrif_dz / Real(newgrid % spacerefz)
C      
            newgrid % Agrif_z = g % Agrif_z +
     &                        (parcours % r % kmin - 1)* g % Agrif_dz   
C
        End If                      
C
C       Pointer on the parent grid                        
        newgrid % parent => g      
C      
C       Grid pointed by newgrid is a fixed grid      
        If (parcours % r % number .GT. 0) Then      
C         
            newgrid % fixed = .true.  
C  
          Else
C        
            newgrid % fixed = .false.
C      
        End If
C
C       Number of the grid pointed by newgrid
        newgrid % fixedrank = parcours % r % number
C           
C       No time calculation on this grid         
        newgrid % ngridstep = 0      
C
C
C       Calculation of the time refinement factor             
        res = 1
C      
        res = max(res,parcours % r % timerefx)
C
        If (Agrif_Probdim .GE. 2) Then  
C      
            res = max(res,parcours % r % timerefy)
C
        End If      
C
        If (Agrif_Probdim .EQ. 3) Then
C      
            res = max(res,parcours % r % timerefz)      
C
        End If       
C      
C
C       Test indicating if the current grid has a common border with the root 
C       coarse grid in the x direction
        newgrid % NearRootBorderX = .FALSE.
C        
        If ((newgrid % parent % NearRootBorderX) .AND. 
     &      (newgrid % ix == 1)) Then
C
            newgrid % NearRootBorderX = .TRUE.
C
        End If
C
        newgrid % DistantRootBorderX = .FALSE.
C 
        If ((newgrid % parent % DistantRootBorderX) .AND. 
     &      (newgrid % ix + (newgrid % nbx/newgrid % spacerefx) - 1 
     &               == newgrid % parent % nbx)) Then
C   
            newgrid % DistantRootBorderX = .TRUE.
C
        End If
C
C       Test indicating if the current grid has a common border with the root 
C       coarse grid in the y direction
        If (Agrif_Probdim .GE. 2) Then
C      
            newgrid % NearRootBorderY = .FALSE. 
C        
            If ((newgrid % parent % NearRootBorderY) .AND. 
     &          (newgrid % iy == 1)) Then
C
                newgrid % NearRootBorderY = .TRUE.
C
            End If
C
            newgrid % DistantRootBorderY = .FALSE. 
C       
            If ((newgrid % parent % DistantRootBorderY) .AND. 
     &          (newgrid % iy + (newgrid % nby/newgrid % spacerefy) - 1 
     &          == newgrid % parent % nby)) Then
C
                newgrid % DistantRootBorderY = .TRUE.
C
            End If
C
        End If
C
C       Test indicating if the current grid has a common border with the root 
C       coarse grid in the z direction
        If (Agrif_Probdim .EQ. 3) Then
C       
        newgrid % NearRootBorderZ = .FALSE.  
C        
            If ((newgrid % parent % NearRootBorderZ) .AND. 
     &          (newgrid % iz == 1)) Then
C
                newgrid % NearRootBorderZ = .TRUE.
C
            End If
C
            newgrid % DistantRootBorderZ = .FALSE.
C        
           If ((newgrid % parent % DistantRootBorderZ) .AND. 
     &         (newgrid % iz + (newgrid % nbz/newgrid % spacerefz) - 1 
     &               == newgrid % parent % nbz)) then
C
                newgrid % DistantRootBorderZ = .TRUE.
C
           End If
C
        End If      
C
C
C       Writing in output files
C
C       Number of time steps for 1 time step on the root coarse grid            
        newgrid % nstepdt = g % nstepdt * res
C
C       Output saveguard after the first time step            
        newgrid % test = .FALSE.
C
        newgrid % ngapdt = 0
C
        newgrid % oldgrid = .FALSE.      
C
C     
C       Definition of the characteristics of the variable of the grid pointed by
C       newgrid 
        Call Agrif_Create_Var
     &           (newgrid)
C      
C       Instanciation of the grid pointed by newgrid and its variables
        Call Agrif_Base_Instance
     &           (newgrid)
C       
C       Nullify the variable of the grid pointed by newgrid
        Call Agrif_Nullify
     &           (newgrid)
C
C
C       Addition of this grid to the grid hierarchy 
C      
        nullify(newgrid % child_grids)
C      
        allocate(newpgrid)  
C  
        newpgrid % gr => newgrid
C      
        nullify(newpgrid % next)
C      
        end_list % next => newpgrid
C      
        end_list => end_list % next               
C      
        parcours => parcours % next
C
C
C       Updating the total number of fixed grids
        If (newgrid % fixed) Then
C        
            AGRIF_nbfixedgrids = AGRIF_nbfixedgrids + 1
C      
        End If
C                  
      End do
C
C
      If (nullliste) Then
C
          g % child_grids => g % child_grids % next
C                  
          parcours2 => g % child_grids
C      
          deallocate(parcours3)
C        
        Else
C        
          parcours2 => parcours3 % next
C      
      End If
C      
      parcours => coarsegrid % childgrids
C            
C
C     Recursive call to the subroutine Agrif_Create_Fixed_Grids to create the
C     grid hierarchy
C      
      Do while (associated(parcours))
C      
        Call Agrif_Create_Grids
     &           (parcours2 % gr,parcours % r)
C     
        parcours => parcours % next
C      
        parcours2 => parcours2 % next
C      
      End do 
C      
      Return      
C                  
      End Subroutine Agrif_Create_Grids
C
C      
C
C     **************************************************************************  
CCC   Subroutine Agrif_Init_Hierarchy  
C     **************************************************************************  
C
      Recursive Subroutine Agrif_Init_Hierarchy(g) 
C
CCC   Description:
CCC   Subroutine to initialize all the grids except the root coarse grid (this 
CCC   one, pointed by AGRIF_mygrid, is initialized by the subroutine 
CCC   Agrif_Init_Grids defined in the module Agrif_Util and called in the main 
CCC   program ). 
C
CC    Method:
CC    Recursive subroutine.        
C
C     Declarations:
C
      Implicit none
C
C     Pointer argument      
      Type(AGRIF_grid), Pointer  :: g         ! Pointer on the root coarse grid 
C      
C     Local variables      
      Type(AGRIF_pgrid), Pointer :: parcours  ! Pointer for the recursive call
      Logical :: Init_Hierarchy
C      
C
      parcours=>g%child_grids
C      
      Do while (associated(parcours))
C
        Init_Hierarchy = .false.
C
#if defined AGRIF_USE_FIXED_GRIDS || defined AGRIF_USE_ONLY_FIXED_GRIDS        
C
        If ((parcours%gr%fixed) 
     &      .AND. (Agrif_mygrid%ngridstep == 0)) Then
C      
            Init_Hierarchy = .true.
C          
        End If      
C
#endif
C
        If (.not. parcours%gr%fixed) Init_Hierarchy = .true.
C
        If (parcours % gr % oldgrid) Init_Hierarchy = .false.
C
        If (Init_Hierarchy) Then 
C
            Call Agrif_Nullify
     &           (parcours % gr)       
C
C           Instanciation of the grid pointed by parcours%gr and its variables 
            Call Agrif_Base_Instance
     &           (parcours % gr)
C    
C           Allocation of the arrays containing values of the variables of the grid
C           pointed by parcours%gr      
C      
            Call Agrif_Allocation
     &           (parcours % gr)     
C       
            Call Agrif_Instance
     &           (parcours % gr)
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C
            Call Agrif_Allocate_Restore      
     &           (parcours % gr)
C
#endif      
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS      
C 
C           Initialization by copy of the grids created by clustering 
            Call AGRIF_CopyFromold_All
     &           (parcours % gr,Agrif_oldmygrid)
C
#endif                             
C
C           Initialization by interpolation 
C           (this routine is written by the user) 
            Call Agrif_Initvalues()
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C 
            Call Agrif_Free_Restore      
     &           (parcours % gr)
C
#endif     
C
C           Copy of the new initialisations for static variables
            Call Agrif_Back_Instance
     &           (parcours % gr) 
C
        End If                                  
C      
        parcours => parcours % next 
C         
      End do
C      
C
      parcours => g % child_grids
C
C      
C     Recursive operation to initialize all the grids
      Do while (associated(parcours))
C      
        Call Agrif_Init_Hierarchy
     &           (parcours%gr)
C
        parcours => parcours%next
C      
      End do
C
C      
      End Subroutine Agrif_Init_Hierarchy 
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Allocate_Restore  
C     **************************************************************************  
C
      Subroutine Agrif_Allocate_Restore(Agrif_Gr)
C      
C      
#include "include_agrif.h"
C
      Type(AGRIF_grid), Pointer  :: Agrif_Gr   ! Pointer on the root coarse grid      
C
#include "allocate_restore_agrif.h"      
C
      Return
C
C      
      End Subroutine Agrif_Allocate_Restore      
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Free_Restore  
C     **************************************************************************  
C
      Subroutine Agrif_Free_Restore(Agrif_Gr)
C
C      
C     Pointer argument      
      Type(AGRIF_grid), Pointer  :: Agrif_Gr   ! Pointer on the root coarse grid      
C
C
#include "free_restore_agrif.h"      
C 
      Return
C
C      
      End Subroutine Agrif_Free_Restore
C
C
C
      End Module Agrif_Clustering  
        
