C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Init
C
#include "keys_agrif.h"
C
      Module Agrif_Init 
C
CCC   Description:
CCC   Several operations on the variables of the current grid (creation,
CCC   instanciation, ...) used during the creation of the grid hierarchy and
CCC   during the time integration.
C
C     Modules used: 
C
      Use Agrif_Types 
C   
      Contains
C     Defininition of the procedures contained in this module
C  
C
C
C     ************************************************************************** 
CCC   Subroutine AGRIF_Nullify 
C     ************************************************************************** 
C 
      Subroutine Agrif_Nullify(AGRIF_GR)
C
CCC   Description:
CCC   Subroutine to nullify the arrays of the variables of the current grid.
C
CC    Method:
CC    Using the nullify function.  
C
C     Declarations:
C       
#include "include_agrif.h"
C
C     Pointer argument: 
      Type(AGRIF_grid), Pointer :: AGRIF_GR  ! Pointer on the current grid  
C
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C     Nullify the arrays restore 
c      Nullify(Agrif_Gr % restore1D)
c      Nullify(Agrif_Gr % restore2D)
c      Nullify(Agrif_Gr % restore3D)
c      Nullify(Agrif_Gr % restore4D)
C
C
#endif
C 
C     Nullify the arrays 
c#include "nullify_agrif.h"
C
      End Subroutine Agrif_Nullify 
C
C
C
C
C
C
C     ************************************************************************** 
CCC   Subroutine Agrif_Instance 
C     ************************************************************************** 
C 
      Subroutine Agrif_Instance(AGRIF_GR)
C
CCC   Description:
CCC   Subroutine to do an instance of the common variables to the variables of  
CCC   the current grid.
C
CC    Method:
CC    Pointing the common variables on these of the current grid.  
C
C     Declarations:
C        
#include "include_agrif.h"
C
C     Pointer argument: 
      Type(AGRIF_grid), Pointer :: AGRIF_GR   ! Pointer on the current grid  
C
      Interface
#include "instance_interface_agrif.h"
      End Interface
C 
#include "instance_agrif.h" 
C
      Call AGRIF_Instance_Parent()
C
C     Calculation of isf,jsf,nzsf and of the index of the output file 
      Call Agrif_InitWorkSpace() 
C
C 
      End Subroutine Agrif_Instance  
C 
C
C
C     ************************************************************************** 
CCC   Subroutine Agrif_Instance_Parent 
C     ************************************************************************** 
C 
      Subroutine Agrif_Instance_Parent()
C
CCC   Description:
C
CC    Method:  
C
C     Declarations:
#include "include_agrif.h"
C
C
#include "instance_parents_agrif.h"
C
C
      End Subroutine Agrif_Instance_Parent
C
C
C
C
C
C
C     ************************************************************************** 
CCC   Subroutine Agrif_Back_Instance 
C     ************************************************************************** 
C 
      Subroutine Agrif_Back_Instance(AGRIF_GR)
C
CCC   Description:
CCC   Subroutine to do an instance of the variables in common to the variables of
CCC   the grid pointed by AGRIF_GR.
C
CC    Method:
CC    Pointing the common variables on these of the grid pointed by AGRIF_GR.  
C
C     Declarations:
C        
#include "include_agrif.h"
C
C     Pointer argument: 
      Type(AGRIF_grid), Pointer :: AGRIF_GR   ! Pointer on the current grid  
C
      Interface
#include "back_instance_interface_agrif.h"
      End Interface
C 
#include "back_instance_agrif.h"  
C 
C
      End Subroutine Agrif_Back_Instance  
C 
C
C
C     ************************************************************************** 
CCC   Subroutine Agrif_Base_Instance 
C     ************************************************************************** 
C 
      Subroutine Agrif_Base_Instance(AGRIF_GR)
C
CCC   Description:
CCC   Subroutine to do an instance of the array sizes
C
CC    Method:
CC    Pointing the common variables on these of the grid pointed by AGRIF_GR.  
C
C     Declarations:
C        
#include "include_agrif.h"
C
C     Pointer argument: 
      Type(AGRIF_grid), Pointer :: AGRIF_GR   ! Pointer on the current grid  
C
C 
#include "instance_base_agrif.h" 
C
C     Calculation of isf,jsf,nzsf and of the index of output file 
      Call Agrif_InitWorkSpace() 
C
C 
      End Subroutine Agrif_Base_Instance
C
C
C
      End Module AGRIF_Init 

#include "instance_commons_agrif.h"
#include "back_instance_commons_agrif.h" 
