!
! $Id: modinterp.F,v 1.2 2003/12/17 13:56:10 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Interpolation
C
      Module Agrif_Interpolation
C
CCC   Description:
CCC   Module to initialize a fine grid from its parent grid, by using a space 
CCC   interpolation
C
C     Modules used:
C  
      Use Agrif_Interpbasic
      Use Agrif_CurgridFunctions
      Use Agrif_Mask 
C      
      CONTAINS
C     Define procedures contained in this module        
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_1d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_1d(TypeInterp,parent,child,tab,torestore)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions of a fine grid for a 1D grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                  ! Kind of interpolation (linear,lagrange,spline)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Logical :: torestore
      Real, Dimension(lbound(child%var%array1,1):
     &         ubound(child%var%array1,1)), Target :: tab    ! Result
C
C
      allocate(childtemp % var)  
C      
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 1
C
C     Tab is the result of the interpolation
      childtemp % var % array1 => tab 
C      
      If (torestore) Then
C 
          childtemp % var % array1 = child % var % array1
C          
          childtemp % var % restore1D => child % var % restore1D
C      
        Else
C        
          Nullify(childtemp % var % restore1D)
C      
      End If     
C 
C     Index indicating (in the Agrif_Interp1D procedure) if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex       
C      
      Call Agrif_InterpVariable
     &     (TypeInterp,parent,childtemp,torestore)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_1D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_2d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_2d(TypeInterp,parent,child,tab,torestore)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions of a fine grid for a 2D grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                  ! Kind of interpolation (linear,lagrange,spline)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Logical :: torestore
      Real, Dimension(lbound(child%var%array2,1):
     &    ubound(child%var%array2,1),lbound(child%var%array2,2):
     &    ubound(child%var%array2,2)), Target :: tab    ! Result
C
C
      allocate(childtemp % var)  
C      
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 2
C
C     Tab is the result of the interpolation
      childtemp % var % array2 => tab  
C      
      If (torestore) Then      
C 
          childtemp % var % array2 = child % var % array2
C 
          childtemp % var % restore2D => child % var % restore2D        
C      
        Else
C        
          Nullify(childtemp % var % restore2D)
C      
      End If       
C 
C     Index indicating (in the Agrif_Interp2D procedure) if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex       
C     
      Call Agrif_InterpVariable
     &     (TypeInterp,parent,childtemp,torestore)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_2D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_3d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_3d(TypeInterp,parent,child,tab,torestore)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions of a fine grid for a 3D grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                  ! Kind of interpolation (linear,lagrange,spline)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Logical :: torestore
      Real, Dimension(lbound(child%var%array3,1):
     &      ubound(child%var%array3,1),lbound(child%var%array3,2):
     &      ubound(child%var%array3,2),lbound(child%var%array3,3):
     &      ubound(child%var%array3,3)), Target :: tab  ! Results
C
C
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 3  
C     
C     Tab is the result of the interpolation 
      childtemp % var % array3 => tab 
C      
      If (torestore) Then
C     
          childtemp % var % array3 = child % var % array3
C
          childtemp % var % restore3D => child % var % restore3D
C      
        Else
C        
          Nullify(childtemp % var % restore3D)
C      
      End If
C 
C     Index indicating (in the Agrif_Interp3D procedure) if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex    
C     
      Call Agrif_InterpVariable
     &     (TypeInterp,parent,childtemp,torestore)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_3D               
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_4d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_4d(TypeInterp,parent,child,tab,torestore)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions of a fine grid for a 4D grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                  ! Kind of interpolation (linear,lagrange,spline)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Logical :: torestore
      Real, Dimension(lbound(child%var%array4,1):
     &      ubound(child%var%array4,1),lbound(child%var%array4,2):
     &      ubound(child%var%array4,2),lbound(child%var%array4,3):
     &      ubound(child%var%array4,3),lbound(child%var%array4,4):
     &      ubound(child%var%array4,4)), Target :: tab  ! Results
C
C
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 4  
C      
C     Tab is the result of the interpolation
      childtemp % var % array4 => tab 
C 
      If (torestore) Then
C 
          childtemp % var % array4 = child % var % array4
C
          childtemp % var % restore4D => child % var % restore4D
C      
        Else
C        
          Nullify(childtemp % var % restore4D)
C      
      End If        
C 
C     Index indicating (in the Agrif_Interp4D procedure) if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex    
C     
      Call Agrif_InterpVariable
     &     (TypeInterp,parent,childtemp,torestore)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_4D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Interp_5d
C     **************************************************************************
C 
      Subroutine Agrif_Interp_5d(TypeInterp,parent,child,tab,torestore)            
C
CCC   Description:
CCC   Subroutine to calculate the boundary conditions of a fine grid for a 5D grid variable.
C
C     Declarations:
C      
      Implicit none
C
C     Arguments      
      Integer :: TypeInterp                  ! Kind of interpolation (linear,lagrange,spline)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Logical :: torestore
      Real, Dimension(lbound(child%var%array5,1):
     &      ubound(child%var%array5,1),lbound(child%var%array5,2):
     &      ubound(child%var%array5,2),lbound(child%var%array5,3):
     &      ubound(child%var%array5,3),lbound(child%var%array5,4):
     &      ubound(child%var%array5,4),lbound(child%var%array5,5):
     &      ubound(child%var%array5,5)),  Target :: tab  ! Results
C
C
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 5  
C      
C     Tab is the result of the interpolation
      childtemp % var % array5 => tab  
C      
      If (torestore) Then
C 
          childtemp % var % array5 = child % var % array5
C
          childtemp % var % restore5D => child % var % restore5D
C      
        Else
C        
          Nullify(childtemp % var % restore5D)        
C      
      End If       
C 
C     Index indicating (in the Agrif_Interp5D procedure) if a space interpolation is necessary
      childtemp % var % interpIndex => child % var % interpIndex    
C      
      Call Agrif_InterpVariable
     &     (TypeInterp,parent,childtemp,torestore)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Interp_5D
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_InterpVariable    
C     **************************************************************************  
C   
      Subroutine Agrif_InterpVariable(TYPEinterp,parent,child,torestore)   
C
CCC   Description:
CCC   Subroutine to set some arguments of subroutine Agrif_InterpnD, n being the 
CCC   dimension of the grid variable.
C
CC    Declarations:
C      
c      Implicit none
C      
C
C     Scalar argument
      Integer :: TYPEinterp             ! Type of interpolation (linear,spline,...)
C     Data type arguments                                   
      Type(AGRIF_PVariable) :: parent   ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child    ! Variable on the child grid
C
C     Logical argument      
      Logical:: torestore               ! Its value is .false., it indicates the
                                        ! results of the interpolation are    
                                        ! applied on the whole current grid   
C
C     Local scalars      
      Integer :: nbdim                  ! Number of dimensions of the current grid      
      Integer :: n
      
      Integer,Dimension(5) :: pttab_child  
      Integer,Dimension(5) :: petab_child      
      Integer,Dimension(5) :: pttab_parent  
      Integer,Dimension(5) :: petab_parent      
      Integer,Dimension(5) :: itab_child      
      Integer,Dimension(5) :: rhotab_child    ! Space refinement factors 
      Integer,Dimension(5) :: nbtab_child     ! Number of cells in each direction
      Integer,Dimension(5) :: posvartab_child ! Position of the variable on a cell
      Type(AGRIF_Variable), Pointer :: root ! Pointer on the variable of the 
                                            ! root grid
      Type(Agrif_Grid), Pointer :: Agrif_Child_Gr,Agrif_Parent_Gr
      Real, Dimension(5) :: s_child,s_parent
      Real, Dimension(5) :: ds_child,ds_parent
                                    
      Real :: eps
C
C
      Agrif_Child_Gr => Agrif_Curgrid
      Agrif_Parent_Gr => Agrif_Curgrid % parent
C
      root => child % var % root_var 
C
C     Number of dimensions of the current grid
      nbdim = root % nbdim
C       
      Do n=1,nbdim
C      
        posvartab_child(n) = root % posvar(n)      
C      
      End Do            
C      
C      
      Do n=1,nbdim
C            
        Select case(root % interptab(n))
C
C       Value of interptab(n) can be either x,y,z or noint for a no space 
C       dimension            
C
C         The dimension is 'x'
          case('x')
C
            pttab_Child(n) = root % pointX 
C            
            pttab_Parent(n) = root % pointx
C        
            s_Child(n) = Agrif_Child_Gr % Agrif_x
C
            s_Parent(n) = Agrif_Parent_Gr % Agrif_x 
C        
            ds_Child(n) = Agrif_Child_Gr % Agrif_dx
C
            ds_Parent(n) = Agrif_Parent_Gr % Agrif_dx
C                      
            If (root % posvar(n).EQ.1) Then
C          
                petab_Child(n) = pttab_Child(n) + Agrif_Child_Gr%nbx
C          
                petab_Parent(n) = pttab_Parent(n) + Agrif_Parent_Gr%nbx         
C        
              Else
C          
                petab_Child(n) = pttab_Child(n) + 
     &                                            Agrif_Child_Gr%nbx - 1
C          
                petab_Parent(n) = pttab_Parent(n) + 
     &                                           Agrif_Parent_Gr%nbx - 1
C
                s_Child(n) = s_Child(n) + ds_Child(n)/2.
C
                s_Parent(n) = s_Parent(n) + ds_Parent(n)/2.                                
C        
            End If                  
C
C         The dimension is 'y'
          case('y')
C
            pttab_Child(n) = root % pointY
C            
            pttab_Parent(n) = root % pointy
C        
            s_Child(n) = Agrif_Child_Gr % Agrif_y
C
            s_Parent(n) = Agrif_Parent_Gr % Agrif_y 
C        
            ds_Child(n) = Agrif_Child_Gr % Agrif_dy
C
            ds_Parent(n) = Agrif_Parent_Gr % Agrif_dy
C                      
            If (root % posvar(n).EQ.1) Then
C          
                petab_Child(n) = pttab_Child(n) + Agrif_Child_Gr%nby
C          
                petab_Parent(n) = pttab_Parent(n) + Agrif_Parent_Gr%nby         
C        
              Else
C          
                petab_Child(n) = pttab_Child(n) + 
     &                                           Agrif_Child_Gr%nby - 1
C          
                petab_Parent(n) = pttab_Parent(n) + 
     &                                          Agrif_Parent_Gr%nby - 1
C
                s_Child(n) = s_Child(n) + ds_Child(n)/2.
C
                s_Parent(n) = s_Parent(n) + ds_Parent(n)/2.                                
C        
            End If
            
C
C         The dimension is 'z'                            
          case('z')
C
            pttab_Child(n) = root % pointZ
C            
            pttab_Parent(n) = root % pointz
C        
            s_Child(n) = Agrif_Child_Gr % Agrif_z
C
            s_Parent(n) = Agrif_Parent_Gr % Agrif_z 
C        
            ds_Child(n) = Agrif_Child_Gr % Agrif_dz
C
            ds_Parent(n) = Agrif_Parent_Gr % Agrif_dz
C                      
            If (root % posvar(n).EQ.1) Then
C          
                petab_Child(n) = pttab_Child(n) + Agrif_Child_Gr%nbz
C          
                petab_Parent(n) = pttab_Parent(n) + Agrif_Parent_Gr%nbz         
C        
              Else
C          
                petab_Child(n) = pttab_Child(n) + 
     &                                           Agrif_Child_Gr%nbz - 1
C          
                petab_Parent(n) = pttab_Parent(n) + 
     &                                          Agrif_Parent_Gr%nbz - 1
C
                s_Child(n) = s_Child(n) + ds_Child(n)/2.
C
                s_Parent(n) = s_Parent(n) + ds_Parent(n)/2.                                
C        
            End If       
C 
C         The dimension is not space                            
          case('noint')
C
C         The next coefficients are calculated in order to do a simple copy of the 
C         values of the grid variable when the procedure of interpolation is 
C         called for this dimension 
C      
            select case (nbdim) 
C      
              case(1)
C      
                pttab_Child(n) = lbound(child % var % array1,n)
                petab_Child(n) = ubound(child % var % array1,n)
C        
              case(2)
C      
                pttab_Child(n) = lbound(child % var % array2,n)
                petab_Child(n) = ubound(child % var % array2,n)
C      
              case(3)
C      
                pttab_Child(n) = lbound(child % var % array3,n)
                petab_Child(n) = ubound(child % var % array3,n)  
C      
              case(4)
C      
                pttab_Child(n) = lbound(child % var % array4,n)
                petab_Child(n) = ubound(child % var % array4,n)
C
              case(5)
C      
                pttab_Child(n) = lbound(child % var % array5,n) 
                petab_Child(n) = ubound(child % var % array5,n)     
C
            end select
C
C           No interpolation but only a copy of the values of the grid variable      
C
            pttab_Parent(n) = pttab_Child(n)
C
            petab_Parent(n) = petab_Child(n)
C              
            s_Child(n)=0.
C      
            s_Parent(n)=0. 
C      
            ds_Child(n)=1.
C      
            ds_Parent(n)=1.
C
        End select
C            
      End do     
C
C
C     Call to a procedure of interpolation against the number of dimensions of 
C     the grid variable
C
      If (nbdim .EQ. 1) then
C      
         call Agrif_Interp1D
     &            (TYPEinterp,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &             child%var%restore1D,torestore,nbdim)
C          
        else if (nbdim .EQ. 2) then
C      
         call Agrif_Interp2D
     &            (TYPEinterp,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &             child%var%restore2D,torestore,nbdim)
C
        else if (nbdim .EQ. 3) then
C
         call Agrif_Interp3D
     &            (TYPEinterp,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &             child%var%restore3D,torestore,nbdim) 
C
        else if (nbdim .EQ. 4) then
C
         call Agrif_Interp4D
     &            (TYPEinterp,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &             child%var%restore4D,torestore,nbdim)
C
        else if (nbdim .EQ. 5) then
C
         call Agrif_Interp5D
     &            (TYPEinterp,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),pttab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),
     &             child%var%restore5D,torestore,nbdim) 
C      
      End if
C
      Return
C
C
      End subroutine Agrif_InterpVariable       
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_Interp1D  
C     **************************************************************************  
C  
      Subroutine Agrif_Interp1D(TYPEinterp,parent,child,
     &                          pttab,petab,
     &                          pttab_Child,pttab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          restore,torestore,nbdim)  
C
C     Description:
C     Subroutine to interpolate a 1D grid variable from its parent grid,
C     by using a space interpolation. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: nbdim
      Integer                :: TYPEinterp             ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)  :: parent                 ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)  :: child                  ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: petab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable      
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids 
      Integer,Dimension(:),Pointer :: restore        ! Indicates points where interpolation is done 						     
      Logical                :: torestore              ! Indicates if the array restore is used            
C
C     Local pointers
      Real, Dimension(:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,j,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob 
      Integer                     :: New_Comm
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: i1,j1,i2,k,size,pglob
      Logical                 :: value,res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Integer                 :: New_Group,New_Nbprocs,New_Rank
      Logical, Dimension(0:Agrif_Nbprocs-1) :: tab_rangs1
      Integer,Dimension(:),Allocatable :: tab_rangs2
      Real,Dimension(:),Allocatable  :: parentvalues
      Integer,Dimension(nbdim) :: lb,ub
C
#endif      
C     
C   
C     Boundaries of the current grid where interpolation is done
      Call Agrif_Childbounds(nbdim,lbound(child%var%array1),
     &                       ubound(child%var%array1),pttab,petab,
     &                       pttruetab,cetruetab)  
C
#ifdef AGRIF_MPI    
C
C     Creation of a communicator containing all processors concerned by 
C     the interpolation 
C
      value = .true.
C
      Do i = 1,nbdim
C
        If (pttruetab(i) > cetruetab(i)) value = .false.
C
      End Do
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
      Call MPI_ALLGATHER(value,1,MPI_LOGICAL,tab_rangs1,1,
     &                   MPI_LOGICAL,MPI_COMM_WORLD,code)         
C
      New_Nbprocs = COUNT(tab_rangs1)
      
      New_Nbprocs = Agrif_Nbprocs
C
      Allocate(tab_rangs2(0:New_Nbprocs-1))
C
      i2 = 0
C
      Do i1 = 0,Agrif_Nbprocs-1
C
        If (tab_rangs1(i1)) Then
C
c            tab_rangs2(i2) = i1
C
            i2 = i2 + 1
C
        End If
        tab_rangs2(i1) = i1
C
      End Do                  
C
      Call MPI_GROUP_INCL(Agrif_Group,New_Nbprocs,tab_rangs2,
     &                    New_Group,code)
C 
      Call MPI_COMM_CREATE(MPI_COMM_WORLD,New_Group,New_Comm,code)
C
      Deallocate(tab_rangs2)            
C
C
c      If (.not. value) return   ! Current processor is not concerned by the interpolation
C
C
#endif 
C
C     Boundaries of the parent grid for the interpolation
      parentglob(:,1) = lbound(parent % var % array1)
      parentglob(:,2) = ubound(parent % var % array1)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Parentbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                        s_Child_temp,s_Child,ds_Child,
     &                        s_Parent,ds_Parent,
     &                        pttruetab,cetruetab,pttab_Child,
     &                        pttab_Parent,parentglob,NEW_COMM)
C
C
      allocate(tempC(pttruetab(1):cetruetab(1))) 
C
      allocate(tempP(indmin(1):indmax(1)))      
C
      tempP = 0.      
C
C
#ifdef AGRIF_MPI     
C
C     tab2 contains the necessary limits of the parent grid for each processor
      Call MPI_GROUP_TRANSLATE_RANKS(Agrif_Group,1,Agrif_ProcRank,
     &                               New_Group,New_Rank,code)     
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(New_Comm,code)
C
      Allocate(tab2(nbdim,2,0:New_Nbprocs-1))
C
      Call MPI_ALLGATHER(tab1,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,New_Comm,code)            
C
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      lb=lbound(parent%var%array1)
      ub=ubound(parent%var%array1)
      Call Agrif_GlobtoLocInd(parentarray,lb,ub,
     &                      parentglob(1:nbdim,1),parentglob(1:nbdim,2),
     &                        nbdim,Agrif_Procrank)
C
      Call Agrif_ParentGrid_to_ChildGrid()      
C      
C
C     Informations to be sent to the others processors (necessary limits of the parent grid) 
      Call Agrif_Sendtoproc(Agrif_Nbprocs,New_Nbprocs,sendtoproc,nbdim,
     &                      imin,imax,iminloc,imaxloc,parentglob,tab2,
     &                      parentarray) 
C
C
C     imin,imax,iminloc,imaxloc and a part of the parent grid are sent to others processors
      Do k = 0,New_Nbprocs-1        
C
        If (k == New_Rank) Then
C
            tempP(imin(1,k):imax(1,k)) = 
     &      parent%var%array1(iminloc(1,k):imaxloc(1,k))
C
          Else        
C
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,code)
C
            If (sendtoproc(k)) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imaxloc(i,k)-iminloc(i,k)+1)
C
                End Do
C
                Call MPI_SEND(parent%var%array1(
     &                        iminloc(1,k):imaxloc(1,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,code)  
C
            End If
C
        End If
C
      End Do  
C
C
C     Reception from others processors of the necessary part of the parent grid
      Do k = 0,New_Nbprocs-1        
C
        If (k /= New_Rank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,statut,code) 
C
            recvfromproc(k) = res       
C
            If (recvfromproc(k)) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(tempP(
     &                        imin_recv(1,k):imax_recv(1,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,statut,code)
C
            End If 
C
        End If
C
      End Do          
C
#else
C
      tempP(indmin(1):indmax(1)) = 
     &parent % var % array1(indmin(1):indmax(1))  
C
#endif
C
C             
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
          noraftab(1:nbdim) = 
     &         child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(parentvalues(indmin(1):indmax(1)))
C
          parentvalues = tempP
C
          Call Agrif_CheckMask1D(tempP,parentvalues,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(parentvalues)
C
#else
C
          Call Agrif_CheckMask1D(tempP,parent%var%array1,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           lbound(parent%var%array1),
     &                           ubound(parent%var%array1),
     &                           noraftab(1:nbdim))
C
#endif
C
      End if      
C
C
C     Interpolation of the current grid
#ifdef AGRIF_MPI
      if (value) then
#endif      
      Call Agrif_Interp_1D_recursive(TypeInterp,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)
#ifdef AGRIF_MPI                 
      endif
#endif      
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      lb=lbound(parent%var%array1)
      ub=ubound(parent%var%array1)
      
      Call Agrif_GlobtoLocInd(childarray,lb,ub,pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)
C
#endif
C
C
C     Special values on the child grid  
      If (Agrif_UseSpecialValueFineGrid) Then
C
#ifdef AGRIF_MPI
C	
          Where (child%var%array1(childarray(pttruetab(1),1,2):
     &                            childarray(cetruetab(1),1,2))
     & 					== Agrif_SpecialValueFineGrid)
C
#else
C
          Where (child%var%array1(pttruetab(1):cetruetab(1))
     & 					== Agrif_SpecialValueFineGrid)              
C
#endif 
C
            tempC(pttruetab(1):cetruetab(1)) 
     &      = Agrif_SpecialValueFineGrid
C	  
          End Where
C	
      End If
C  
C
C     Results of the interpolation written in child % var % array2
      If (torestore) Then
C
          Do i = pttruetab(1),cetruetab(1) 	
C         
            If (restore(i) == 0) Then
C
#ifdef AGRIF_MPI
C
                Do i1 = 1,childarray(i,1,1)
C
	          child%var%array1(childarray(i,1,2)) = 
     &                tempC(i)
C
                End Do  
C
#else 
C          
                child % var % array1(i) = tempC(i)
C
#endif
C	  
            End If
C
          End Do
C	
        Else
C
#ifdef AGRIF_MPI
C
          Do i = pttruetab(1),cetruetab(1)
C
            Do i1 = 1,childarray(i,1,1)
C
	      child%var%array1(childarray(i,1,2)) = 
     &        tempC(i)
C
            End Do 
C
          End Do            
C
#else 
C       
          child % var % array1(pttruetab(1):cetruetab(1)) =
     &    tempC(pttruetab(1):cetruetab(1))
C
#endif 
C     	
      End If
C
C
      Deallocate(tempP)
      Deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      Call MPI_COMM_FREE(New_Comm,code)      
C
C     Deallocations
      Deallocate(tab2)
      Deallocate(parentarray,childarray)      
C
#endif
C
C		  
      End Subroutine Agrif_Interp1D
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_Interp2D  
C     **************************************************************************  
C  
      Subroutine Agrif_Interp2D(TYPEinterp,parent,child,
     &                          pttab,petab,
     &                          pttab_Child,pttab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          restore,torestore,nbdim)  
C
C     Description:
C     Subroutine to interpolate a 2D grid variable from its parent grid,
C     by using a space interpolation. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: nbdim
      Integer                :: TYPEinterp             ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)  :: parent                 ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)  :: child                  ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: petab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable      
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids 
      Integer,Dimension(:,:),Pointer :: restore        ! Indicates points where interpolation is done 						     
      Logical                :: torestore              ! Indicates if the array restore is used            
C
C     Local pointers
      Real, Dimension(:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,j,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob  
      Integer                     :: New_Comm
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: i1,j1,i2,k,size,pglob
      Logical                 :: value,res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Integer                 :: New_Group,New_Nbprocs,New_Rank
      Logical, Dimension(0:Agrif_Nbprocs-1) :: tab_rangs1
      Integer,Dimension(:),Allocatable :: tab_rangs2
      Real,Dimension(:,:),Allocatable  :: parentvalues
      Integer,Dimension(nbdim) :: lb,ub      
C
#endif      
C     
C   
C     Boundaries of the current grid where interpolation is done
      Call Agrif_Childbounds(nbdim,lbound(child%var%array2),
     &                       ubound(child%var%array2),pttab,petab,
     &                       pttruetab,cetruetab)  
C
        
#ifdef AGRIF_MPI    
C
C     Creation of a communicator containing all processors concerned by 
C     the interpolation 
C
      value = .true.
C
      Do i = 1,nbdim
C
        If (pttruetab(i) > cetruetab(i)) value = .false.
C
      End Do
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
      Call MPI_ALLGATHER(value,1,MPI_LOGICAL,tab_rangs1,1,
     &                   MPI_LOGICAL,MPI_COMM_WORLD,code)         
C
      New_Nbprocs = COUNT(tab_rangs1)
      
      New_Nbprocs = Agrif_Nbprocs
      
C
      Allocate(tab_rangs2(0:New_Nbprocs-1))
C
      i2 = 0
C
      Do i1 = 0,Agrif_Nbprocs-1
C
        If (tab_rangs1(i1)) Then
C
c            tab_rangs2(i2) = i1
C
            i2 = i2 + 1
C
        End If
        
        tab_rangs2(i1)=i1
C
      End Do                  
C
      Call MPI_GROUP_INCL(Agrif_Group,New_Nbprocs,tab_rangs2,
     &                    New_Group,code)
C 
      Call MPI_COMM_CREATE(MPI_COMM_WORLD,New_Group,New_Comm,code)
C
      Deallocate(tab_rangs2)            
C
C
c      If (.not. value) return   ! Current processor is not concerned by the interpolation
C
C
#endif 
C
C     Boundaries of the parent grid for the interpolation
      parentglob(:,1) = lbound(parent % var % array2)
      parentglob(:,2) = ubound(parent % var % array2)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Parentbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                        s_Child_temp,s_Child,ds_Child,
     &                        s_Parent,ds_Parent,
     &                        pttruetab,cetruetab,pttab_Child,
     &                        pttab_Parent,parentglob,NEW_COMM)
C
C
      allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2))) 
C
      allocate(tempP(indmin(1):indmax(1),
     &               indmin(2):indmax(2)))      
C
      tempP = 0.      
C
C
#ifdef AGRIF_MPI     
C
C     tab2 contains the necessary limits of the parent grid for each processor
      Call MPI_GROUP_TRANSLATE_RANKS(Agrif_Group,1,Agrif_ProcRank,
     &                               New_Group,New_Rank,code)     
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(New_Comm,code)
C
      Allocate(tab2(nbdim,2,0:New_Nbprocs-1))
C
      Call MPI_ALLGATHER(tab1,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,New_Comm,code)            
C
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      lb=lbound(parent%var%array2)
      ub=ubound(parent%var%array2)
      
      Call Agrif_GlobtoLocInd(parentarray,lb,ub,
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)
C
      Call Agrif_ParentGrid_to_ChildGrid()      
C      
C
C     Informations to be sent to the others processors (necessary limits of the parent grid) 
      Call Agrif_Sendtoproc(Agrif_Nbprocs,New_Nbprocs,sendtoproc,nbdim,
     &                      imin,imax,iminloc,imaxloc,parentglob,tab2,
     &                      parentarray) 
C
C
C     imin,imax,iminloc,imaxloc and a part of the parent grid are sent to others processors
      Do k = 0,New_Nbprocs-1        
C
        If (k == New_Rank) Then
C
            tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k)) = 
     &      parent%var%array2(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k))
C
          Else        
C
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,code)
C
            If (sendtoproc(k)) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imaxloc(i,k)-iminloc(i,k)+1)
C
                End Do
C
     
                Call MPI_SEND(parent%var%array2(
     &                        iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,code)
C
            End If
C
        End If
        
C
      End Do  
C
C
C     Reception from others processors of the necessary part of the parent grid
      Do k = 0,New_Nbprocs-1        
C
        If (k /= New_Rank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,statut,code) 
C
            recvfromproc(k) = res       
C
            If (recvfromproc(k)) Then
C
        
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(tempP(
     &                        imin_recv(1,k):imax_recv(1,k),
     &                        imin_recv(2,k):imax_recv(2,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,statut,code)
C
            End If 
C
        End If
        
C
      End Do    
C
#else
C
      tempP(indmin(1):indmax(1),indmin(2):indmax(2)) = 
     &parent % var % array2(indmin(1):indmax(1),indmin(2):indmax(2))  
C
#endif
C
C             
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
          noraftab(1:nbdim) = 
     &         child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(parentvalues(indmin(1):indmax(1),
     &                          indmin(2):indmax(2)))
C
          parentvalues = tempP
C
          Call Agrif_CheckMask2D(tempP,parentvalues,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(parentvalues)
C
#else
C
          Call Agrif_CheckMask2D(tempP,parent%var%array2,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           lbound(parent%var%array2),
     &                           ubound(parent%var%array2),
     &                           noraftab(1:nbdim))
C
#endif
C
      End if      
C
C
C     Interpolation of the current grid
#ifdef AGRIF_MPI
      if (value) then
#endif      
      Call Agrif_Interp_2D_recursive(TypeInterp,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)            
#ifdef AGRIF_MPI     
       endif
#endif       
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      lb=lbound(parent%var%array2)
      ub=ubound(parent%var%array2)
      Call Agrif_GlobtoLocInd(childarray,lb,ub,pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)
C
#endif
C
C
C     Special values on the child grid  
      If (Agrif_UseSpecialValueFineGrid) Then
C
#ifdef AGRIF_MPI
C	
          Where (child%var%array2(childarray(pttruetab(1),1,2):
     &                            childarray(cetruetab(1),1,2),
     &                            childarray(pttruetab(2),2,2):
     &                            childarray(cetruetab(2),2,2))
     & 					== Agrif_SpecialValueFineGrid)
C
#else
C
          Where (child%var%array2(pttruetab(1):cetruetab(1),
     &                            pttruetab(2):cetruetab(2))
     & 					== Agrif_SpecialValueFineGrid)              
C
#endif 
C
            tempC(pttruetab(1):cetruetab(1),pttruetab(2):cetruetab(2)) 
     &      = Agrif_SpecialValueFineGrid
C	  
          End Where
C	
      End If
C  
C
C     Results of the interpolation written in child % var % array2
      If (torestore) Then
C
          Do j = pttruetab(2),cetruetab(2)
C
            Do i = pttruetab(1),cetruetab(1) 	
C         
              If (restore(i,j) == 0) Then
C
#ifdef AGRIF_MPI
C
                  Do j1 = 1,childarray(j,2,1)
C
                    Do i1 = 1,childarray(i,1,1)
C
	              child%var%array2(childarray(i,1,2),
     &                                 childarray(j,2,2)) = 
     &                tempC(i,j)
C
                    End Do
C
                  End Do  
C
#else 
C          
                  child % var % array2(i,j) = tempC(i,j)
C
#endif
C	  
              End If
C
            End Do
C
          End Do
C	
        Else
C
#ifdef AGRIF_MPI
C
          Do j = pttruetab(2),cetruetab(2)
C
            Do i = pttruetab(1),cetruetab(1)
C
              Do j1 = 1,childarray(j,2,1)
C
                    Do i1 = 1,childarray(i,1,1)
C
	              child%var%array2(childarray(i,1,2),
     &                                 childarray(j,2,2)) = 
     &                tempC(i,j)
C
                    End Do
C
                  End Do 
C
            End do 
C
          End Do            
C
#else 
C       
          child % var % array2(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2)) =
     &    tempC(pttruetab(1):cetruetab(1),
     &          pttruetab(2):cetruetab(2))
C
#endif 
C     	
      End If
C
C
      Deallocate(tempP)
      Deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      Call MPI_COMM_FREE(New_Comm,code)      
C
C     Deallocations
      Deallocate(tab2)
      Deallocate(parentarray,childarray)      
C
#endif
C
C		  
      End Subroutine Agrif_Interp2D 
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_Interp3D  
C     **************************************************************************  
C  
      Subroutine Agrif_Interp3D(TYPEinterp,parent,child,
     &                          pttab,petab,
     &                          pttab_Child,pttab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          restore,torestore,nbdim)  
C
C     Description:
C     Subroutine to interpolate a 3D grid variable from its parent grid,
C     by using a space interpolation. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: nbdim
      Integer                :: TYPEinterp             ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)  :: parent                 ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)  :: child                  ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: petab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable      
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids 
      Integer,Dimension(:,:,:),Pointer :: restore        ! Indicates points where interpolation is done 						     
      Logical                :: torestore              ! Indicates if the array restore is used            
C
C     Local pointers
      Real, Dimension(:,:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,j,k,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp 
      Integer,Dimension(nbdim,2)  :: parentglob
      Integer                     :: New_Comm
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: i1,j1,k1,i2,size,pglob
      Logical                 :: value,res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Integer                 :: New_Group,New_Nbprocs,New_Rank
      Logical, Dimension(0:Agrif_Nbprocs-1) :: tab_rangs1
      Integer,Dimension(:),Allocatable :: tab_rangs2
      Real,Dimension(:,:,:),Allocatable  :: parentvalues
      Integer,Dimension(nbdim) :: lb,ub      
C
#endif      
C     
C   
C     Boundaries of the current grid where interpolation is done
      Call Agrif_Childbounds(nbdim,lbound(child%var%array3),
     &                       ubound(child%var%array3),pttab,petab,
     &                       pttruetab,cetruetab)  
C
#ifdef AGRIF_MPI    
C
C     Creation of a communicator containing all processors concerned by 
C     the interpolation 
C
      value = .true.
C
      Do i = 1,nbdim
C
        If (pttruetab(i) > cetruetab(i)) value = .false.
C
      End Do
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
      Call MPI_ALLGATHER(value,1,MPI_LOGICAL,tab_rangs1,1,
     &                   MPI_LOGICAL,MPI_COMM_WORLD,code)         
C
      New_Nbprocs = COUNT(tab_rangs1)
      
      New_Nbprocs=Agrif_Nbprocs
C
      Allocate(tab_rangs2(0:New_Nbprocs-1))
C
      i2 = 0
C
      Do i1 = 0,Agrif_Nbprocs-1
C
        If (tab_rangs1(i1)) Then
C
c            tab_rangs2(i2) = i1
C
            i2 = i2 + 1
C
        End If
        
        tab_rangs2(i1) = i1
        
C
      End Do                  
C
      Call MPI_GROUP_INCL(Agrif_Group,New_Nbprocs,tab_rangs2,
     &                    New_Group,code)
C 
      Call MPI_COMM_CREATE(MPI_COMM_WORLD,New_Group,New_Comm,code)
C
      Deallocate(tab_rangs2)            
C
C
c      If (.not. value) return   ! Current processor is not concerned by the interpolation
C
C
#endif 
C
C     Boundaries of the parent grid for the interpolation
      parentglob(:,1) = lbound(parent % var % array3)
      parentglob(:,2) = ubound(parent % var % array3)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Parentbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                        s_Child_temp,s_Child,ds_Child,
     &                        s_Parent,ds_Parent,
     &                        pttruetab,cetruetab,pttab_Child,
     &                        pttab_Parent,parentglob,NEW_COMM)
C
C
      allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2),
     &               pttruetab(3):cetruetab(3))) 
C
      allocate(tempP(indmin(1):indmax(1),
     &               indmin(2):indmax(2),
     &               indmin(3):indmax(3)))      
C
      tempP = 0.    
C
C
#ifdef AGRIF_MPI     
C
C     tab2 contains the necessary limits of the parent grid for each processor
      Call MPI_GROUP_TRANSLATE_RANKS(Agrif_Group,1,Agrif_ProcRank,
     &                               New_Group,New_Rank,code)     
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(New_Comm,code)
C
      Allocate(tab2(nbdim,2,0:New_Nbprocs-1))
C
      Call MPI_ALLGATHER(tab1,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,New_Comm,code)            
C
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      lb=lbound(parent%var%array3)
      ub=ubound(parent%var%array3)
      Call Agrif_GlobtoLocInd(parentarray,lb,ub,
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)
C
      Call Agrif_ParentGrid_to_ChildGrid()      
C      
C
C     Informations to be sent to the others processors (necessary limits of the parent grid) 
      Call Agrif_Sendtoproc(Agrif_Nbprocs,New_Nbprocs,sendtoproc,nbdim,
     &                      imin,imax,iminloc,imaxloc,parentglob,tab2,
     &                      parentarray) 
C
C
C     imin,imax,iminloc,imaxloc and a part of the parent grid are sent to others processors
      Do k = 0,New_Nbprocs-1        
C
        If (k == New_Rank) Then
C
            tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k),
     &            imin(3,k):imax(3,k)) = 
     &      parent%var%array3(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k))
C
          Else        
C
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,code)
C
            If (sendtoproc(k)) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imaxloc(i,k)-iminloc(i,k)+1)
C
                End Do
C
                Call MPI_SEND(parent%var%array3(
     &                        iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,code)  
C
            End If
C
        End If
C
      End Do  
C
C
C     Reception from others processors of the necessary part of the parent grid
      Do k = 0,New_Nbprocs-1        
C
        If (k /= New_Rank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,statut,code) 
C
            recvfromproc(k) = res       
C
            If (recvfromproc(k)) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(tempP(
     &                        imin_recv(1,k):imax_recv(1,k),
     &                        imin_recv(2,k):imax_recv(2,k),
     &                        imin_recv(3,k):imax_recv(3,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,statut,code)
C
            End If 
C
        End If
C
      End Do          
C
#else
C
      tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &      indmin(3):indmax(3)) = 
     &parent % var % array3(indmin(1):indmax(1),indmin(2):indmax(2),
     &                      indmin(3):indmax(3))  
C
#endif
C
C             
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
          noraftab(1:nbdim) = 
     &         child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(parentvalues(indmin(1):indmax(1),
     &                          indmin(2):indmax(2),
     &                          indmin(3):indmax(3)))
C
          parentvalues = tempP
C
          Call Agrif_CheckMask3D(tempP,parentvalues,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(parentvalues)
C
#else
C
          Call Agrif_CheckMask3D(tempP,parent%var%array3,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           lbound(parent%var%array3),
     &                           ubound(parent%var%array3),
     &                           noraftab(1:nbdim))
C
#endif
C
      End if      
C
C
C     Interpolation of the current grid
#ifdef AGRIF_MPI
      if (value) then
#endif      
      Call Agrif_Interp_3D_recursive(TypeInterp,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)            
#ifdef AGRIF_MPI     
      endif
#endif      
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      lb=lbound(parent%var%array3)
      ub=ubound(parent%var%array3)
      Call Agrif_GlobtoLocInd(childarray,lb,ub,pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)
C
#endif
C
C
C     Special values on the child grid  
      If (Agrif_UseSpecialValueFineGrid) Then
C
#ifdef AGRIF_MPI
C	
          Where (child%var%array3(childarray(pttruetab(1),1,2):
     &                            childarray(cetruetab(1),1,2),
     &                            childarray(pttruetab(2),2,2):
     &                            childarray(cetruetab(2),2,2),
     &                            childarray(pttruetab(3),3,2):
     &                            childarray(cetruetab(3),3,2))
     & 					== Agrif_SpecialValueFineGrid)
C
#else
C
          Where (child%var%array3(pttruetab(1):cetruetab(1),
     &                            pttruetab(2):cetruetab(2),
     &                            pttruetab(3):cetruetab(3))
     & 					== Agrif_SpecialValueFineGrid)              
C
#endif 
C
            tempC(pttruetab(1):cetruetab(1),pttruetab(2):cetruetab(2),
     &            pttruetab(3):cetruetab(3)) 
     &      = Agrif_SpecialValueFineGrid
C	  
          End Where
C	
      End If
C  
C
C     Results of the interpolation written in child % var % array2
      If (torestore) Then
C
          Do k = pttruetab(3),cetruetab(3)
C
            Do j = pttruetab(2),cetruetab(2)
C
              Do i = pttruetab(1),cetruetab(1) 	
C         
                If (restore(i,j,k) == 0) Then
C
#ifdef AGRIF_MPI
C
                    Do k1 = 1,childarray(k,3,1)
C
                      Do j1 = 1,childarray(j,2,1)
C
                        Do i1 = 1,childarray(i,1,1)
C
	                  child%var%array3(childarray(i,1,2),
     &                                     childarray(j,2,2),
     &                                     childarray(k,3,2)) = 
     &                    tempC(i,j,k)
C
                        End do
C
                      End Do
C
                    End Do  
C
#else 
C          
                    child % var % array3(i,j,k) = tempC(i,j,k)
C
#endif
C	  
                End If
C
              End Do
C
            End Do
C
          End Do
C	
        Else
C
#ifdef AGRIF_MPI
C
          Do k = pttruetab(3),cetruetab(3)
C
            Do j = pttruetab(2),cetruetab(2)
C
              Do i = pttruetab(1),cetruetab(1)
C
                Do k1 = 1,childarray(k,3,1)
C
                  Do j1 = 1,childarray(j,2,1)
C
                    Do i1 = 1,childarray(i,1,1)
C
	              child%var%array3(childarray(i,1,2),
     &                                 childarray(j,2,2),
     &                                 childarray(k,3,2)) = 
     &                tempC(i,j,k)
C
                    End Do
C
                  End Do 
C
                End Do
C
              End Do 
C
            End do 
C
          End Do            
C
#else 
C       
          child % var % array3(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2),
     &                         pttruetab(3):cetruetab(3)) =
     &    tempC(pttruetab(1):cetruetab(1),
     &          pttruetab(2):cetruetab(2),
     &          pttruetab(3):cetruetab(3))
C
#endif 
C     	
      End If
C
C
      Deallocate(tempP)
      Deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      Call MPI_COMM_FREE(New_Comm,code)      
C
C     Deallocations
      Deallocate(tab2)
      Deallocate(parentarray,childarray)      
C
#endif
C
C		  
      End Subroutine Agrif_Interp3D 
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_Interp4D  
C     **************************************************************************  
C  
      Subroutine Agrif_Interp4D(TYPEinterp,parent,child,
     &                          pttab,petab,
     &                          pttab_Child,pttab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          restore,torestore,nbdim)  
C
C     Description:
C     Subroutine to interpolate a 4D grid variable from its parent grid,
C     by using a space interpolation. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: nbdim
      Integer                :: TYPEinterp             ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)  :: parent                 ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)  :: child                  ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: petab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable      
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids 
      Integer,Dimension(:,:,:,:),Pointer :: restore        ! Indicates points where interpolation is done 						     
      Logical                :: torestore              ! Indicates if the array restore is used            
C
C     Local pointers
      Real, Dimension(:,:,:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:,:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,j,k,l,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob 
      Integer                     :: New_Comm
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: i1,j1,k1,l1,i2,size,pglob
      Logical                 :: value,res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Integer                 :: New_Group,New_Nbprocs,New_Rank
      Logical, Dimension(0:Agrif_Nbprocs-1) :: tab_rangs1
      Integer,Dimension(:),Allocatable :: tab_rangs2
      Real,Dimension(:,:,:,:),Allocatable  :: parentvalues
      Integer,Dimension(nbdim) :: lb,ub      
C
#endif      
C     
C   
C     Boundaries of the current grid where interpolation is done
      Call Agrif_Childbounds(nbdim,lbound(child%var%array4),
     &                       ubound(child%var%array4),pttab,petab,
     &                       pttruetab,cetruetab)  
C
#ifdef AGRIF_MPI    
C
C     Creation of a communicator containing all processors concerned by 
C     the interpolation 
C
      value = .true.
C
      Do i = 1,nbdim
C
        If (pttruetab(i) > cetruetab(i)) value = .false.
C
      End Do
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
      Call MPI_ALLGATHER(value,1,MPI_LOGICAL,tab_rangs1,1,
     &                   MPI_LOGICAL,MPI_COMM_WORLD,code)         
C
      New_Nbprocs = COUNT(tab_rangs1)
      
      New_Nbprocs = Agrif_Nbprocs
C
      Allocate(tab_rangs2(0:New_Nbprocs-1))
C
      i2 = 0
C
      Do i1 = 0,Agrif_Nbprocs-1
C
        If (tab_rangs1(i1)) Then
C
c            tab_rangs2(i2) = i1
C
            i2 = i2 + 1
C
        End If
        
         tab_rangs2(i1) = i1        
C
      End Do                  
C
      Call MPI_GROUP_INCL(Agrif_Group,New_Nbprocs,tab_rangs2,
     &                    New_Group,code)
C 
      Call MPI_COMM_CREATE(MPI_COMM_WORLD,New_Group,New_Comm,code)
C
      Deallocate(tab_rangs2)            
C
C
c      If (.not. value) return   ! Current processor is not concerned by the interpolation
C
C
#endif 
C
C     Boundaries of the parent grid for the interpolation
      parentglob(:,1) = lbound(parent % var % array4)
      parentglob(:,2) = ubound(parent % var % array4)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Parentbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                        s_Child_temp,s_Child,ds_Child,
     &                        s_Parent,ds_Parent,
     &                        pttruetab,cetruetab,pttab_Child,
     &                        pttab_Parent,parentglob,NEW_COMM)
C
C
      allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2),
     &               pttruetab(3):cetruetab(3),
     &               pttruetab(4):cetruetab(4))) 
C
      allocate(tempP(indmin(1):indmax(1),
     &               indmin(2):indmax(2),
     &               indmin(3):indmax(3),
     &               indmin(4):indmax(4)))      
C
      tempP = 0.      
C
C
#ifdef AGRIF_MPI     
C
C     tab2 contains the necessary limits of the parent grid for each processor
      Call MPI_GROUP_TRANSLATE_RANKS(Agrif_Group,1,Agrif_ProcRank,
     &                               New_Group,New_Rank,code)     
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(New_Comm,code)
C
      Allocate(tab2(nbdim,2,0:New_Nbprocs-1))
C
      Call MPI_ALLGATHER(tab1,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,New_Comm,code)            
C
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      lb=lbound(parent%var%array4)
      ub=ubound(parent%var%array4)
      Call Agrif_GlobtoLocInd(parentarray,lb,ub,
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)
C
      Call Agrif_ParentGrid_to_ChildGrid()      
C      
C
C     Informations to be sent to the others processors (necessary limits of the parent grid) 
      Call Agrif_Sendtoproc(Agrif_Nbprocs,New_Nbprocs,sendtoproc,nbdim,
     &                      imin,imax,iminloc,imaxloc,parentglob,tab2,
     &                      parentarray) 
C
C
C     imin,imax,iminloc,imaxloc and a part of the parent grid are sent to others processors
      Do k = 0,New_Nbprocs-1        
C
        If (k == New_Rank) Then
C
            tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k),
     &            imin(3,k):imax(3,k),imin(4,k):imax(4,k)) = 
     &      parent%var%array4(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k),
     &                        iminloc(4,k):imaxloc(4,k))
C
          Else        
C
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,code)
C
            If (sendtoproc(k)) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imaxloc(i,k)-iminloc(i,k)+1)
C
                End Do
C
                Call MPI_SEND(parent%var%array4(
     &                        iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k),
     &                        iminloc(4,k):imaxloc(4,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,code)  
C
            End If
C
        End If
C
      End Do  
C
C
C     Reception from others processors of the necessary part of the parent grid
      Do k = 0,New_Nbprocs-1        
C
        If (k /= New_Rank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,statut,code) 
C
            recvfromproc(k) = res       
C
            If (recvfromproc(k)) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(tempP(
     &                        imin_recv(1,k):imax_recv(1,k),
     &                        imin_recv(2,k):imax_recv(2,k),
     &                        imin_recv(3,k):imax_recv(3,k),
     &                        imin_recv(4,k):imax_recv(4,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,statut,code)
C
            End If 
C
        End If
C
      End Do          
C
#else
C
      tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &      indmin(3):indmax(3),indmin(4):indmax(4)) = 
     &parent % var % array4(indmin(1):indmax(1),indmin(2):indmax(2),
     &                      indmin(3):indmax(3),indmin(4):indmax(4))  
C
#endif
C
C             
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
          noraftab(1:nbdim) = 
     &         child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(parentvalues(indmin(1):indmax(1),
     &                          indmin(2):indmax(2),
     &                          indmin(3):indmax(3),
     &                          indmin(4):indmax(4)))
C
          parentvalues = tempP
C
          Call Agrif_CheckMask4D(tempP,parentvalues,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(parentvalues)
C
#else
C
          Call Agrif_CheckMask4D(tempP,parent%var%array4,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           lbound(parent%var%array4),
     &                           ubound(parent%var%array4),
     &                           noraftab(1:nbdim))
C
#endif
C
      End if      
C
C
C     Interpolation of the current grid
#ifdef AGRIF_MPI
      if (value) then
#endif      
      Call Agrif_Interp_4D_recursive(TypeInterp,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)            
#ifdef AGRIF_MPI     
      endif
#endif      
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      lb=lbound(parent%var%array4)
      ub=ubound(parent%var%array4)
      Call Agrif_GlobtoLocInd(childarray,lb,ub,pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)
C
#endif
C
C
C     Special values on the child grid  
      If (Agrif_UseSpecialValueFineGrid) Then
C
#ifdef AGRIF_MPI
C	
          Where (child%var%array4(childarray(pttruetab(1),1,2):
     &                            childarray(cetruetab(1),1,2),
     &                            childarray(pttruetab(2),2,2):
     &                            childarray(cetruetab(2),2,2),
     &                            childarray(pttruetab(3),3,2):
     &                            childarray(cetruetab(3),3,2),
     &                            childarray(pttruetab(4),4,2):
     &                            childarray(cetruetab(4),4,2))
     & 					== Agrif_SpecialValueFineGrid)
C
#else
C
          Where (child%var%array4(pttruetab(1):cetruetab(1),
     &                            pttruetab(2):cetruetab(2),
     &                            pttruetab(3):cetruetab(3),
     &                            pttruetab(4):cetruetab(4))
     & 					== Agrif_SpecialValueFineGrid)              
C
#endif 
C
            tempC(pttruetab(1):cetruetab(1),pttruetab(2):cetruetab(2),
     &            pttruetab(3):cetruetab(3),pttruetab(4):cetruetab(4)) 
     &      = Agrif_SpecialValueFineGrid
C	  
          End Where
C	
      End If
C  
C
C     Results of the interpolation written in child % var % array2
      If (torestore) Then
C
          Do l = pttruetab(4),cetruetab(4)
C
            Do k = pttruetab(3),cetruetab(3)
C
              Do j = pttruetab(2),cetruetab(2)
C
                Do i = pttruetab(1),cetruetab(1) 	
C         
                  If (restore(i,j,k,l) == 0) Then
C
#ifdef AGRIF_MPI
C
                      Do l1 = 1,childarray(l,4,1) 
C
                        Do k1 = 1,childarray(k,3,1)
C
                          Do j1 = 1,childarray(j,2,1)
C
                            Do i1 = 1,childarray(i,1,1)
C
	                      child%var%array4(childarray(i,1,2),
     &                                         childarray(j,2,2),
     &                                         childarray(k,3,2),
     &                                         childarray(l,4,2)) = 
     &                        tempC(i,j,k,l)
C
                            End Do
C
                          End Do
C
                        End do
C
                      End Do  
C
#else 
C          
                      child % var % array4(i,j,k,l) = tempC(i,j,k,l)
C
#endif
C	  
                  End If
C
                End Do
C
              End Do
C
            End Do
C
          End Do
C	
        Else
C
#ifdef AGRIF_MPI
C
          Do l = pttruetab(4),cetruetab(4)  
C
            Do k = pttruetab(3),cetruetab(3)
C
              Do j = pttruetab(2),cetruetab(2)
C
                Do i = pttruetab(1),cetruetab(1)
C
                  Do l1 = 1,childarray(l,4,1) 
C
                    Do k1 = 1,childarray(k,3,1)
C
                      Do j1 = 1,childarray(j,2,1)
C
                        Do i1 = 1,childarray(i,1,1)
C
	                  child%var%array4(childarray(i,1,2),
     &                                     childarray(j,2,2),
     &                                     childarray(k,3,2),
     &                                     childarray(l,4,2)) = 
     &                    tempC(i,j,k,l)
C
                        End Do  
C
                      End Do
C
                    End Do
C
                  End Do 
C
                End Do
C
              End Do 
C
            End do 
C
          End Do            
C
#else 
C       
          child % var % array4(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2),
     &                         pttruetab(3):cetruetab(3),
     &                         pttruetab(4):cetruetab(4)) =
     &    tempC(pttruetab(1):cetruetab(1),
     &          pttruetab(2):cetruetab(2),
     &          pttruetab(3):cetruetab(3),
     &          pttruetab(4):cetruetab(4))
C
#endif 
C     	
      End If
C
C
      Deallocate(tempP)
      Deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      Call MPI_COMM_FREE(New_Comm,code)      
C
C     Deallocations
      Deallocate(tab2)
      Deallocate(parentarray,childarray)      
C
#endif
C
C		  
      End Subroutine Agrif_Interp4D               
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_Interp5D  
C     **************************************************************************  
C  
      Subroutine Agrif_Interp5D(TYPEinterp,parent,child,
     &                          pttab,petab,
     &                          pttab_Child,pttab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          restore,torestore,nbdim)  
C
C     Description:
C     Subroutine to interpolate a 5D grid variable from its parent grid,
C     by using a space interpolation. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: nbdim
      Integer                :: TYPEinterp             ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)  :: parent                 ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)  :: child                  ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: petab              ! Index of the first point inside the domain 
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable      
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids 
      Integer,Dimension(:,:,:,:,:),Pointer :: restore        ! Indicates points where interpolation is done 						     
      Logical                :: torestore              ! Indicates if the array restore is used            
C
C     Local pointers
      Real, Dimension(:,:,:,:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:,:,:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,j,k,l,m,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp 
      Integer,Dimension(nbdim,2)  :: parentglob
      Integer                     :: New_Comm
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: i1,j1,k1,l1,m1,i2,size,pglob
      Logical                 :: value,res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Integer                 :: New_Group,New_Nbprocs,New_Rank
      Logical, Dimension(0:Agrif_Nbprocs-1) :: tab_rangs1
      Integer,Dimension(:),Allocatable :: tab_rangs2
      Real,Dimension(:,:,:,:,:),Allocatable  :: parentvalues
      Integer,Dimension(nbdim) :: lb,ub      
C
#endif      
C     
C   
C     Boundaries of the current grid where interpolation is done

      Call Agrif_Childbounds(nbdim,lbound(child%var%array5),
     &                       ubound(child%var%array5),pttab,petab,
     &                       pttruetab,cetruetab)  
C
#ifdef AGRIF_MPI    
C
C     Creation of a communicator containing all processors concerned by 
C     the interpolation 
C
      value = .true.
C
      Do i = 1,nbdim
C
        If (pttruetab(i) > cetruetab(i)) value = .false.
C
      End Do
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
      Call MPI_ALLGATHER(value,1,MPI_LOGICAL,tab_rangs1,1,
     &                   MPI_LOGICAL,MPI_COMM_WORLD,code)         
C
      New_Nbprocs = COUNT(tab_rangs1)
      
      New_Nbprocs=Agrif_Nbprocs
C
      Allocate(tab_rangs2(0:New_Nbprocs-1))
C
      i2 = 0
C
      Do i1 = 0,Agrif_Nbprocs-1
C
        If (tab_rangs1(i1)) Then
C
c            tab_rangs2(i2) = i1
C
            i2 = i2 + 1
C
        End If
        tab_rangs2(i1) = i1
C
      End Do                  
C
      Call MPI_GROUP_INCL(Agrif_Group,New_Nbprocs,tab_rangs2,
     &                    New_Group,code)
C 
      Call MPI_COMM_CREATE(MPI_COMM_WORLD,New_Group,New_Comm,code)
C
      Deallocate(tab_rangs2)            
C
C
c      If (.not. value) return   ! Current processor is not concerned by the interpolation
C
C
#endif 
C
C     Boundaries of the parent grid for the interpolation
      parentglob(:,1) = lbound(parent % var % array5)
      parentglob(:,2) = ubound(parent % var % array5)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Parentbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                        s_Child_temp,s_Child,ds_Child,
     &                        s_Parent,ds_Parent,
     &                        pttruetab,cetruetab,pttab_Child,
     &                        pttab_Parent,parentglob,NEW_COMM)
C
C
      allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2),
     &               pttruetab(3):cetruetab(3),
     &               pttruetab(4):cetruetab(4),
     &               pttruetab(5):cetruetab(5))) 
C
      allocate(tempP(indmin(1):indmax(1),
     &               indmin(2):indmax(2),
     &               indmin(3):indmax(3),
     &               indmin(4):indmax(4),
     &               indmin(5):indmax(5)))      
C
      tempP = 0.      
C
C
#ifdef AGRIF_MPI     
C
C     tab2 contains the necessary limits of the parent grid for each processor
      Call MPI_GROUP_TRANSLATE_RANKS(Agrif_Group,1,Agrif_ProcRank,
     &                               New_Group,New_Rank,code)     
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(New_Comm,code)
C
      Allocate(tab2(nbdim,2,0:New_Nbprocs-1))
C
      Call MPI_ALLGATHER(tab1,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,New_Comm,code)            
C
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      lb=lbound(parent%var%array5)
      ub=ubound(parent%var%array5)
      Call Agrif_GlobtoLocInd(parentarray,lb,ub,
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)
C
      Call Agrif_ParentGrid_to_ChildGrid()      
C      
C
C     Informations to be sent to the others processors (necessary limits of the parent grid) 
      Call Agrif_Sendtoproc(Agrif_Nbprocs,New_Nbprocs,sendtoproc,nbdim,
     &                      imin,imax,iminloc,imaxloc,parentglob,tab2,
     &                      parentarray) 
C
C
C     imin,imax,iminloc,imaxloc and a part of the parent grid are sent to others processors
      Do k = 0,New_Nbprocs-1        
C
        If (k == New_Rank) Then
C
            tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k),
     &            imin(3,k):imax(3,k),imin(4,k):imax(4,k),
     &            imin(5,k):imax(5,k)) = 
     &      parent%var%array5(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k),
     &                        iminloc(4,k):imaxloc(4,k),
     &                        iminloc(5,k):imaxloc(5,k))
C
          Else        
C
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,code)
C
            If (sendtoproc(k)) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imaxloc(i,k)-iminloc(i,k)+1)
C
                End Do
C
                Call MPI_SEND(parent%var%array5(
     &                        iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k),
     &                        iminloc(4,k):imaxloc(4,k),
     &                        iminloc(5,k):imaxloc(5,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,code)  
C
            End If
C
        End If
C
      End Do  
C
C
C     Reception from others processors of the necessary part of the parent grid
      Do k = 0,New_Nbprocs-1        
C
        If (k /= New_Rank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    New_Comm,statut,code) 
C
            recvfromproc(k) = res       
C
            If (recvfromproc(k)) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        New_Comm,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(tempP(
     &                        imin_recv(1,k):imax_recv(1,k),
     &                        imin_recv(2,k):imax_recv(2,k),
     &                        imin_recv(3,k):imax_recv(3,k),
     &                        imin_recv(4,k):imax_recv(4,k),
     &                        imin_recv(5,k):imax_recv(5,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        New_Comm,statut,code)
C
            End If 
C
        End If
C
      End Do          
C
#else
C
      tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &      indmin(3):indmax(3),indmin(4):indmax(4),
     &      indmin(5):indmax(5)) = 
     &parent % var % array5(indmin(1):indmax(1),indmin(2):indmax(2),
     &                      indmin(3):indmax(3),indmin(4):indmax(4),
     &                      indmin(5):indmax(5))  
C
#endif
C
C             
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
          noraftab(1:nbdim) = 
     &         child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(parentvalues(indmin(1):indmax(1),
     &                          indmin(2):indmax(2),
     &                          indmin(3):indmax(3),
     &                          indmin(4):indmax(4),
     &                          indmin(5):indmax(5)))
C
          parentvalues = tempP
C
          Call Agrif_CheckMask5D(tempP,parentvalues,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(parentvalues)
C
#else
C
          Call Agrif_CheckMask5D(tempP,parent%var%array5,
     &                           indmin(1:nbdim),indmax(1:nbdim),
     &                           lbound(parent%var%array5),
     &                           ubound(parent%var%array5),
     &                           noraftab(1:nbdim))
C
#endif
C
      End if      
C
C
C     Interpolation of the current grid
#ifdef AGRIF_MPI
      if (value) then
#endif      
      Call Agrif_Interp_5D_recursive(TypeInterp,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)            
#ifdef AGRIF_MPI     
      endif
#endif      
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      lb=lbound(parent%var%array5)
      ub=ubound(parent%var%array5)
      Call Agrif_GlobtoLocInd(childarray,lb,ub,pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)
C
#endif
C
C
C     Special values on the child grid  
      If (Agrif_UseSpecialValueFineGrid) Then
C
#ifdef AGRIF_MPI
C	
          Where (child%var%array5(childarray(pttruetab(1),1,2):
     &                            childarray(cetruetab(1),1,2),
     &                            childarray(pttruetab(2),2,2):
     &                            childarray(cetruetab(2),2,2),
     &                            childarray(pttruetab(3),3,2):
     &                            childarray(cetruetab(3),3,2),
     &                            childarray(pttruetab(4),4,2):
     &                            childarray(cetruetab(4),4,2),
     &                            childarray(pttruetab(5),5,2):
     &                            childarray(cetruetab(5),5,2))
     & 					== Agrif_SpecialValueFineGrid)
C
#else
C
          Where (child%var%array5(pttruetab(1):cetruetab(1),
     &                            pttruetab(2):cetruetab(2),
     &                            pttruetab(3):cetruetab(3),
     &                            pttruetab(4):cetruetab(4),
     &                            pttruetab(5):cetruetab(5))
     & 					== Agrif_SpecialValueFineGrid)              
C
#endif 
C
            tempC(pttruetab(1):cetruetab(1),pttruetab(2):cetruetab(2),
     &            pttruetab(3):cetruetab(3),pttruetab(4):cetruetab(4),
     &            pttruetab(5):cetruetab(5)) 
     &      = Agrif_SpecialValueFineGrid
C	  
          End Where
C	
      End If
C  
C
C     Results of the interpolation written in child % var % array2
      If (torestore) Then
C
          Do m = pttruetab(5),cetruetab(5)
C
            Do l = pttruetab(4),cetruetab(4)
C
              Do k = pttruetab(3),cetruetab(3)
C
                Do j = pttruetab(2),cetruetab(2)
C
                  Do i = pttruetab(1),cetruetab(1) 	
C         
                    If (restore(i,j,k,l,m) == 0) Then
C
#ifdef AGRIF_MPI
C
                        Do m1 = 1,childarray(m,5,1)
C
                          Do l1 = 1,childarray(l,4,1) 
C
                            Do k1 = 1,childarray(k,3,1)
C
                              Do j1 = 1,childarray(j,2,1)
C
                                Do i1 = 1,childarray(i,1,1)
C
	                          child%var%array5(childarray(i,1,2),
     &                                             childarray(j,2,2),
     &                                             childarray(k,3,2),
     &                                             childarray(l,4,2),
     &                                             childarray(m,5,2)) = 
     &                            tempC(i,j,k,l,m)
C
                                End Do
C
                              End Do
C
                            End Do
C
                          End Do
C
                        End do
C
#else 
C          
                        child % var % array5(i,j,k,l,m) = 
     &                  tempC(i,j,k,l,m)
C
#endif
C	  
                    End If
C
                  End Do
C
                End Do
C
              End Do
C
            End Do
C
          End Do
C	
        Else
C
#ifdef AGRIF_MPI
C
          Do m = pttruetab(5),cetruetab(5) 
C
            Do l = pttruetab(4),cetruetab(4)  
C
              Do k = pttruetab(3),cetruetab(3)
C
                Do j = pttruetab(2),cetruetab(2)
C
                  Do i = pttruetab(1),cetruetab(1)
C
                    Do m1 = 1,childarray(m,5,1)
C
                      Do l1 = 1,childarray(l,4,1) 
C
                        Do k1 = 1,childarray(k,3,1)
C
                          Do j1 = 1,childarray(j,2,1)
C
                            Do i1 = 1,childarray(i,1,1)
C
	                      child%var%array5(childarray(i,1,2),
     &                                         childarray(j,2,2),
     &                                         childarray(k,3,2),
     &                                         childarray(l,4,2),
     &                                         childarray(m,5,2)) = 
     &                        tempC(i,j,k,l,m)
C
                            End Do 
C
                          End Do
C
                        End Do  
C
                      End Do
C
                    End Do
C
                  End Do 
C
                End Do
C
              End Do 
C
            End do 
C
          End Do            
C
#else 
C       
          child % var % array5(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2),
     &                         pttruetab(3):cetruetab(3),
     &                         pttruetab(4):cetruetab(4),
     &                         pttruetab(5):cetruetab(5)) =
     &    tempC(pttruetab(1):cetruetab(1),
     &          pttruetab(2):cetruetab(2),
     &          pttruetab(3):cetruetab(3),
     &          pttruetab(4):cetruetab(4),
     &          pttruetab(5):cetruetab(5))
C
#endif 
C     	
      End If
C
C
      Deallocate(tempP)
      Deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      Call MPI_COMM_FREE(New_Comm,code)      
C
C     Deallocations
      Deallocate(tab2)
      Deallocate(parentarray,childarray)      
C
#endif
C
C		  
      End Subroutine Agrif_Interp5D
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Childbounds
C     **************************************************************************
C
      Subroutine Agrif_Childbounds(nbdim,lbound,ubound,pttab,petab,
     &                             pttruetab,cetruetab)     
C
CCC   Description:
CCC   Subroutine calculating the global indices of the child grid     
C
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Integer :: nbdim
      Integer,Dimension(nbdim) :: lbound,ubound
      Integer,Dimension(nbdim) :: pttab,petab,pttruetab,cetruetab
C
C     Local variables
      Integer :: i,lbglob,ubglob,indglob1,indglob2
C
C
      Do i = 1,nbdim         
C
        lbglob = lbound(i)
        ubglob = ubound(i)
C
#ifdef AGRIF_MPI
C
        Call AGRIF_InvLoc(lbglob,Agrif_ProcRank,i,indglob1)
C
        Call AGRIF_InvLoc(ubglob,Agrif_ProcRank,i,indglob2)
C
        pttruetab(i) = max(pttab(i),indglob1)        
C
        cetruetab(i) = min(petab(i),indglob2)                 
C
#else
C
        pttruetab(i) = max(pttab(i),lbglob)        
C
        cetruetab(i) = min(petab(i),ubglob)
C
#endif
C
      End Do
C
      Return
C
C
      End Subroutine Agrif_Childbounds
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Parentbounds
C     **************************************************************************
C
      Subroutine Agrif_Parentbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                              s_Child_temp,s_Child,ds_Child,
     &                              s_Parent,ds_Parent,
     &                              pttruetab,cetruetab,pttab_Child,
     &                              pttab_Parent,parentglob,COMM)     
C
CCC   Description:
CCC   Subroutine calculating the bounds of the parent grid for the interpolation
CCC   of the child grid     
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer :: nbdim
      Integer,Dimension(nbdim) :: indmin,indmax
      Real,Dimension(nbdim) :: s_Parent_temp,s_child_temp
      Real,Dimension(nbdim) :: s_Child,ds_child
      Real,Dimension(nbdim) :: s_Parent,ds_Parent
      Integer,Dimension(nbdim) :: pttruetab,cetruetab
      Integer,Dimension(nbdim) :: pttab_Child,pttab_Parent
      Integer,Dimension(nbdim,2) :: parentglob
      Integer :: COMM      
C
C     Local variables
      Integer :: i,lbglob,ubglob,code
      Real,Dimension(nbdim) :: dim_newmin,dim_newmax      
C
C
      Do i = 1,nbdim
C
        dim_newmin(i) = s_Child(i) + (pttruetab(i) - 
     &                                pttab_Child(i)) * ds_Child(i)
C
        dim_newmax(i) = s_Child(i) + (cetruetab(i) - 
     &                                pttab_Child(i)) * ds_Child(i)         
C     
        indmin(i) = pttab_Parent(i) + 
     &                int((dim_newmin(i)-s_Parent(i))/ds_Parent(i))                  
C
        indmax(i) = pttab_Parent(i) - 
     &                floor(-(dim_newmax(i)-s_Parent(i))/ds_Parent(i))
C
C       Necessary for the Lagrange interpolation
        indmin(i) = indmin(i) - 1  
        indmax(i) = indmax(i) + 1        
C
        lbglob = parentglob(i,1)
        ubglob = parentglob(i,2)
C
#ifdef AGRIF_MPI        
C
        Call MPI_ALLREDUCE(parentglob(i,1),lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     COMM,code)  
C
        Call MPI_ALLREDUCE(parentglob(i,2),ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     COMM,code)   
C
#endif
C
        indmin(i) = max(indmin(i),lbglob)
        indmax(i) = min(indmax(i),ubglob)
C
        s_Parent_temp(i) = s_Parent(i) + 
     &                     (indmin(i) - pttab_Parent(i)) * 
     &                      ds_Parent(i) 
C     
        s_Child_temp(i) = s_Child(i) + 
     &                    (pttruetab(i) - pttab_Child(i)) * 
     &                     ds_Child(i)
C
      End Do
C
      Return
C
C
      End Subroutine Agrif_Parentbounds
C
C
C
#ifdef AGRIF_MPI
C
C     **************************************************************************  
CCC   Subroutine Agrif_GlobtoLocInd
C     **************************************************************************
C
      Subroutine Agrif_GlobtoLocInd(tabarray,lboundl,uboundl,tab1,tab2,
     &                              nbdim,rank)     
C
CCC   Description:
CCC   For a global index located on the current processor, tabarray gives the 
CCC   corresponding local index   
C
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Integer :: nbdim
      Integer,Dimension(nbdim) :: tab1,tab2
      Integer,Dimension(minval(tab1):maxval(tab2),nbdim,2 ) :: tabarray
      Integer,Dimension(2) :: lboundl,uboundl
      Integer :: rank
C
C     Local variables
      Integer :: i,i1,j1,k
C
C
      tabarray(:,:,1) = 0
C	
      Do i = 1,nbdim
C       
        Call Agrif_Invloc(lboundl(i),rank,i,i1)
        
        Do k=tab1(i)+lboundl(i)-i1,tab2(i)+lboundl(i)-i1
          tabarray(k-lboundl(i)+i1,i,1)=1
          tabarray(k-lboundl(i)+i1,i,2)=k
        End Do
        
C
      End Do
C
      Return
C
C
      End Subroutine Agrif_GlobtoLocInd
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Sendtoproc
C     **************************************************************************
C
      Subroutine Agrif_Sendtoproc(Agrif_Nbprocs,New_Nbprocs,sendtoproc,
     &                            nbdim,imin,imax,
     &                            iminloc,imaxloc,parentglob,tab,
     &                            tabarray)   
C
CCC   Description:
CCC   Subroutine calculating the limits of parts of the parent grid sent to others processors   
C
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Integer :: Agrif_Nbprocs,New_Nbprocs,nbdim
      Logical,Dimension(0:Agrif_Nbprocs-1) :: sendtoproc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,2) :: parentglob
      Integer,Dimension(nbdim,2,0:Agrif_Nbprocs-1) :: tab
      Integer,Dimension(minval(parentglob(:,1)):maxval(parentglob(:,2)),
     &            nbdim,2) :: tabarray
      
C
C     Local variables
      Integer :: k,i,i1
C
C
      Do k = 0,New_Nbprocs-1
C
        sendtoproc(k) = .true.
C
        Do i = 1,nbdim
C
          imin(i,k) = max(parentglob(i,1),tab(i,1,k))
          imax(i,k) = min(parentglob(i,2),tab(i,2,k))
C
          If (imin(i,k) > imax(i,k)) Then
C
              sendtoproc(k) = .false.
C
            Else
C
              Do i1 = 1,tabarray(imin(i,k),i,1)
C
                iminloc(i,k) = tabarray(imin(i,k),i,i1+1)
C         
              End Do
C
              Do i1 = 1,tabarray(imax(i,k),i,1)
C
                imaxloc(i,k) = tabarray(imax(i,k),i,i1+1)
C
              End Do       
C
          End If
C
        End Do 
C
      End Do       
C
      Return
C
C
      End Subroutine Agrif_Sendtoproc
C
#endif      
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Interp_1D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Interp_1D_recursive(TypeInterp,tabin,tabout,
     &           indmin,indmax, 
     &           pttab_child,petab_child,
     &           s_child,s_parent,ds_child,ds_parent,nbdim)     
C
CCC   Description:
CCC   Subroutine for the interpolation of a 1D grid variable. 
CCC   It calls Agrif_InterpBase. 
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Integer :: nbdim
      Integer :: TypeInterp
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim) :: s_child,s_parent
      Real, Dimension(nbdim) :: ds_child,ds_parent
      Real, Dimension(indmin(nbdim):indmax(nbdim)) :: tabin	
      Real, Dimension(pttab_child(nbdim):petab_child(nbdim)) :: tabout
C
C
C     Commentaire perso : nbdim vaut toujours 1 ici. 
C
      Call Agrif_InterpBase(TypeInterp,
     &                  tabin(indmin(nbdim):indmax(nbdim)),
     &                  tabout(pttab_child(nbdim):petab_child(nbdim)),
     &                  indmin(nbdim),indmax(nbdim),           
     &                  pttab_child(nbdim),petab_child(nbdim),
     &                  s_parent(nbdim),s_child(nbdim),
     &                  ds_parent(nbdim),ds_child(nbdim))
C		
      Return
C
C
      End Subroutine Agrif_Interp_1D_recursive
C
C
C      
C     **************************************************************************  
CCC   Subroutine Agrif_Interp_2D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Interp_2D_recursive(TypeInterp,tabin,tabout,
     &           indmin,indmax,   
     &           pttab_child,petab_child,
     &           s_child,s_parent,ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine for the interpolation of a 2D grid variable. 
CCC   It calls Agrif_Interp_1D_recursive and Agrif_InterpBase.    
C
C     Declarations:
C
      Implicit none
C     
      Integer :: nbdim
      Integer :: TypeInterp
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim) :: s_child,s_parent,ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),
     &                indmin(2):indmax(2)) :: tabin	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2)) :: tabout
C
C     Local variables      
      Real, Dimension(:,:), Allocatable :: tabtemp
      Integer i,j
C
C	
      Allocate(tabtemp(pttab_child(1):petab_child(1),
     &                 indmin(2):indmax(2)))
C
C
C     Commentaire perso : nbdim vaut toujours 2 ici.
C
      Do j = indmin(nbdim),indmax(nbdim)
C	
        Call Agrif_Interp_1D_recursive(TypeInterp,    
     &         tabin(indmin(nbdim-1):indmax(nbdim-1),j),
     &         tabtemp(pttab_child(nbdim-1):petab_child(nbdim-1),j),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C	
      Do i=pttab_child(1),petab_child(1)
C
        Call Agrif_InterpBase(TypeInterp,
     &           tabtemp(i,indmin(nbdim):indmax(nbdim)),
     & 	         tabout(i,pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C        
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C
      End Subroutine Agrif_Interp_2D_recursive
C
C
C      
C     **************************************************************************  
CCC   Subroutine Agrif_Interp_3D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Interp_3D_recursive(TypeInterp,tabin,tabout,
     &           indmin,indmax,   
     &           pttab_child,petab_child,
     &           s_child,s_parent,ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine for the interpolation of a 3D grid variable. 
CCC   It calls Agrif_Interp_2D_recursive and Agrif_InterpBase.    
C
C     Declarations:
C
      Implicit none
C     
      Integer :: nbdim
      Integer :: TypeInterp
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim) :: s_child,s_parent,ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),indmin(2):indmax(2),
     &                indmin(3):indmax(3)) :: tabin	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2),
     &                pttab_child(3):petab_child(3)) :: tabout
C
C     Local variables      
      Real, Dimension(:,:,:), Allocatable :: tabtemp
      Integer i,j,k
C
C	
      Allocate(tabtemp(pttab_child(1):petab_child(1),
     &                 pttab_child(2):petab_child(2),
     &                 indmin(3):indmax(3)))
C
      Do k = indmin(nbdim),indmax(nbdim)
C	
        Call Agrif_Interp_2D_recursive(TypeInterp,    
     &         tabin(indmin(nbdim-2):indmax(nbdim-2),
     &               indmin(nbdim-1):indmax(nbdim-1),k),
     &         tabtemp(pttab_child(nbdim-2):petab_child(nbdim-2),
     &                 pttab_child(nbdim-1):petab_child(nbdim-1),k),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C
      Do j=pttab_child(2),petab_child(2) 
C	
        Do i=pttab_child(1),petab_child(1)
C
          Call Agrif_InterpBase(TypeInterp,
     &           tabtemp(i,j,indmin(nbdim):indmax(nbdim)),
     & 	         tabout(i,j,pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C
        End Do 
C       
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C	
C
      End Subroutine Agrif_Interp_3D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Interp_4D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Interp_4D_recursive(TypeInterp,tabin,tabout,
     &           indmin,indmax,   
     &           pttab_child,petab_child,
     &           s_child,s_parent,ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine for the interpolation of a 4D grid variable. 
CCC   It calls Agrif_Interp_3D_recursive and Agrif_InterpBase.    
C
C     Declarations:
C
      Implicit none
C     
      Integer :: nbdim
      Integer :: TypeInterp
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim) :: s_child,s_parent,ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),indmin(2):indmax(2),
     &                indmin(3):indmax(3),indmin(4):indmax(4)) :: tabin	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2),
     &                pttab_child(3):petab_child(3),
     &                pttab_child(4):petab_child(4)) :: tabout
C
C     Local variables      
      Real, Dimension(:,:,:,:), Allocatable :: tabtemp
      Integer i,j,k,l
C
C	
      Allocate(tabtemp(pttab_child(1):petab_child(1),
     &                 pttab_child(2):petab_child(2),
     &                 pttab_child(3):petab_child(3),  
     &                 indmin(4):indmax(4)))
C
      Do l = indmin(nbdim),indmax(nbdim)
C	
        Call Agrif_Interp_3D_recursive(TypeInterp,    
     &         tabin(indmin(nbdim-3):indmax(nbdim-3),
     &               indmin(nbdim-2):indmax(nbdim-2),
     &               indmin(nbdim-1):indmax(nbdim-1),l),
     &         tabtemp(pttab_child(nbdim-3):petab_child(nbdim-3),
     &                 pttab_child(nbdim-2):petab_child(nbdim-2),
     &                 pttab_child(nbdim-1):petab_child(nbdim-1),l),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C
      Do k = pttab_child(3),petab_child(3)
C
        Do j = pttab_child(2),petab_child(2) 
C	
          Do i = pttab_child(1),petab_child(1)
C
            Call Agrif_InterpBase(TypeInterp,
     &           tabtemp(i,j,k,indmin(nbdim):indmax(nbdim)),
     & 	         tabout(i,j,k,pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C
          End Do
C
        End Do 
C       
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C	
      End Subroutine Agrif_Interp_4D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Interp_5D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Interp_5D_recursive(TypeInterp,tabin,tabout,
     &           indmin,indmax,   
     &           pttab_child,petab_child,
     &           s_child,s_parent,ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine for the interpolation of a 5D grid variable. 
CCC   It calls Agrif_Interp_4D_recursive and Agrif_InterpBase.    
C
C     Declarations:
C
      Implicit none
C     
      Integer :: nbdim
      Integer :: TypeInterp
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim) :: s_child,s_parent,ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),indmin(2):indmax(2),
     &                indmin(3):indmax(3),indmin(4):indmax(4),
     &                indmin(5):indmax(5)) :: tabin	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2),
     &                pttab_child(3):petab_child(3),
     &                pttab_child(4):petab_child(4),
     &                pttab_child(5):petab_child(5)) :: tabout
C
C     Local variables      
      Real, Dimension(:,:,:,:,:), Allocatable :: tabtemp
      Integer i,j,k,l,m
C
C	
      Allocate(tabtemp(pttab_child(1):petab_child(1),
     &                 pttab_child(2):petab_child(2),
     &                 pttab_child(3):petab_child(3),
     &                 pttab_child(4):petab_child(4),    
     &                 indmin(nbdim):indmax(nbdim)))
C
      Do m = indmin(nbdim),indmax(nbdim)
C	
        Call Agrif_Interp_4D_recursive(TypeInterp,    
     &         tabin(indmin(nbdim-4):indmax(nbdim-4),
     &               indmin(nbdim-3):indmax(nbdim-3),
     &               indmin(nbdim-2):indmax(nbdim-2),
     &               indmin(nbdim-1):indmax(nbdim-1),m),
     &         tabtemp(pttab_child(nbdim-4):petab_child(nbdim-4),
     &                 pttab_child(nbdim-3):petab_child(nbdim-3),
     &                 pttab_child(nbdim-2):petab_child(nbdim-2),
     &                 pttab_child(nbdim-1):petab_child(nbdim-1),m),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C
      Do l = pttab_child(4),petab_child(4) 
C
        Do k = pttab_child(3),petab_child(3)
C
          Do j = pttab_child(2),petab_child(2) 
C	
            Do i = pttab_child(1),petab_child(1)
C
              Call Agrif_InterpBase(TypeInterp,
     &             tabtemp(i,j,k,l,indmin(nbdim):indmax(nbdim)),
     & 	           tabout(i,j,k,l,
     &                    pttab_child(nbdim):petab_child(nbdim)),
     &             indmin(nbdim),indmax(nbdim),
     &             pttab_child(nbdim),petab_child(nbdim),
     &             s_parent(nbdim),s_child(nbdim),
     &             ds_parent(nbdim),ds_child(nbdim))
C
            End Do
C
          End Do
C
        End Do 
C       
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C	
      End Subroutine Agrif_Interp_5D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_InterpBase  
C     **************************************************************************  
C  
      Subroutine Agrif_InterpBase(TypeInterp,
     &                           parenttab,childtab,
     &                           indmin,indmax,pttab_child,petab_child,
     &                           s_parent,s_child,ds_parent,ds_child)   
C
CCC   Description:
CCC   Subroutine calling the interpolation method chosen by the user (linear, 
CCC   lagrange or spline). 
C
C     Declarations:
C
      Implicit none
C
      Integer                :: TypeInterp
      Integer :: indmin,indmax
      Integer :: pttab_child,petab_child
      Real,Dimension(indmin:indmax)           :: parenttab       
      Real,Dimension(pttab_child:petab_child) :: childtab      
      Real    :: s_parent,s_child,ds_parent,ds_child 
C 
C
      If (TYPEinterp .EQ. AGRIF_LINEAR) then    
C
C         Linear interpolation         
          Call linear1D
     &         (parenttab,childtab,
     &          indmax-indmin+1,petab_child-pttab_child+1,
     &          s_parent,s_child,ds_parent,ds_child)
C          
        Else If (TYPEinterp .EQ. AGRIF_LAGRANGE) then
C          
C         Lagrange interpolation    
          Call lagrange1D
     &        (parenttab,childtab,
     &         indmax-indmin+1,petab_child-pttab_child+1,
     &         s_parent,s_child,ds_parent,ds_child)
C            
        Else if (TYPEinterp .EQ. AGRIF_SPLINE) then
C          
C         Spline interpolation
          Call spline1D
     &         (parenttab,childtab,
     &         indmax-indmin+1,petab_child-pttab_child+1,
     &         s_parent,s_child,ds_parent,ds_child)
C              
c      else if (TYPEinterp .EQ. 'AGRIF_USER_INTERP') then
C              call interpuser1D(tempP,temp1,rho1)          
C
      End If 
C
C      
      End Subroutine Agrif_InterpBase 
C
C
C                        
      End Module Agrif_Interpolation
