!
! $Id: modinterpbasic.F,v 1.3 2004/11/29 13:26:08 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Interpbasic
C
      Module Agrif_Interpbasic
C
CCC   Description:
CCC   Module containing different procedures of interpolation (linear,lagrange,
CCC   spline,...) used in the Agrif_Interpolation module.
C
C     Modules used:
C             
      CONTAINS
C     Define procedures contained in this module
C 
C
C     **************************************************************************  
CCC   Subroutine Linear1d  
C     ************************************************************************** 
C 
      Subroutine Linear1d(x,y,np,nc,
     &                    s_parent,s_child,ds_parent,ds_child) 
C
CCC   Description:
CCC   Subroutine to do a linear 1D interpolation on a child grid (vector y) from
CCC   its parent grid (vector x).  
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C        
C     Arguments
      Integer             :: np,nc      
      Real, Dimension(np) :: x      
      Real, Dimension(nc) :: y  
      Real                :: s_parent,s_child,ds_parent,ds_child
C
C     Local scalars
      Integer :: i,coeffraf,locind_parent_left
      Real    :: ypos,globind_parent_left
C
C

      coeffraf = nint(ds_parent/ds_child)
C
      If (coeffraf == 1) Then
C
          locind_parent_left = 1 + int((s_child - s_parent)/ds_parent)
C        
          y(1:nc) = x(locind_parent_left:locind_parent_left+nc-1)
C
          return
C
      End If                          
C
      ypos = s_child      
C
      Do i = 1,nc-1
C
        locind_parent_left = 1 + int((ypos - s_parent)/ds_parent)
C
        globind_parent_left = s_parent 
     &                        + (locind_parent_left - 1)*ds_parent
C        
        y(i) = ((globind_parent_left + ds_parent - ypos)
     &          *x(locind_parent_left)
     &        + (ypos - globind_parent_left)
     &          *x(locind_parent_left+1))
     &         / ds_parent                
C
        ypos = ypos + ds_child
C
      End Do
C
      locind_parent_left = 1 + int((ypos - s_parent)/ds_parent)
C
      If (locind_parent_left == np) Then
C
          y(nc) = x(np)
C
        Else
C
          globind_parent_left = s_parent 
     &                        + (locind_parent_left - 1)*ds_parent  
C      
          y(nc) = ((globind_parent_left + ds_parent - ypos)
     &            *x(locind_parent_left)
     &          + (ypos - globind_parent_left)
     &            *x(locind_parent_left+1))
     &           / ds_parent     
C
      End If                                          
C           
      Return
C
C       
      End Subroutine Linear1d   
C
C
C
C     **************************************************************************  
CCC   Subroutine Lagrange1d  
C     **************************************************************************
C
      Subroutine Lagrange1d(x,y,np,nc,
     &                      s_parent,s_child,ds_parent,ds_child)
C
CCC   Description:
CCC   Subroutine to do a lagrange 1D interpolation on a child grid (vector y) 
CCC   from its parent grid (vector x).  
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C             
C     Arguments
      Integer             :: np,nc      
      Real, Dimension(np) :: x      
      Real, Dimension(nc) :: y  
      Real                :: s_parent,s_child,ds_parent,ds_child 
C
C     Local scalars
      Integer :: i,coeffraf,locind_parent_left
      Real    :: ypos,globind_parent_left
      Real    :: X1,X2,X3 
C
C 
      If (np <= 2) Then
C      
          Call Linear1D(x,y,np,nc,
     &                  s_parent,s_child,ds_parent,ds_child)
C         
         Return
C 
      End If
C
      coeffraf = nint(ds_parent/ds_child)
C
      If (coeffraf == 1) Then
C
          locind_parent_left = 1 + int((s_child - s_parent)/ds_parent)
C        
          y(1:nc) = x(locind_parent_left:locind_parent_left+nc-1)
C
          return
C
      End If
C
      ypos = s_child      
C
      Do i = 1,nc
C
        locind_parent_left = 1 + int((ypos - s_parent)/ds_parent)
C
        globind_parent_left = s_parent 
     &                        + (locind_parent_left - 1)*ds_parent 
C
        If (locind_parent_left+2 <= np) Then            
C
            X1 = (x(locind_parent_left+1)-x(locind_parent_left))
     &           /ds_parent
C 
            X2 = (x(locind_parent_left+2)-x(locind_parent_left+1))
     &           /ds_parent
C
            X3 = (X2 - X1)/(2.*ds_parent)                 
C
            y(i) = x(locind_parent_left) + 
     &             (ypos - globind_parent_left)*X1 +
     &             (ypos - globind_parent_left)*
     &             (ypos - globind_parent_left - ds_parent)*X3 
C
          Else If (locind_parent_left+1 <= np) Then 
C
            X1 = (x(locind_parent_left)-x(locind_parent_left-1))
     &           /ds_parent
C     
            X2 = (x(locind_parent_left+1)-x(locind_parent_left))
     &           /ds_parent
C 
            X3 = (X2 - X1)/(2.*ds_parent)                 
C
            y(i) = x(locind_parent_left-1) + 
     &             (ypos - globind_parent_left - ds_parent)*X1 +
     &             (ypos - globind_parent_left - ds_parent)*
     &             (ypos - globind_parent_left)*X3
C
          Else
C
            X1 = (x(locind_parent_left-1)-x(locind_parent_left-2))
     &           /ds_parent
C     
            X2 = (x(locind_parent_left)-x(locind_parent_left-1))
     &           /ds_parent
C 
            X3 = (X2 - X1)/(2.*ds_parent)                 
C
            y(i) = x(locind_parent_left-2) + 
     &             (ypos - globind_parent_left - 2.*ds_parent)*X1 +
     &             (ypos - globind_parent_left - 2.*ds_parent)*
     &             (ypos - globind_parent_left - ds_parent)*X3
C
        End If   
C       
        ypos = ypos + ds_child 
C
      End Do
C
      return
C
C       
      End Subroutine Lagrange1d 
C
C
C
C     **************************************************************************  
CCC   Subroutine Spline1d  
C     **************************************************************************
C
      Subroutine Spline1d(x,y,np,nc,
     &                    s_parent,s_child,ds_parent,ds_child)
C
CCC   Description:
CCC   Subroutine to do a 1D interpolation by using a cubic spline. It is done on
CCC   a child grid (vector y) from its parent grid (vector x).  
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C             
C     Arguments
      Integer             :: np,nc      
      Real, Dimension(np) :: x      
      Real, Dimension(nc) :: y  
      Real                :: s_parent,s_child,ds_parent,ds_child 
C
C     Local scalars
      Integer :: i,coeffraf,locind_parent_left
      Real    :: ypos,globind_parent_left
      Real    :: X1,X2
C     Local arrays
      Real, Dimension(1:np)   :: p 
      Real, Dimension(2:np-1) :: A,B,C,F 
C
C 
      If (np <= 2) Then
C      
          Call Linear1D(x,y,np,nc,
     &                  s_parent,s_child,ds_parent,ds_child)
C         
         Return
C 
      End If
C
      coeffraf = nint(ds_parent/ds_child)
C
      If (coeffraf == 1) Then
C
          locind_parent_left = 1 + int((s_child - s_parent)/ds_parent)
C        
          y(1:nc) = x(locind_parent_left:locind_parent_left+nc-1)
C
          return
C
      End If
C
C
      Do i = 2,np-1
C
        F(i) = (3./ds_parent)*(x(i+1)-x(i-1))
C
      End Do        
C     
      A(2:np-1) = 1. ! Diagonale inferieure
      B(2:np-1) = 4. ! Diagonale principale
      C(2:np-1) = 1. ! Diagonale superieure        
C      
      Call Gauss(np-2,A(2:np-1),B(2:np-1),
     &                C(2:np-1),F(2:np-1),p(2:np-1))  
C
      p(1) = (1/2.)*(3*(x(2)-x(1))/ds_parent - p(2))
      
      p(np) = (1/2.)*(3*(x(np)-x(np-1))/ds_parent - p(np-1))                  
C           
C
      ypos = s_child      
C
      Do i = 1,nc
C
        locind_parent_left = 1 + int((ypos - s_parent)/ds_parent)
C
        globind_parent_left = s_parent 
     &                        + (locind_parent_left - 1)*ds_parent 
C
        If (locind_parent_left+1 <= np) Then 
C
            X1 = (x(locind_parent_left+1)-x(locind_parent_left))
     &           /ds_parent                 
C
            X2 = (p(locind_parent_left+1) - X1)*
     &           (ypos-globind_parent_left) + 
     &           (p(locind_parent_left) - X1)*
     &           (ypos-globind_parent_left-ds_parent)       
C
            y(i) = x(locind_parent_left) + 
     &             (ypos - globind_parent_left)*X1 +
     &             (ypos - globind_parent_left)*
     &             (ypos - globind_parent_left - ds_parent)*X2 / 
     &             (ds_parent)*(ds_parent)
C
          Else
C
            X1 = (x(locind_parent_left)-x(locind_parent_left-1))
     &           /ds_parent                 
C
            X2 = (p(locind_parent_left) - X1)*
     &           (ypos-globind_parent_left-ds_parent) + 
     &           (p(locind_parent_left-1) - X1)*
     &           (ypos-globind_parent_left)       
C
            y(i) = x(locind_parent_left-1) + 
     &             (ypos - globind_parent_left - ds_parent)*X1 +
     &             (ypos - globind_parent_left - ds_parent)*
     &             (ypos - globind_parent_left)*X2 / 
     &             (ds_parent)*(ds_parent)  
C
        End If   
C       
        ypos = ypos + ds_child 
C
      End Do
C
      return
C
C       
      End Subroutine Spline1d      
C
C
C
C     **************************************************************************  
CCC   Subroutine Gauss  
C     **************************************************************************
C
      Subroutine Gauss
     &           (N,A,B,C,F,X)
C
CCC   Description:
CCC   Subroutine to solve a linear system by using the Gauss method for a 
CCC   triangular matrix. The three diagonals are A,B,C, B being the main one.
CCC   F is the second member.   
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C             
C     Arguments 
      Integer              :: N
      Real, Dimension(N)   :: A,B,F,X  ! A va de 2 a n, B de 1 a n, C de 1 a n-1
      Real, Dimension(N)   :: C
C
C     Local variable
      Integer              :: i
      Real                 :: piv        
C
C
      F(1) = F(1)/B(1)
      C(1) = C(1)/B(1)
C      
      Do i = 2,N      
C
        piv = B(i) - A(i)*C(i-1)
C        
        F(i) = (F(i) - A(i)*F(i-1))/piv
C        
        If (i < N) Then
C        
            C(i) = C(i)/piv
C            
        End If
C        
      End Do
C      
      X(N) = F(N)
C      
      Do i = N-1,1,-1
C      
        X(i) = F(i) - C(i)*X(i+1)
C        
      End Do
C
      return      
C
C
      End Subroutine Gauss
C
C
C    
      End Module Agrif_Interpbasic
