!
! $Id: modsauv.F,v 1.2 2003/12/17 13:56:11 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Save  
C
#include "keys_agrif.h"
C
      Module Agrif_Save 
C
CCC   Description:   
CCC   Module for the initialization by copy of the grids created by clustering.
C
C     Modules used:
C      
      Use Agrif_types      
C
C
      Contains
C     Definition of procedures contained in this module.
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Free_data_before
C     **************************************************************************
C
      Subroutine Agrif_Free_data_before(Agrif_Gr)
C
CCC   Description:
CCC   
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(Agrif_Grid),pointer   :: Agrif_Gr        ! Pointer on the current grid
C
C      
#include "deallocations_agrif_before.h"      
C 
C 
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C
#if defined Agrif_Probdim_1D
C
      Deallocate(Agrif_Gr%tabpoint1D)      
C
#endif
C
#if defined Agrif_Probdim_2D
C
      Deallocate(Agrif_Gr%tabpoint2D)      
C
#endif
C
#if defined Agrif_Probdim_3D
C
      Deallocate(Agrif_Gr%tabpoint3D)      
C
#endif
C
#endif     
C  
C          
      Return
C
C
      End Subroutine Agrif_Free_data_before
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Free_data_after
C     **************************************************************************      
C      
      Subroutine Agrif_Free_data_after(Agrif_Gr)
C
CCC   Description:
CCC   
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Pointer argument    
      Type(Agrif_Grid),pointer   :: Agrif_Gr        ! Pointer on the current grid
C
C      
#include "deallocations_agrif_after.h"       
C
C      
      Return
C
C
      End Subroutine Agrif_Free_data_after      
C
C
C
C     **************************************************************************
CCC   Subroutine AGRIF_CopyFromold_All
C     **************************************************************************      
C
      Recursive Subroutine AGRIF_CopyFromold_All(g,oldchildgrids)
C
CCC   Description:
CCC   Routine called in the Agrif_Init_Hierarchy procedure (Agrif_Clustering module). 
C
CC    Method:        
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(AGRIF_grid),pointer   :: g ! Pointer on the current grid
      Type(AGRIF_pgrid),pointer   :: oldchildgrids
C
C     Local pointer
      Type(AGRIF_pgrid),pointer  :: parcours ! Pointer for the recursive 
                                             ! procedure 
      Real g_eps,eps,oldgrid_eps
C
C                                                                           
      parcours => oldchildgrids  
C
      Do while (associated(parcours))
C 
        If ((.not. g % fixed) .and. (parcours % gr %oldgrid)) Then
C        
            g_eps = g % Agrif_dx
C          
            oldgrid_eps = parcours % gr % Agrif_dx
C          
            If (Agrif_Probdim .GE. 2) Then
C          
                g_eps = min(g_eps,g % Agrif_dy)
C            
                oldgrid_eps = min(oldgrid_eps,parcours % gr % Agrif_dy)
C            
                If (Agrif_Probdim .GE. 3) Then
C              
                    g_eps = min(g_eps,g % Agrif_dz)
C              
                    oldgrid_eps 
     &              = min(oldgrid_eps,parcours % gr % Agrif_dz)            
C            
                End If
C          
            End If
C
            eps = min(g_eps,oldgrid_eps)/100.                  
C
            If (g % Agrif_dx .LT. 
     &          (parcours % gr % Agrif_dx - eps)) Then
C            
                parcours => parcours % next
C            
                Cycle
C          
            End If
C      
            If (Agrif_Probdim .GE. 2) Then
C        
                If (g % Agrif_dy .LT. 
     &              (parcours % gr % Agrif_dy - eps)) Then
C            
                    parcours => parcours % next
C                    
                    Cycle
C 
                End If       
C          
                If (Agrif_Probdim .GE. 3) Then
C            
                    If (g % Agrif_dz .LT. 
     &                  (parcours % gr % Agrif_dz - eps)) Then
C                  
                        parcours => parcours % next
C                        
                        Cycle
C 
                    End If
C          
                 End If         
C      
            End If             
C
            Call AGRIF_CopyFromOld(g,parcours%gr)
C
        End If
C                     
        Call Agrif_CopyFromold_All
     &             (g, parcours % gr % child_grids)
C        
        parcours => parcours % next
C        
      End Do
C
C     
      Return      
C
C
      End Subroutine AGRIF_CopyFromold_All
C
C
C      
C     **************************************************************************
CCC   Subroutine Agrif_CopyFromold
C     **************************************************************************
C      
      Subroutine Agrif_CopyFromold(Agrif_New_Gr,Agrif_Old_Gr)
C
CCC   Description:
CCC   Call to the Agrif_Copy procedure.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(Agrif_Grid),Pointer   :: Agrif_New_Gr     ! Pointer on the current grid
      Type(Agrif_Grid), Pointer :: Agrif_Old_Gr      ! Pointer on an old grid
C
C      
#include "copylevel_agrif.h"       
C      
C            
      Return
C
C      
      End Subroutine Agrif_CopyFromold
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Copy
C     **************************************************************************
C      
      Subroutine Agrif_Copy(Agrif_New_Gr,Agrif_Old_Gr,New_Var,Old_Var)
C     
CCC   Description:
CCC   Sets arguments of the Agrif_UpdatenD procedures, n being the number of 
CCC   dimension of the grid variable.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
C     Pointer argument    
      Type(Agrif_Grid),Pointer   :: Agrif_New_Gr        ! Pointer on the current grid
      Type(Agrif_Grid), Pointer :: Agrif_Old_Gr      ! Pointer on an old grid
      Type(Agrif_Pvariable) :: New_Var, Old_Var
      Integer :: nbdim                  ! Number of dimensions of the current 
                                        ! grid      
      Integer,Dimension(5) :: pttabnew     ! Indexes of the first point in the 
                                        ! domain
      Integer,Dimension(5) :: petabnew     ! Indexes of the first point in the 
                                        ! domain
      Integer,Dimension(5) :: pttabold     ! Indexes of the first point in the 
                                        ! domain
      Integer,Dimension(5) :: petabold     ! Indexes of the first point in the 
                                        ! domain                                                            
      Integer,Dimension(5) :: itab      ! Minimal positions of the current grid  
                                        ! on its parent grid      
      Integer,Dimension(5) :: nbtabold     ! Number of cells in each direction
      
      Integer,Dimension(5) :: nbtabnew     ! Number of cells in each direction      
            
      Integer,Dimension(5) :: posvartab ! Position of the variable on a cell
      Type(AGRIF_Variable), Pointer :: root ! Pointer on the variable of the 
                                            ! root grid
      Real, Dimension(5) :: snew,sold
      Real, Dimension(5) :: dsnew,dsold                              
      Real :: eps                                                                          
      Integer :: ji,jj,jk,n
C
C  
      root => New_Var % var % root_var 
C      
      nbdim = root % nbdim
C         
      Do n=1,nbdim
C  
        select case(root % interptab(n))
C        
        case('x')                
C
          pttabnew(n) = root % pointx
C        
          pttabold(n) = root % pointx
C        
          snew(n) = Agrif_New_Gr % Agrif_x
C
          sold(n) = Agrif_Old_Gr % Agrif_x 
C        
          dsnew(n) = Agrif_New_Gr % Agrif_dx
C
          dsold(n) = Agrif_Old_Gr % Agrif_dx
C                      
          If (root % posvar(n) .EQ. 1) Then
C          
              petabnew(n) = pttabnew(n) + Agrif_New_Gr % nbx
C          
              petabold(n) = pttabold(n) + Agrif_Old_Gr % nbx         
C        
            Else
C          
              petabnew(n) = pttabnew(n) + Agrif_New_Gr % nbx - 1
C          
              petabold(n) = pttabold(n) + Agrif_Old_Gr % nbx - 1
C
              snew(n) = snew(n) + dsnew(n)/2.
C
              sold(n) = sold(n) + dsold(n)/2.                                
C        
          End If           
C        
        case('y')
C        
          pttabnew(n) = root % pointy
C        
          pttabold(n) = root % pointy
C        
          snew(n) = Agrif_New_Gr % Agrif_y
C        
          sold(n) = Agrif_Old_Gr % Agrif_y 
C        
          dsnew(n) = Agrif_New_Gr % Agrif_dy
C        
          dsold(n) = Agrif_Old_Gr % Agrif_dy
C                
          If (root % posvar(n) .EQ. 1) Then
C        
              petabnew(n) = pttabnew(n) + Agrif_New_Gr % nby
C          
              petabold(n) = pttabold(n) + Agrif_Old_Gr % nby         
C        
            Else
C          
              petabnew(n) = pttabnew(n) + Agrif_New_Gr % nby - 1
C          
              petabold(n) = pttabold(n) + Agrif_Old_Gr % nby - 1
C
              snew(n) = snew(n) + dsnew(n)/2.
C
              sold(n) = sold(n) + dsold(n)/2.                    
C        
          End If 
C        
        case('z')
C        
          pttabnew(n) = root % pointz
C        
          pttabold(n) = root % pointz
C        
          snew(n) = Agrif_New_Gr % Agrif_z
C        
          sold(n) = Agrif_Old_Gr % Agrif_z 
C        
          dsnew(n) = Agrif_New_Gr % Agrif_dz
C        
          dsold(n) = Agrif_Old_Gr % Agrif_dz
C                
          If (root % posvar(n) .EQ. 1) Then
C        
              petabnew(n) = pttabnew(n) + Agrif_New_Gr % nbz
C          
              petabold(n) = pttabold(n) + Agrif_Old_Gr % nbz         
C        
            Else
C          
             petabnew(n) = pttabnew(n) + Agrif_New_Gr % nbz - 1
C          
             petabold(n) = pttabold(n) + Agrif_Old_Gr % nbz - 1
C
             snew(n) = snew(n) + dsnew(n)/2.
C
             sold(n) = sold(n) + dsold(n)/2.                   
C        
          End If
C          
        case('noint')
C        
          select case (nbdim) 
C        
          case(1)
C        
            pttabnew(n) = lbound(New_Var % var % array1,n)
C        
            petabnew(n) = ubound(New_Var % var % array1,n)
C          
          case(2)
C          
            pttabnew(n) = lbound(New_Var % var % array2,n)
C        
            petabnew(n) = ubound(New_Var % var % array2,n)
C          
          case(3)
C        
            pttabnew(n) = lbound(New_Var % var % array3,n)
C          
            petabnew(n) = ubound(New_Var % var % array3,n)
C          
          case(4)
C        
            pttabnew(n) = lbound(New_Var % var % array4,n)
C          
            petabnew(n) = ubound(New_Var % var % array4,n)
C            
          case(5)
C        
            pttabnew(n) = lbound(New_Var % var % array5,n)
C          
            petabnew(n) = ubound(New_Var % var % array5,n)      
C        
          end select
C        
          pttabold(n) = pttabnew(n)
C      
          petabold(n) = petabnew(n)
C              
          snew(n) = 0.
C      
          sold(n) = 0. 
C      
          dsnew(n) = 1.
C      
          dsold(n) = 1.
C        
        end select
C      
      End do
C      
      Do n = 1,nbdim
C      
        nbtabnew(n) = petabnew(n) - pttabnew(n)
C      
        nbtabold(n) = petabold(n) - pttabold(n)      
C 
      End Do      
C
      eps = min(minval(dsnew(1:nbdim)),minval(dsold(1:nbdim)))
C     
      eps = eps/100.      
C      
      Do n = 1,nbdim
C      
        If (snew(n) .GT. (sold(n)+dsold(n)*nbtabold(n)+eps)) Return
C      
        If ((snew(n)+dsnew(n)*nbtabnew(n)-eps) .LT. sold(n)) Return        
C      
      End Do
C            
      If (nbdim .EQ. 1) Then
C      
         Call AGRIF_Copy1D
     &        (New_Var,Old_Var,pttabold,petabold,pttabnew,petabnew,
     &         sold,snew,dsold,dsnew,eps)
C          
        Else If (nbdim .EQ. 2) Then        
C      
         Call AGRIF_Copy2D
     &        (New_Var,Old_Var,pttabold,petabold,pttabnew,petabnew,
     &         sold,snew,dsold,dsnew,eps)
C
        Else If (nbdim .EQ. 3) Then
C
         Call AGRIF_Copy3D
     &        (New_Var,Old_Var,pttabold,petabold,pttabnew,petabnew,
     &         sold,snew,dsold,dsnew,eps) 
C
        Else If (nbdim .EQ. 4) Then
C
         call AGRIF_Copy4D
     &        (New_Var,Old_Var,pttabold,petabold,pttabnew,petabnew,
     &         sold,snew,dsold,dsnew,eps)  
C
        Else If (nbdim .EQ. 5) Then
C
         Call AGRIF_Copy5D
     &        (New_Var,Old_Var,pttabold,petabold,pttabnew,petabnew,
     &         sold,snew,dsold,dsnew,eps) 
C      
      End If
C            
C 
      Return
C      
C
      End Subroutine Agrif_Copy
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Copy1D
C     **************************************************************************
C      
      Subroutine Agrif_Copy1D(New_Var,Old_Var,pttabold,petabold,
     &                        pttabnew,petabnew,sold,snew,dsold,
     &                        dsnew,eps)
C     
CCC   Description:
CCC   Copy of the 1D New_Var variable from the 1D Old_Var variable.    
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
      Type(Agrif_Pvariable) :: New_Var, Old_Var     
      Integer,Dimension(5) :: pttabnew 
      Integer,Dimension(5) :: petabnew
      Integer,Dimension(5) :: pttabold
      Integer,Dimension(5) :: petabold      
      Real, Dimension(5) :: snew,sold
      Real, Dimension(5) :: dsnew,dsold 
      Real :: eps 
C
      Integer :: i,i0   
C
      Real, Dimension(1) :: dim_gmin,dim_gmax
      Real, Dimension(1) :: dim_newmin,dim_newmax
      Real, Dimension(1) :: dim_min,dim_max
      Integer, Dimension(1) :: ind_gmin,ind_newmin,ind_gmax,ind_newmax
C               
C
      dim_gmin(1) = sold(1)
      dim_gmax(1) = sold(1) + (petabold(1)-pttabold(1)) * dsold(1)      
C 
      dim_newmin(1) = snew(1)
      dim_newmax(1) = snew(1) + (petabnew(1)-pttabnew(1)) * dsnew(1)         
C
      If (dim_gmax(1).LT.dim_newmin(1)) Return
C      
      If (dim_gmin(1).GT.dim_newmax(1)) Return
C
C
      ind_newmin(1) = pttabnew(1)-floor(-(max(dim_gmin(1),dim_newmin(1))
     &                    -dim_newmin(1))/dsnew(1)) 
C      
      dim_min(1) = snew(1) + (ind_newmin(1)-pttabnew(1))*dsnew(1)          
C      
      ind_gmin(1) = pttabold(1) 
     &              + nint((dim_min(1)-dim_gmin(1))/dsold(1))      
C
      ind_newmax(1) = pttabnew(1) + int((min(dim_gmax(1),dim_newmax(1))
     &                      -dim_newmin(1))/dsnew(1))
C      
      dim_max(1) = snew(1) + (ind_newmax(1)-pttabnew(1))*dsnew(1) 
C 
      ind_gmax(1) = pttabold(1) 
     &              + nint((dim_max(1)-dim_gmin(1))/dsold(1)) 
C
C
      i0 = ind_gmin(1)
C        
      Do i = ind_newmin(1),ind_newmax(1)
C          
        New_Var % var % array1(i) =
     &                         Old_Var % var % array1(i0)
C  
        New_Var % var % restore1D(i) = 1   
C                   
        i0 = i0 + int(dsnew(1)/dsold(1))
C               
      End Do
C
C           
      Return
C
C
      End Subroutine Agrif_Copy1D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Copy2D
C     **************************************************************************
C      
      Subroutine Agrif_Copy2D(New_Var,Old_Var,pttabold,petabold,
     &                        pttabnew,petabnew,sold,snew,dsold,
     &                        dsnew,eps)
C     
CCC   Description:
CCC   Copy of the 2D New_Var variable from the 2D Old_Var variable. 
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
      Type(Agrif_Pvariable) :: New_Var, Old_Var     
      Integer,Dimension(5) :: pttabnew 
      Integer,Dimension(5) :: petabnew
      Integer,Dimension(5) :: pttabold
      Integer,Dimension(5) :: petabold      
      Real, Dimension(5) :: snew,sold
      Real, Dimension(5) :: dsnew,dsold 
      Real :: eps 
C
      Integer :: i,j,i0,j0   
C
      Real, Dimension(2) :: dim_gmin,dim_gmax
      Real, Dimension(2) :: dim_newmin,dim_newmax
      Real, Dimension(2) :: dim_min,dim_max
      Integer, Dimension(2) :: ind_gmin,ind_newmin,ind_gmax,ind_newmax
C               
C
      Do i = 1,2
C      
        dim_gmin(i) = sold(i)
        dim_gmax(i) = sold(i) + (petabold(i)-pttabold(i)) * dsold(i)      
C 
        dim_newmin(i) = snew(i)
        dim_newmax(i) = snew(i) + (petabnew(i)-pttabnew(i)) * dsnew(i)
C
      End Do         
C
      Do i = 1,2
C      
        If (dim_gmax(i) .LT. dim_newmin(i)) Return
C      
        If (dim_gmin(i) .GT. dim_newmax(i)) Return
C 
      End Do
C
C
      Do i = 1,2
C 
        ind_newmin(i) = pttabnew(i) - floor(-(max(dim_gmin(i),
     &                       dim_newmin(i))-dim_newmin(i))/dsnew(i)) 
C      
        dim_min(i) = snew(i) + (ind_newmin(i)-pttabnew(i))*dsnew(i)          
C      
        ind_gmin(i) = pttabold(i) + nint((dim_min(i)
     &                - dim_gmin(i))/dsold(i))      
C
        ind_newmax(i) = pttabnew(i) 
     &                  + int((min(dim_gmax(i),dim_newmax(i))
     &                        - dim_newmin(i))/dsnew(i))
C      
        dim_max(i) = snew(i) + (ind_newmax(i)-pttabnew(i))*dsnew(i) 
C 
        ind_gmax(i) = pttabold(i) + nint((dim_max(i) -
     &                dim_gmin(i))/dsold(i))
C
      End Do 
C
C
      j0 = ind_gmin(2)
C      
      Do j = ind_newmin(2),ind_newmax(2)
C      
        i0 = ind_gmin(1)
C        
        Do i = ind_newmin(1),ind_newmax(1)
C
          New_Var % var % array2(i,j) =
     &                         Old_Var % var % array2(i0,j0)
C  
          New_Var % var % restore2D(i,j) = 1   
C                   
          i0 = i0 + int(dsnew(1)/dsold(1))
C               
        End Do
C            
        j0 = j0 + int(dsnew(2)/dsold(2))
C        
      End Do
C
C           
      Return
C
C
      End Subroutine Agrif_Copy2D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Copy3D
C     **************************************************************************
C      
      Subroutine Agrif_Copy3D(New_Var,Old_Var,pttabold,petabold,
     &                        pttabnew,petabnew,sold,snew,dsold,
     &                        dsnew,eps)
C     
CCC   Description:
CCC   Copy of the 3D New_Var variable from the 3D Old_Var variable. 
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
      Type(Agrif_Pvariable) :: New_Var, Old_Var     
      Integer,Dimension(5) :: pttabnew 
      Integer,Dimension(5) :: petabnew
      Integer,Dimension(5) :: pttabold
      Integer,Dimension(5) :: petabold      
      Real, Dimension(5) :: snew,sold
      Real, Dimension(5) :: dsnew,dsold 
      Real :: eps 
C
      Integer :: i,j,k,i0,j0,k0   
C
      Real, Dimension(3) :: dim_gmin,dim_gmax
      Real, Dimension(3) :: dim_newmin,dim_newmax
      Real, Dimension(3) :: dim_min,dim_max
      Integer, Dimension(3) :: ind_gmin,ind_newmin,ind_gmax,ind_newmax
C               
C
      Do i = 1,3
C      
        dim_gmin(i) = sold(i)
        dim_gmax(i) = sold(i) + (petabold(i)-pttabold(i)) * dsold(i)      
C 
        dim_newmin(i) = snew(i)
        dim_newmax(i) = snew(i) + (petabnew(i)-pttabnew(i)) * dsnew(i)
C
      End Do         
C
      Do i = 1,3
C  
        If (dim_gmax(i) .LT. dim_newmin(i)) Return
C      
        If (dim_gmin(i) .GT. dim_newmax(i)) Return
C
      End Do
C
C
      Do i = 1,3
C 
        ind_newmin(i) = pttabnew(i) - floor(-(max(dim_gmin(i),
     &                      dim_newmin(i))-dim_newmin(i))/dsnew(i)) 
C      
        dim_min(i) = snew(i) + (ind_newmin(i)-pttabnew(i))*dsnew(i)          
C      
        ind_gmin(i) = pttabold(i) + nint((dim_min(i)-
     &                             dim_gmin(i))/dsold(i))      
C
        ind_newmax(i) = pttabnew(i) 
     &                + int((min(dim_gmax(i),dim_newmax(i))
     &                      - dim_newmin(i))/dsnew(i))
C      
        dim_max(i) = snew(i) + (ind_newmax(i)-pttabnew(i))*dsnew(i) 
C 
        ind_gmax(i) = pttabold(i) + nint((dim_max(i)-
     &                                     dim_gmin(i))/dsold(i))
C
      End Do 
C
C
      k0 = ind_gmin(3)
C      
      Do k = ind_newmin(3),ind_newmax(3)
C      
        j0 = ind_gmin(2)
C      
        Do j = ind_newmin(2),ind_newmax(2)
C      
          i0 = ind_gmin(1)
C        
          Do i = ind_newmin(1),ind_newmax(1)
C          
            New_Var % var % array3(i,j,k) =
     &                         Old_Var % var % array3(i0,j0,k0)
C  
            New_Var % var % restore3D(i,j,k) = 1   
C                   
            i0 = i0 + int(dsnew(1)/dsold(1))
C               
          End Do
C            
          j0 = j0 + int(dsnew(2)/dsold(2))
C        
        End Do
C      
        k0 = k0 + int(dsnew(3)/dsold(3))
C      
      End Do
C
C           
      Return
C
C
      End Subroutine Agrif_Copy3D   
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Copy4D
C     **************************************************************************
C      
      Subroutine Agrif_Copy4D(New_Var,Old_Var,pttabold,petabold,
     &                        pttabnew,petabnew,sold,snew,dsold,
     &                        dsnew,eps)
C     
CCC   Description:
CCC   Copy of the 4D New_Var variable from the 4D Old_Var variable.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
      Type(Agrif_Pvariable) :: New_Var, Old_Var     
      Integer,Dimension(5) :: pttabnew 
      Integer,Dimension(5) :: petabnew
      Integer,Dimension(5) :: pttabold
      Integer,Dimension(5) :: petabold      
      Real, Dimension(5) :: snew,sold
      Real, Dimension(5) :: dsnew,dsold 
      Real :: eps 
C
      Integer :: i,j,k,l,i0,j0,k0,l0   
C
      Real, Dimension(4) :: dim_gmin,dim_gmax
      Real, Dimension(4) :: dim_newmin,dim_newmax
      Real, Dimension(4) :: dim_min,dim_max
      Integer, Dimension(4) :: ind_gmin,ind_newmin,ind_gmax,ind_newmax
C               
C
      Do i = 1,4
C      
        dim_gmin(i) = sold(i)
        dim_gmax(i) = sold(i) + (petabold(i)-pttabold(i)) * dsold(i)      
C 
        dim_newmin(i) = snew(i)
        dim_newmax(i) = snew(i) + (petabnew(i)-pttabnew(i)) * dsnew(i)
C
      End Do         
C
      Do i = 1,4
C  
        If (dim_gmax(i) .LT. dim_newmin(i)) Return
C      
        If (dim_gmin(i) .GT. dim_newmax(i)) Return
C
      End Do
C
C
      Do i = 1,4
C 
        ind_newmin(i) = pttabnew(i) - floor(-(max(dim_gmin(i),
     &                  dim_newmin(i))-dim_newmin(i))/dsnew(i)) 
C      
        dim_min(i) = snew(i) + (ind_newmin(i)-pttabnew(i))*dsnew(i)          
C      
        ind_gmin(i) = pttabold(i)+nint((dim_min(i)-
     &                dim_gmin(i))/dsold(i))      
C
        ind_newmax(i) = pttabnew(i) 
     &                  + int((min(dim_gmax(i),dim_newmax(i))
     &                         -dim_newmin(i))/dsnew(i))
C      
        dim_max(i) = snew(i) + (ind_newmax(i)-pttabnew(i))*dsnew(i) 
C 
        ind_gmax(i) = pttabold(i) + nint((dim_max(i)-
     &                dim_gmin(i))/dsold(i))
C
      End Do 
C
C
      l0 = ind_gmin(4)
C
      Do l = ind_newmin(4),ind_newmax(4) 
C
        k0 = ind_gmin(3)
C      
        Do k = ind_newmin(3),ind_newmax(3)
C      
          j0 = ind_gmin(2)
C      
          Do j = ind_newmin(2),ind_newmax(2)
C      
            i0 = ind_gmin(1)
C        
            Do i = ind_newmin(1),ind_newmax(1)
C          
              New_Var % var % array4(i,j,k,l) =
     &                         Old_Var % var % array4(i0,j0,k0,l0)
C  
              New_Var % var % restore4D(i,j,k,l) = 1   
C                   
              i0 = i0 + int(dsnew(1)/dsold(1))
C               
            End Do
C            
            j0 = j0 + int(dsnew(2)/dsold(2))
C        
          End Do
C      
          k0 = k0 + int(dsnew(3)/dsold(3))
C      
        End Do
C
        l0 = l0 + int(dsnew(4)/dsold(4))
C
      End Do             
C
C           
      Return
C
C
      End Subroutine Agrif_Copy4D   
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Copy5D
C     **************************************************************************
C      
      Subroutine Agrif_Copy5D(New_Var,Old_Var,pttabold,petabold,
     &                        pttabnew,petabnew,sold,snew,dsold,
     &                        dsnew,eps)
C     
CCC   Description:
CCC   Copy of the 5D New_Var variable from the 5D Old_Var variable.
C
CC    Method:
CC            
C
C     Declarations:
C
      Implicit none
C
      Type(Agrif_Pvariable) :: New_Var, Old_Var     
      Integer,Dimension(5) :: pttabnew 
      Integer,Dimension(5) :: petabnew
      Integer,Dimension(5) :: pttabold
      Integer,Dimension(5) :: petabold      
      Real, Dimension(5) :: snew,sold
      Real, Dimension(5) :: dsnew,dsold 
      Real :: eps 
C
      Integer :: i,j,k,l,m,i0,j0,k0,l0,m0   
C
      Real, Dimension(5) :: dim_gmin,dim_gmax
      Real, Dimension(5) :: dim_newmin,dim_newmax
      Real, Dimension(5) :: dim_min,dim_max
      Integer, Dimension(5) :: ind_gmin,ind_newmin,ind_gmax,ind_newmax
C               
C
      Do i = 1,5
C      
        dim_gmin(i) = sold(i)
        dim_gmax(i) = sold(i) + (petabold(i)-pttabold(i)) * dsold(i)      
C 
        dim_newmin(i) = snew(i)
        dim_newmax(i) = snew(i) + (petabnew(i)-pttabnew(i)) * dsnew(i)
C
      End Do         
C
      Do i = 1,5
C  
        If (dim_gmax(i) .LT. dim_newmin(i)) Return
C      
        If (dim_gmin(i) .GT. dim_newmax(i)) Return
C
      End Do
C
C
      Do i = 1,5
C 
        ind_newmin(i) = pttabnew(i) - floor(-(max(dim_gmin(i),
     &                      dim_newmin(i))-dim_newmin(i))/dsnew(i)) 
C      
        dim_min(i) = snew(i) + (ind_newmin(i)-pttabnew(i))*dsnew(i)          
C      
        ind_gmin(i) = pttabold(i) + nint((dim_min(i)-
     &                dim_gmin(i))/dsold(i))      
C
        ind_newmax(i) = pttabnew(i) 
     &                  + int((min(dim_gmax(i),dim_newmax(i))
     &                         -dim_newmin(i))/dsnew(i))
C      
        dim_max(i) = snew(i) + (ind_newmax(i)-pttabnew(i))*dsnew(i) 
C 
        ind_gmax(i) = pttabold(i) + nint((dim_max(i)-
     &                dim_gmin(i))/dsold(i))
C
      End Do 
C
C
      m0 = ind_gmin(5)
C
      Do m = ind_newmin(5),ind_newmax(5)
C
        l0 = ind_gmin(4)
C
        Do l = ind_newmin(4),ind_newmax(4) 
C
          k0 = ind_gmin(3)
C      
          Do k = ind_newmin(3),ind_newmax(3)
C      
            j0 = ind_gmin(2)
C      
            Do j = ind_newmin(2),ind_newmax(2)
C      
              i0 = ind_gmin(1)
C        
              Do i = ind_newmin(1),ind_newmax(1)
C          
                New_Var % var % array5(i,j,k,l,m) =
     &                         Old_Var % var % array5(i0,j0,k0,l0,m0)
C  
                New_Var % var % restore5D(i,j,k,l,m) = 1   
C                   
                i0 = i0 + int(dsnew(1)/dsold(1))
C               
              End Do
C            
              j0 = j0 + int(dsnew(2)/dsold(2))
C        
            End Do
C      
            k0 = k0 + int(dsnew(3)/dsold(3))
C      
          End Do
C
          l0 = l0 + int(dsnew(4)/dsold(4))
C
        End Do
C
        m0 = m0 + int(dsnew(5)/dsold(5)) 
C
      End Do             
C
C           
      Return
C
C
      End Subroutine Agrif_Copy5D                    
C
C
C
      End module Agrif_Save
