C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Types  
C
      Module Agrif_Types 
C
CCC   Description:
CCC   Definition of data types used in AGRIF, of several variables and
CCC   parameters.     
C
C
C
C     Local scalar
      Integer  AGRIF_Coeffrefttmp
C
C
C
C     **************************************************************************
CCC   Agrif_Probdim : Problem dimension
C     **************************************************************************
#include "probdim_agrif.h" 
C
C
C
C     **************************************************************************   
CCC   Type AGRIF_LRECTANGLE
C     **************************************************************************  
C
CCC   Description:
CCC   Data type allowing a grid to reach a grid on the same level or its child 
CCC   grids.
C
      Type AGRIF_lrectangle  
C      
           Type(AGRIF_rectangle), Pointer  :: r    ! to reach a child grid 
           Type(AGRIF_lrectangle), Pointer :: next ! to reach a grid on the same 
                                                   ! level
C                         
      End Type AGRIF_lrectangle  
C
C
C            
C     **************************************************************************  
CCC   Type AGRIF_RECTANGLE: 
C     **************************************************************************
C
CCC   Description:
CCC   Data type to define several characteristics of a grid (number,position,
CCC   time and space refinement factors,...). 
C
      Type AGRIF_rectangle
C      
           Integer     :: number      ! Number of the grid
           Integer     :: nbgridchild ! Number of child grids 
           Integer     :: imin        ! Minimal position in the x direction
           Integer     :: imax        ! Maximal position in the x direction 
           Integer     :: jmin        ! Minimal position in the y direction
           Integer     :: jmax        ! Maximal position in the y direction
           Integer     :: kmin        ! Minimal position in the z direction
           Integer     :: kmax        ! Maximal position in the z direction 
           Integer     :: spacerefx   ! Space refinement factor in the x direction  
           Integer     :: spacerefy   ! Space refinement factor in the y direction
           Integer     :: spacerefz   ! Space refinement factor in the z direction
           Integer     :: timerefx    ! Time refinement factor in the x direction
           Integer     :: timerefy    ! Time refinement factor in the y direction
           Integer     :: timerefz    ! Time refinement factor in the z direction
           Type(AGRIF_lrectangle), Pointer :: childgrids ! Pointer to reach a grid on the same 
                                                         ! level or a child grid
C  
      End Type AGRIF_rectangle  
C
C
C
C     **************************************************************************
CCC   Type AGRIF_PGrid
C     **************************************************************************  
C
CCC   Description:
CCC   Data type to go over the grid hierarchy (used for the creation of this 
CCC   grid hierarchy and during the time integration).
C 
      Type AGRIF_pgrid  
C      
           Type(AGRIF_grid), Pointer  :: gr   ! allows to reach a child grid  
           Type(AGRIF_pgrid), Pointer :: next ! allows to reach the grids of the
                                              ! same level 
C
      End Type AGRIF_pgrid 
C
C
C
C     **************************************************************************
CCC   Type AGRIF_PVariable
C     ************************************************************************** 
C
CCC   Description:
CCC   Data type to define a grid variable.
C
      Type AGRIF_PVariable 
C       
           Type(AGRIF_Variable), Pointer  :: var  ! This field allows to get the various 
                                                  ! characteristics of the variable (defined by
                                                  ! the Agrif_Variable data type)         
           Type(AGRIF_PVariable), Pointer :: parent_var ! Pointer on the parent grid   
C   
      End Type AGRIF_PVariable 
C
C
C 
C     Space and time refinement factors, number of variables  
#include "modtype_agrif.h"
C
C
C
C     **************************************************************************
CCC   Type AGRIF_Grid
C     ************************************************************************** 
C
CCC   Description:
CCC   Data type to define a grid (position, space and time refinement factors, ...).
C
      Type AGRIF_grid  
C            
           Integer   :: nbx       ! number of cells in the x direction
           Integer   :: nby       ! number of cells in the y direction
           Integer   :: nbz       ! number of cells in the z direction
           Integer   :: ix        ! minimal position in the x direction
           Integer   :: iy        ! minimal position in the y direction
           Integer   :: iz        ! minimal position in the z direction 
           Integer   :: spacerefx ! Space refinement factor in the x direction     
           Integer   :: spacerefy ! Space refinement factor in the y direction
           Integer   :: spacerefz ! Space refinement factor in the z direction
           Integer   :: timerefx  ! Time refinement factor in the x direction 
           Integer   :: timerefy  ! Time refinement factor in the y direction
           Integer   :: timerefz  ! Time refinement factor in the z direction   
           Integer   :: ngridstep ! number of time step  
           Integer   :: rank  
           Real      :: Agrif_x   ! Global x position
           Real      :: Agrif_y   ! Global y position
           Real      :: Agrif_z   ! Global z position
           Real      :: Agrif_dx  ! Global space step in the x direction
           Real      :: Agrif_dy  ! Global space step in the y direction 
           Real      :: Agrif_dz  ! Global space step in the z direction 
           Type(AGRIF_grid), Pointer   :: parent      ! pointer on the parent grid 
           Type(AGRIF_pgrid), Pointer  :: child_grids ! pointer on the child grids 
           Type(AGRIF_PVariable), Dimension(:), Pointer :: bound     
           Type(AGRIF_PVariable), Dimension(:), Pointer 
     &               :: tabvars  ! List of the grid variables   
           Logical   :: fixed     ! fixed or moving grid ?  
           Integer   :: fixedrank ! number of the grid 
           Integer   :: nstepdt   ! number of time steps for 1 time step on the 
                                  ! root coarse grid
           Logical   :: test      ! writing in output file after the first time step
           Integer   :: ngapdt    ! time step for writing in output file
           Integer,Dimension (:),Pointer         :: restore1D   
           Integer,Dimension (:,:),Pointer       :: restore2D   
           Integer,Dimension (:,:,:),Pointer     :: restore3D 
           Integer,Dimension (:,:,:,:),Pointer   :: restore4D
           Integer,Dimension (:,:,:,:,:),Pointer :: restore5D
C
C          Logicals indicating if the current grid has a common border with the 
C          root coarse grid    
           Logical   :: NearRootBorderX        
           Logical   :: DistantRootBorderX    
           Logical   :: NearRootBorderY 
           Logical   :: DistantRootBorderY
           Logical   :: NearRootBorderZ 
           Logical   :: DistantRootBorderZ   
C
           Logical :: oldgrid   
C              
C          Arrays for adaptive grid refinement
           Real, Dimension(:), Pointer        :: valtabpoint1D       
           Real, Dimension(:,:), Pointer      :: valtabpoint2D
           Real, Dimension(:,:,:), Pointer    :: valtabpoint3D  
           Integer, Dimension(:), Pointer     :: tabpoint1D
           Integer, Dimension(:,:), Pointer   :: tabpoint2D
           Integer, Dimension(:,:,:), Pointer :: tabpoint3D            
C     
C          Inclusion of the grid dependent variables  
#include "modgrid_agrif.h"
C  
      End Type AGRIF_grid
C
C
C
C     **************************************************************************  
CCC   Type AGRIF_VARIABLE
C     **************************************************************************  
C     
CCC   Description:
CCC   Data type to characterize a grid variable.   
C
      Type AGRIF_Variable 
C       
           Integer :: pointx  ! index of the first point in the real domain (x direction)
           Integer :: pointy  ! index of the first point in the real domain (y direction)
           Integer :: pointz  ! index of the first point in the real domain (z direction)
           Integer, Dimension(:), Pointer :: posvar ! position of the variable on the cell (1 for the
                                                    ! boarder of the edge, 2 for the center)
           Integer :: nbdim  ! number of dimensions of the grid variable
           Integer,Pointer :: interpIndex ! Indication for the space interpolation (module 
                                          ! Agrif_Boundary)           
           Character*5, Dimension(:),Pointer :: interptab ! Array indicating the type of dimension
                                                          ! (space or not) for each of them 
C   
C          Arrays containing the values of the grid variables 
           Real, Pointer                          :: array0 => NULL() ! Scalar variable   
           Real, Dimension (:), Pointer           :: array1 => NULL() ! 1D variable 
           Real, Dimension (:,:), Pointer         :: array2 => NULL() ! 2D variable 
           Real, Dimension (:,:,:), Pointer       :: array3 => NULL() ! 3D variable 
           Real, Dimension (:,:,:,:), Pointer     :: array4 => NULL() ! 4D variable
           Real, Dimension (:,:,:,:,:), Pointer   :: array5 => NULL() ! 5D variable
           
C          Arrays containing the values of the grid variables (real*8)
           Real*8, Pointer                        :: darray0 => NULL() ! Scalar variable   
           Real*8, Dimension (:), Pointer         :: darray1 => NULL() ! 1D variable 
           Real*8, Dimension (:,:), Pointer       :: darray2 => NULL() ! 2D variable 
           Real*8, Dimension (:,:,:), Pointer     :: darray3 => NULL() ! 3D variable 
           Real*8, Dimension (:,:,:,:), Pointer   :: darray4 => NULL() ! 4D variable
           Real*8, Dimension (:,:,:,:,:), Pointer :: darray5 => NULL() ! 5D variable           
C   
           Logical, Pointer                       :: larray0 => NULL()
           Logical, Dimension (:), Pointer        :: larray1 => NULL()
           Logical, Dimension (:,:), Pointer      :: larray2 => NULL()
           Logical, Dimension (:,:,:), Pointer    :: larray3 => NULL()
           Logical, Dimension (:,:,:,:), Pointer  :: larray4 => NULL()
           Logical, Dimension (:,:,:,:,:), Pointer:: larray5 => NULL()
C    
           Integer, Pointer                       :: iarray0 => NULL()
           Integer, Dimension (:), Pointer        :: iarray1 => NULL()
           Integer, Dimension (:,:), Pointer      :: iarray2 => NULL()
           Integer, Dimension (:,:,:), Pointer    :: iarray3 => NULL()
           Integer, Dimension (:,:,:,:), Pointer  :: iarray4 => NULL()
           Integer, Dimension (:,:,:,:,:), Pointer:: iarray5 => NULL()
C     
           Real, Dimension (:,:), Pointer         :: oldvalues2D ! Array used for the time 
                                                                 ! interpolation
C 
           Integer, Dimension(:), Pointer         :: restore1D => NULL()
           Integer, Dimension(:,:), Pointer       :: restore2D => NULL()
           Integer, Dimension(:,:,:), Pointer     :: restore3D => NULL()
           Integer, Dimension(:,:,:,:), Pointer   :: restore4D => NULL()
           Integer, Dimension(:,:,:,:,:), Pointer :: restore5D => NULL()
C   
           Character*80, Pointer                  :: carray0 => NULL()
           Character*80, Dimension (:), Pointer   :: carray1 => NULL()
           Character*80, Dimension (:,:), Pointer :: carray2 => NULL()
C
           Type(AGRIF_Variable), Pointer :: root_var ! Pointer on the variable of the root grid 
C
           Character*80                           :: namevar  ! Name of the variable          
C   
      End Type AGRIF_Variable   
C
C  
C
C     Values of the grid variables on the parent grid
#include "parents_agrif.h"    
C
C
C  
C     **************************************************************************  
CCC   AGRIF_mygrid : this pointer always points on the root grid of the grid 
CCC   hierarchy 
C     **************************************************************************   
      Type(Agrif_grid), Pointer :: Agrif_mygrid
C
C
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS      
C
C     **************************************************************************  
CCC   AGRIF_oldmygrid : Pointer used in the Agrif_regrid subroutine (Agrif_Util 
CCC   module). It contains the safeguard of the grid hierarchy.  
C     **************************************************************************
      Type(Agrif_pgrid), Pointer :: Agrif_oldmygrid
C
#endif            
C
C
C
C     **************************************************************************  
CCC   AGRIF_CURGRID : pointer to the current grid (the link is done by using the
CCC   Agrif_Instance procedure (module Agrif_Init)) 
C     ************************************************************************** 
      Type(AGRIF_grid), Pointer :: AGRIF_CURGRID
C
C
C
C     **************************************************************************  
CCC   AGRIF_saveCURGRID : Pointer used in the Agrif_ChildGrid_to_ParentGrid and 
CCC   Agrif_ParentGrid_to_ChildGrid subroutines (Agrif_CurgridFunctions module).
CCC   It contains the safeguard of the current grid hierarchy.
C     ************************************************************************** 
      Type(AGRIF_grid), Pointer :: AGRIF_saveCURGRID
C
C
C
C     **************************************************************************   
CCC   AGRIF_nbfixedgrids : number of fixed grids in the grid hierarchy 
CCC   (calculated in the Agrif_Create_Fixed_Grids subroutine of the 
CCC   Agrif_Clustering module)  
C     **************************************************************************
      Integer :: AGRIF_nbfixedgrids
C
C
C
C     **************************************************************************
CCC   Different parameters
C     **************************************************************************
C
C     Parameters for the interpolation of the child grids
      Integer,Parameter :: AGRIF_linear=1      ! linear interpolation
      Integer,Parameter :: AGRIF_lagrange=2    ! lagrange interpolation
      Integer,Parameter :: AGRIF_spline=3      ! spline interpolation
      Integer,Parameter :: AGRIF_user_interp=4 ! user s interpolation
C     
C     Parameters for the update of the parent grids      
      Integer,Parameter :: AGRIF_Update_Copy=1           ! copy
      Integer,Parameter :: AGRIF_Update_Average=2        ! average
      Integer,Parameter :: AGRIF_Update_Full_Weighting=3 ! full-weighting 
C
C     Logical to use special values on the parent grid
      Logical :: Agrif_UseSpecialValue
C
C     Logical to use special values on the current grid        
      Logical :: Agrif_UseSpecialValueFineGrid
C      
C     Special values on the parent grid
      Real :: Agrif_SpecialValue
C
C     Special values on the current grid
      Real :: Agrif_SpecialValueFineGrid 
C
#ifdef AGRIF_MPI
C
C     Number of processors
      Integer :: Agrif_Nbprocs
C
C     Rank of the current processor 
      Integer :: Agrif_ProcRank
C
C     Group associated to MPI_COMM_WORLD
      Integer :: Agrif_Group      
C
#endif
C
C
C     
      End Module AGRIF_Types  
