!
! $Id: modutil.F,v 1.2 2003/12/17 13:56:11 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Util 
C 
#include "keys_agrif.h"
C
      Module Agrif_Util 
C
C
CCC   Description:   
CCC   This module contains the two procedures called in the main program : 
CCC   Agrif_Init_Grids allowing the initialization of the root coarse grid, and
CCC   Agrif_Step allowing the creation of the grid hierarchy and the management
CCC   of the time integration. 
C
C     Modules used:
C
      Use Agrif_Clustering    
      Use Agrif_Save         
C      
#include "declare_indices_points_agrif.h"
C
      Contains
C     Definition of procedures contained in this module.
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Step
C     ************************************************************************** 
C 
      Subroutine Agrif_Step() 
C
CCC   Description:
CCC   Subroutine to create the grid hierarchy and to manage the time integration 
CCC   procedure. It is called in the main program.
C
CC    Method:
CC    Call on subroutines Agrif_Regrid and Agrif_Integrate.
C
C     Declarations:
C
      Implicit none
#ifdef AGRIF_MPI
C
      Integer      :: code
      Include "mpif.h"
C
C
      If (Agrif_mygrid % ngridstep == 0) Then
C                                
          Call MPI_COMM_SIZE(MPI_COMM_WORLD,Agrif_Nbprocs,code)
C
          Call MPI_COMM_RANK(MPI_COMM_WORLD,AGRIF_ProcRank,code)
C
          Call MPI_COMM_GROUP(MPI_COMM_WORLD,Agrif_Group,code)
C      
      End If
C
#endif
C
C    
C     Creation and initialization of the grid hierarchy 
C
      Call Agrif_Back_Instance   
     &         (Agrif_mygrid)
C  
#ifdef AGRIF_USE_ONLY_FIXED_GRIDS
C
      If (AGRIF_mygrid % ngridstep == 0) then
C
          Call Agrif_Regrid
C            
      End if
C
#else
C    
      If (mod(AGRIF_mygrid % ngridstep, Agrif_Regridding) == 0) then
C
          Call Agrif_Regrid
C            
      End if            
C
#endif
C
C     Time integration of the grid hierarchy
      Call Agrif_Integrate       
     &         (AGRIF_mygrid)
C    
      Call Agrif_Instance           
     &          (AGRIF_mygrid)
C
      Return
C             
      End Subroutine Agrif_Step      
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Regrid
C     **************************************************************************
C
      Subroutine Agrif_Regrid
C
CCC   Description:
CCC   Subroutine to create the grid hierarchy from fixed grids and adaptive mesh refinement.
C
CC    Method:        
C
C     Declarations:
C
      Implicit none
C
C     Local variables      
      Integer :: i,j,k
      Type(Agrif_pgrid),pointer :: oldchildgrids      
      Type(Agrif_Rectangle), Pointer     :: coarsegrid_fixed
      Type(Agrif_Rectangle), Pointer     :: coarsegrid_moving  
      Integer :: nunit
      Logical :: BEXIST
C 
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C
C     Detection of areas to be refined  
      Call Agrif_detect_all(Agrif_mygrid)
C
#endif      
C
C
      Allocate(coarsegrid_fixed)
C 
      Allocate(coarsegrid_moving)
C
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C
C     Clustering      
      call Agrif_Cluster_All(Agrif_mygrid,coarsegrid_moving)
C
#endif
C
C
#if defined AGRIF_USE_FIXED_GRIDS || defined AGRIF_USE_ONLY_FIXED_GRIDS
C
      If (Agrif_mygrid % ngridstep == 0) Then
C
C         Opening of the Agrif_FixedGrids.in file
          nunit = Agrif_Get_Unit()
C          
          open(nunit,file='AGRIF_FixedGrids.in',form='formatted',
     &         status="old",ERR=99)     
C        
          j = 1
C      
C         Creation of the grid hierarchy from the Agrif_FixedGrids.in file 
          coarsegrid_fixed % imin = 1
          coarsegrid_fixed % imax = Agrif_mygrid % nbx + 1
C
          If (Agrif_Probdim >= 2) Then
C
              coarsegrid_fixed % jmin = 1
              coarsegrid_fixed % jmax = Agrif_mygrid % nby + 1 
C
          End If
C
          If (Agrif_Probdim == 3) Then
C
              coarsegrid_fixed % kmin = 1
              coarsegrid_fixed % kmax = Agrif_mygrid % nbz + 1 
C
          End If
C    
          Call Agrif_Read_Fixed_Grids
     &         (coarsegrid_fixed,j,nunit)     
C      
C         Closing of the Agrif_FixedGrids.in file 
          close(nunit)
C
          Nullify(Agrif_oldmygrid)
C     
          Nullify(Agrif_mygrid  % child_grids)
C        
C         Creation of the grid hierarchy from coarsegrid_fixed
          Call Agrif_Create_Grids
     &         (Agrif_mygrid,coarsegrid_fixed)
C 
        Else        
C        
          Agrif_oldmygrid => Agrif_mygrid % child_grids
C      
      End If
C
#else
C 
      Agrif_oldmygrid => Agrif_mygrid % child_grids
C 
      Nullify(Agrif_mygrid  % child_grids)                          
C
#endif
C
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C      
      Call Agrif_Free_data_before_All(Agrif_oldmygrid)
C 
C     Creation of the grid hierarchy from coarsegrid_moving    
      Call Agrif_Create_Grids
     &     (Agrif_mygrid,coarsegrid_moving)  
C
#endif
C     
C
C     Initialization of the grid hierarchy by copy or interpolation
      Call Agrif_Init_Hierarchy(Agrif_mygrid)
C
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C
      Call Agrif_Free_data_after_All(Agrif_oldmygrid)
C
#endif     
C    
C
      Deallocate(coarsegrid_fixed)
      Deallocate(coarsegrid_moving)
C      
      Return
C
   99 INQUIRE(FILE='AGRIF_FixedGrids.in',EXIST=BEXIST)
C
      If (.not. BEXIST) Then
C
          print*,'ERROR : File AGRIF_FixedGrids.in not found.'
          STOP 
C
        Else
C
          print*,'Error opening file AGRIF_FixedGrids.in'
          STOP         
C
      End If     
C
C      
      End Subroutine Agrif_Regrid
C
C
C
#if ! defined AGRIF_USE_ONLY_FIXED_GRIDS
C
C
C     **************************************************************************
CCC   Subroutine Agrif_detect_All
C     **************************************************************************      
C
      Recursive Subroutine Agrif_detect_all(g)
C
CCC   Description:
CCC   Subroutine to detect areas to be refined.
C
CC    Method:        
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(AGRIF_grid),pointer   :: g        ! Pointer on the current grid
C      
C     Local variables
      Type(AGRIF_pgrid),pointer  :: parcours ! Pointer for the recursive 
                                             ! procedure      
      Integer, Dimension(:), Allocatable :: tabpoint1D
      Integer, Dimension(:,:), Allocatable :: tabpoint2D
      Integer, Dimension(:,:,:), Allocatable :: tabpoint3D       
      Integer :: sizex,sizey,sizez
      Integer :: i,j
      Real :: g_eps     
C
C    
      parcours => g % child_grids
C  
C     To be positioned on the finer grids of the grid hierarchy          
      Do while (associated(parcours))
C       
        Call AGRIF_detect_all
     &           (parcours % gr)  
C  
        parcours => parcours % next             
C
      End Do
C        
      g_eps=g%Agrif_dx
C          
      If (Agrif_Probdim.GE.2) Then
C          
          g_eps=min(g_eps,g%Agrif_dy)
C            
          If (Agrif_Probdim.GE.3) Then
C              
              g_eps=min(g_eps,g%Agrif_dz)      
C            
          End If
C          
      End If
C
      g_eps = g_eps/100.
C          
#ifdef Agrif_Probdim_1D
C 
      g%tabpoint1D=0
C
#endif
C
#ifdef Agrif_Probdim_2D
C 
      g%tabpoint2D=0
C
#endif
C
#ifdef Agrif_Probdim_3D
C
      g%tabpoint3D=0
C
#endif
C
      If (g%Agrif_dx/Agrif_coeffrefx.LT.(Agrif_mindx-g_eps))
     &          Return
C      
      If (Agrif_Probdim.GE.2) Then
C        
          If (g%Agrif_dy/Agrif_coeffrefy.LT.(Agrif_mindy-g_eps))
     &       Return       
C          
          If (Agrif_Probdim.GE.3) Then
C            
              If (g%Agrif_dz/Agrif_coeffrefz.LT.
     &                      (Agrif_mindz-g_eps)) Return        
C          
           End If         
C      
      End If 
C                                       
      Call Agrif_instance(g)
C  
C
C     Detection (Agrif_detect is a user s routine)
C      
#ifdef Agrif_Probdim_1D
C      
          sizex = g%nbx + 1
C            
          Call Agrif_detect(g%tabpoint1D,sizex)
C
#endif 
C        
#ifdef Agrif_Probdim_2D
C        
          sizex = g%nbx + 1
C        
          sizey = g%nby + 1
C          
          Call Agrif_detect(g%tabpoint2D,sizex,sizey)
C
#endif
C         
#ifdef Agrif_Probdim_3D
C      
          sizex = g%nbx + 1
C        
          sizey = g%nby + 1        
C        
          sizez = g%nbz + 1        
C          
          Call Agrif_detect(g%tabpoint3D,sizex,sizey,sizez)
C        
#endif
C
C
C     Addition of the areas detected on the child grids
C      
      parcours => g % child_grids 
C           
      Do while (associated(parcours))
C       
        Call AGRIF_Add_detected_areas
     &           (g,parcours % gr)
C  
        parcours => parcours % next 
C                    
      End Do
C     
      Return      
C
C
      End Subroutine Agrif_detect_all
C      
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Add_detected_areas
C     **************************************************************************
C      
      Subroutine Agrif_Add_detected_areas(parentgrid,childgrid)
C
CCC   Description:
CCC   Subroutine to add on the parent grid the areas detected on its child grids.
C
CC    Method:        
C
C     Declarations:
C
      Implicit none
C 
      Type(AGRIF_grid),pointer   :: parentgrid,childgrid
C      
      Integer :: i,j,k
C
C      
      If (Agrif_Probdim.EQ.1) Then
C      
          Do i = 1,childgrid%nbx+1
C
            If (childgrid%tabpoint1D(i).EQ.1) Then
C       
                parentgrid%tabpoint1D(childgrid%ix+
     &                                (i-1)/Agrif_Coeffrefx) = 1
C      
            End If
C          
          End Do
C      
        Else If (Agrif_Probdim.EQ.2) Then
C      
          Do j=1,childgrid%nby+1
C      
            Do i=1,childgrid%nbx+1
C      
              If (childgrid%tabpoint2D(i,j).EQ.1) Then
C       
                  parentgrid%tabpoint2D(
     &                       childgrid%ix+(i-1)/Agrif_Coeffrefx,
     &                       childgrid%iy+(j-1)/Agrif_Coeffrefy) = 1
C      
              End If
C            
            End Do
C          
          End Do
C            
        Else If (Agrif_Probdim.EQ.3) Then
C      
          Do k=1,childgrid%nbz+1
C      
            Do j=1,childgrid%nby+1
C      
              Do i=1,childgrid%nbx+1
C      
                If (childgrid%tabpoint3D(i,j,k).EQ.1) Then
C       
                    parentgrid%tabpoint3D(
     &                         childgrid%ix+(i-1)/Agrif_Coeffrefx,
     &                         childgrid%iy+(j-1)/Agrif_Coeffrefy,
     &                         childgrid%iz+(k-1)/Agrif_Coeffrefz) = 1
C      
                End If
C      
              End Do
C      
            End Do      
C      
          End Do
C      
      End If
C
C      
      Return
C
C      
      End Subroutine Agrif_Add_detected_areas   
C      
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Free_data_before_All
C     **************************************************************************      
C
      Recursive Subroutine Agrif_Free_data_before_All(g)
C
CCC   Description:
C
CC    Method:        
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(AGRIF_pgrid),pointer   :: g        ! Pointer on the current grid
C
C     Local pointer
      Type(AGRIF_pgrid),pointer  :: parcours ! Pointer for the recursive 
                                             ! procedure      
C
C
      parcours => g        
C      
      Do while (associated(parcours))     
C
        If (.not. parcours%gr%fixed) Then
C
            Call Agrif_Free_data_before(parcours%gr)
C
            parcours % gr % oldgrid = .TRUE.        
C
        End If
C      
        Call AGRIF_Free_data_before_all
     &           (parcours % gr % child_grids)
C  
        parcours => parcours % next             
C      
      End Do
C     
      Return      
C
C
      End Subroutine Agrif_Free_data_before_All
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Free_data_after_All
C     **************************************************************************      
C
      Recursive Subroutine Agrif_Free_data_after_All(g)
C
CCC   Description:
C
CC    Method:        
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(AGRIF_pgrid),pointer   :: g        ! Pointer on the current grid
C
C     Local pointers
      Type(AGRIF_pgrid),pointer  :: parcours ! Pointer for the recursive 
                                             ! procedure  
      Type(AGRIF_pgrid),pointer  :: preparcours   
      Type(AGRIF_pgrid),pointer  :: preparcoursini 
C
C
      Allocate(preparcours)
C 
      preparcoursini => preparcours
C 
      Nullify(preparcours % gr)
C 
      preparcours % next => g
C
      parcours => g
C      
      Do while (associated(parcours))
C
      If ((.not. parcours% gr% fixed).And.(parcours% gr% oldgrid)) Then
C      
          Call Agrif_Free_data_after(parcours%gr)
C
      End If
C      
      Call AGRIF_Free_data_after_all
     &     (parcours % gr % child_grids)
C
      If (parcours % gr % oldgrid) Then
C      
          Deallocate(parcours % gr)
C              
          preparcours % next => parcours % next
C        
          Deallocate(parcours)
C        
          parcours => preparcours % next
C        
        Else
C      
          preparcours => preparcours % next
C  
          parcours => parcours % next 
C              
      End If       
                  
C      
      End Do
C
      Deallocate(preparcoursini)
C     
      Return      
C
C
      End Subroutine Agrif_Free_data_after_All
C
C
#endif      
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Integrate
C     **************************************************************************
C
      Recursive Subroutine Agrif_Integrate(g)         
C
CCC   Description:
CCC   Subroutine to manage the time integration of the grid hierarchy.
C
CC    Method:
CC    Recursive subroutine and call on subroutines Agrif_Instance & Agrif_Step.        
C
C     Declarations:
C
      Implicit none
C      
C     Pointer argument    
      Type(AGRIF_grid),pointer   :: g        ! Pointer on the current grid
C      
C     Local pointer
      Type(AGRIF_pgrid),pointer  :: parcours ! Pointer for the recursive 
                                             ! procedure
C
C     Local scalars
      Integer                    :: nbt      ! Number of time steps of the 
                                             ! current grid
      Integer                    :: k        
C  
C  
C     Instanciation of the variables of the current grid      
      Call Agrif_Instance
     &     (g)
C       
C     One step on the current grid      
      Call Step
     &     ()
C
      Call Agrif_Back_Instance
     &     (g)      
C
C     Number of time steps on the current grid      
      g%ngridstep = g % ngridstep + 1
C      
      parcours => g % child_grids
C    
C     Recursive procedure for the time integration of the grid hierarchy      
      Do while (associated(parcours))
C
C       Instanciation of the variables of the current grid            
        Call Agrif_Instance
     &           (parcours % gr)
C       
C       Number of time steps
        nbt = Agrif_Rhot()
C       
        Do k = 1,nbt
C       
          Call AGRIF_Integrate
     &             (parcours % gr)
C       
        End do     
C  
        parcours => parcours % next
C     
      End do
C
C      
      End Subroutine Agrif_Integrate 
C
C
C   
C     **************************************************************************
CCC   Subroutine Agrif_Init_Grids
C     **************************************************************************      
C      
      Subroutine Agrif_Init_Grids
C
CCC   Description:
CCC   Subroutine to initialize the root coarse grid pointed by Agrif_mygrid. 
CCC   It is called in the main program.
C
C     Declarations:
C
c      Implicit none
C
#include "include_agrif.h"
C
C
      Agrif_UseSpecialValue = .FALSE.
      Agrif_UseSpecialValueFineGrid = .FALSE.
C      
C     Allocation of Agrif_mygrid
      allocate(Agrif_mygrid)
C      
C     Space and time refinement factors are set to 1 on the root grid
C
      AGRIF_mygrid % spacerefx = 1
      AGRIF_mygrid % timerefx = 1
C
      If (Agrif_Probdim .GE. 2) Then
C  
          AGRIF_mygrid % spacerefy = 1
          AGRIF_mygrid % timerefy = 1    
C
      End If
C
      If (Agrif_Probdim .EQ. 3) Then
C                      
          AGRIF_mygrid % spacerefz = 1
          AGRIF_mygrid % timerefz = 1
C
      End If        
C      
C     Initialization of the number of time steps    
      AGRIF_mygrid % ngridstep = 0
C
C     No parent grid for the root coarse grid
      Nullify(AGRIF_mygrid % parent)
C      
C     Parameters for the output writing     
      AGRIF_mygrid % nstepdt = 1    
      AGRIF_mygrid % test = .FALSE. ! Output saveguard after the first time step
      AGRIF_mygrid % ngapdt = 0
C    
C     Initialization of the minimum positions, global abscissa and space steps    
      AGRIF_mygrid % ix = 1            
      Agrif_mygrid % Agrif_x = 0.
      Agrif_mygrid % Agrif_dx = 1. 
C    
      If (Agrif_Probdim .GE. 2) Then
C        
          AGRIF_mygrid % iy = 1
          Agrif_mygrid % Agrif_y = 0.
          Agrif_mygrid % Agrif_dy = 1.
C
      End If         
C  
      If (Agrif_Probdim .EQ. 3) Then
C         
          AGRIF_mygrid % iz = 1
          Agrif_mygrid % Agrif_z = 0.
          Agrif_mygrid % Agrif_dz = 1.        
C
      End If                
C
C     Borders of the root coarse grid 
      AGRIF_mygrid % NearRootBorderX = .true.  
      AGRIF_mygrid % DistantRootBorderX = .true.      
C
      If (Agrif_Probdim .GE. 2) Then
C            
          AGRIF_mygrid % NearRootBorderY = .true.  
          AGRIF_mygrid % DistantRootBorderY = .true.
C
      End If
C
      If (Agrif_Probdim .EQ. 3) Then
C                      
          AGRIF_mygrid % NearRootBorderZ = .true.  
          AGRIF_mygrid % DistantRootBorderZ = .true.
C
      End If        
C  
C     The root coarse grid is a fixed grid
      AGRIF_mygrid % fixed = .TRUE.
C      
C     Number of the grid pointed by Agrif_mygrid (root coarse grid)
      AGRIF_mygrid % rank = 1      
C 
C     Number of the root grid as a fixed grid   
      AGRIF_mygrid % fixedrank = 0  
C 
C     Initialization of some fields of the root grid variables     
      Call Agrif_Create_Var
     &     (AGRIF_mygrid)
C      
C     Initialization of the other fields of the root grid variables (number of 
C     cells, positions, number and type of its dimensions, ...)  
#include "initialisations_agrif.h"    
C      
      Call Agrif_Nullify            
     &     (AGRIF_mygrid)
C
      Call Agrif_Base_Instance
     &     (AGRIF_mygrid)        
C             
C     Allocation of the array containing the values of the grid variables
      Call Agrif_Allocation         
     &     (AGRIF_mygrid)
C        
      nullify(AGRIF_mygrid % child_grids)  
C 
C     Total number of fixed grids   
      AGRIF_nbfixedgrids = 0         
C      
      Call Agrif_Instance           
     &     (AGRIF_mygrid)
C
C        
      End Subroutine AGRIF_Init_Grids  
C
C      
C      
      End module Agrif_Util 
